#include "cfgdialog.h"
#include "names.h"

extern CFGTree* tree;
extern DevManager devmanager;

static const char* YES[] = {"Yes","No"};
static int Nyes = 2;
static const char* JCOLORS[] = { "black","white","darkGray","gray",
	"lightGray","red","green","blue","cyan","magenta","yellow", 
	"darkRed","darkGreen","darkBlue","darkCyan","darkMagenta", 
	"darkYellow" };
static int Njcolors = 17;
static const char* REFER[] = {"NightMean","LastMean","SeasonMean"};
static int Nrefer = 3;

QValidator::State FloatValidator::validate( QString& text, int& pos ) {
	debug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		debug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		debug("  >= text.length(), valid");
		return Valid;
	}
/*
	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.')
			&& (text[pos-1] != '-') && (text[pos-1] != '+') ) {
		debug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}
*/
	if( text[pos-1] == '.' ) {
		if( text.contains('.') > 1 ) return Invalid;
	}
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( text.length() < 1 ) {
		debug(" Less than five digits; not acceptable" );
		return Valid;
	}
	return Valid;
//	return Acceptable;
}

QValidator::State IntValidator::validate( QString& text, int& pos ) {
	debug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		debug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		debug("  >= text.length(), valid");
		return Valid;
	}
/*
	if( (!isdigit(text[pos-1])) && (text[pos-1] != '-')
			&& (text[pos-1] != '+') ) {
		debug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}
*/
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	return Valid;
}
/*
QValidator::State TimeValidator::validate( QString& text, int& pos ) {
	debug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		debug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		debug("  >= text.length(), valid");
		return Valid;
	}

	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.') && (text[pos-1] != ' ') ) {
		debug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	debug( "b1=%d len=%d",b1,text.length() );	
	if( text.length() > b1 ) b2 = text.find(' ',b1+1);
	debug("b2=%d",b2);
	if( text[pos-1] == ' ' ) {
		if( (b1 != 1) && (b1 != 2) ) return Invalid;
		if( b2 != -1 ) {
			if( text.find(' ',b2+1) != -1 ) return Invalid;
		}
	}
//	if( atoi(&text[0]) >= 24 ) return Invalid;
	if( (b1 != -1) && ((text.length()-1) > b1) )
		if( atoi(&text[b1+1]) > 59 ) return Invalid;
	if( (b2 != -1) && ((text.length()-1) > b2) )
		if( atof(&text[b2+1]) >= 60 ) return Invalid;

	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( (text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2)) ) return Invalid;
	if( (text[pos-1] == '.') && (text.contains('.') > 1) ) return Invalid;

	debug( "Input Valid" );
	return Valid;
}
*/
/*
QValidator::State DegreeValidator::validate( QString& text, int& pos ) {
	debug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		debug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		debug("  >= text.length(), valid");
		return Valid;
	}

	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.') && 
			(text[pos-1] != ' ') && (text[pos-1] != '-') &&
										(text[pos-1] != '+') ) {
		debug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	debug( "b1=%d len=%d",b1,text.length() );	
	if( text.length() > b1 ) b2 = text.find(' ',b1+1);
	debug("b2=%d",b2);
	if( text[pos-1] == ' ' ) {
		if( (b1 != 1) && (b1 != 2) && (b1 != 3) ) return Invalid;
		if( b2 != -1 ) {
			if( text.find(' ',b2+1) != -1 ) return Invalid;
		}
	}
//	debug( "%d",atoi(&text[0]) );
//	if( abs(atoi(&text[0])) > 90 ) return Invalid;
	if( (b1 != -1) && ((text.length()-1) > b1) ){
		if( atoi(&text[b1+1]) > 59 ) return Invalid;
		if( (abs(atoi(&text[0])) == 90) && (atoi(&text[b1+1]) > 0) ) return Invalid;
	}
	if( (b2 != -1) && ((text.length()-1) > b2) ){
		if( atof(&text[b2+1]) >= 60 ) return Invalid;
		if( (abs(atoi(&text[0])) == 90) && (atof(&text[b2+1]) > 0) ) return Invalid;
	}
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( (text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2)) ) return Invalid;
	if( (text[pos-1] == '.') && (text.contains('.') > 1) ) return Invalid;

	debug( "Input Valid" );
	return Valid;
}
*/

SubItem::SubItem( CFGBranch* _branch, CFGBranch* _bak_branch ) {
	_enable = TRUE;
	branch = _branch;
	bak_branch = _bak_branch;
//	bak_branch->copyBranch( bak_branch, branch );
}

SubItem::~SubItem() {
}

void SubItem::setEnabled( void ) {
	_enable = TRUE;
}

void SubItem::setDisabled( void ) {
	_enable = FALSE;
}

void SubItem::searchChangeDisabledLeafs( void ) {
	CFGLeaf *leaf;
	QString text;

	branch->toFirst();
	while( branch->getCurrentLink() ) {
		leaf =  branch->getCurrentLink()->getLeaf();
		if( leaf && !(leaf->isChangeEnabled()) ) {
			text.sprintf( "Field: %s Active after initialization",leaf->getKey() );
			QMessageBox::warning( 0,"cfg",text.data() );
			debug( "leaf: %s - change disabled!!!", leaf->getKey() );
		}
		branch->toNext();
	}
}

void SubItem::slotExecBranchDialog( void ) {

	bak_branch->copyBranch( bak_branch,branch );
	BranchDialog* dialog = new BranchDialog( branch, bak_branch, _enable );
	CHECK_PTR(dialog);
	connect( dialog,SIGNAL(signalShowStatus(const char*,int)),
				this,SIGNAL(signalShowStatus(const char*,int)) );

	int ret = dialog->exec();

	if( ret == QDialog::Accepted ) {
		searchChangeDisabledLeafs();
// lll
		branch->copyBranch( branch,bak_branch );
		tree->setChanged();
		emit signalShowStatus( "Accepted",2000 );
//		emit signalApply();
		tree->emitSignalChanged();
	} else {
//		branch->copyBranch( bak_branch,branch );
//		dialog->slotRestored(); // if "Cancel"
	}
	delete dialog;
}

JKQLineEdit::JKQLineEdit( QWidget* parent, CFGLeaf* _leaf, bool _enable ) 
						: QLineEdit( parent ) {
	leaf = _leaf;
	setEnabled( _enable );
}

JKQLineEdit::~JKQLineEdit() {
}

void JKQLineEdit::slotTextChanged( const QString& _text ) {
	debug( "textChanged: %s", _text.data() );
	leaf->replaceData( _text );
}

void JKQLineEdit::enterEvent( QEvent* e ) {
	emit signalShowStatus( leaf->getHelp()->data(),-1 );
	QLineEdit::enterEvent( e );
}

void JKQLineEdit::leaveEvent( QEvent* e ) {
	emit signalShowStatus( "", 0 );
	QLineEdit::leaveEvent( e );
}


JKQComboBox::JKQComboBox( bool rw, QWidget* parent,CFGLeaf* _leaf,
				bool _enable ) : QComboBox( rw,parent ) {
	leaf = _leaf;
	setEnabled( _enable );
}

JKQComboBox::~JKQComboBox() {
}

void JKQComboBox::setCurrentText( const char* _text ) {
	int i;

	for( i=0;i<count() ;i++ ) {
		if( !strcasecmp(text(i),_text) ) {
			setCurrentItem( i );
		}
	}
}

void JKQComboBox::slotTextChanged( const QString& _text ) {
	debug( "textChanged: %s", _text.data() );
	leaf->replaceData( _text );
}

void JKQComboBox::enterEvent( QEvent* e ) {
	emit signalShowStatus( leaf->getHelp()->data(),-1 );
	QComboBox::enterEvent( e );
}

void JKQComboBox::leaveEvent( QEvent* e ) {
	emit signalShowStatus( "", 0 );
	QComboBox::leaveEvent( e );
}

BranchDialog::BranchDialog( CFGBranch* _branch, CFGBranch* _bak_branch, bool _enable ) :
		QDialog( 0,"BranchDialog",TRUE /*modal*/ ) {
	bak_branch = _bak_branch;
	branch = _branch;
	int layout_row;
	int num_layout_cols=2;
	int w_height = 0;

	setCaption( "Section Dialog" );

	okbutton = new QPushButton( "Apply",this );
	CHECK_PTR(okbutton);
	okbutton->setAutoDefault( TRUE );
	okbutton->setEnabled( _enable );
	connect( okbutton, SIGNAL( clicked() ), this, SLOT( accept() ) );

	restorebutton = new QPushButton( "Restore",this );
	CHECK_PTR(restorebutton);
	restorebutton->setAutoDefault( TRUE );
	restorebutton->setEnabled( _enable );
	connect( restorebutton, SIGNAL( clicked() ), this, SLOT( slotRestored() ) );

	cancelbutton = new QPushButton( "Cancel",this );
	CHECK_PTR(cancelbutton);
	cancelbutton->setAutoDefault( TRUE );
	connect( cancelbutton, SIGNAL( clicked() ), this, SLOT( reject() ) );

	int border = fontMetrics().height();
	layout = new QGridLayout( this,1,num_layout_cols,border );
	CHECK_PTR(layout);
	layout_row = 1;

	int labelwidth;
	int editwidth;
	int maxlabelwidth = okbutton->sizeHint().width();
	int maxeditwidth = 	cancelbutton->sizeHint().width();
	branch->toFirst();
	while( branch->getCurrentLink() ) {
		if( branch->getCurrentLink()->getId() == 'l' ) {
			labelwidth = fontMetrics().width( branch->getCurrentLink()->getLeafKey() );
			editwidth = fontMetrics().width( branch->getCurrentLink()->getLeafData() );
			if( labelwidth > maxlabelwidth ) maxlabelwidth = labelwidth;
			if( editwidth > maxeditwidth ) maxeditwidth = editwidth;
		}
		branch->toNext();
	}
	maxeditwidth += fontMetrics().width( "ww" ); //  ...
	if( maxeditwidth+maxlabelwidth < fontMetrics().width(branch->getName()) ) {
		int dwidth = ( fontMetrics().width(branch->getName())-
									(maxeditwidth+maxlabelwidth) )/2; 
		maxeditwidth += dwidth;
		maxlabelwidth += dwidth;
	}

	int labelheight =  fontMetrics().height()*5/4 ;

	n_edit = 0;
	n_combo = 0;
	n_label = 0;
	edit.resize(10);
	combo.resize(10);
	label.resize(10);
	label[n_label] = new QLabel( branch->getName(),this );
	CHECK_PTR(label[n_label]);
	label[n_label]->setFixedSize( label[n_label]->sizeHint() );
	layout->addMultiCellWidget( label[n_label],0,0,0,1,AlignHCenter );	
	n_label++;

//	floatvalidator = new FloatValidator(this);
//	CHECK_PTR(floatvalidator);
//	timevalidator = new TimeValidator(this);
//	CHECK_PTR(timevalidator);
//	degreevalidator = new DegreeValidator(this);
//	CHECK_PTR(degreevalidator);

	branch->toFirst();
	while( branch->getCurrentLink() ) {
		if( branch->getCurrentLink()->getId() == 'l' ) {
			layout->expand( layout_row+1,num_layout_cols );

			if( (int)label.size() < n_label+1 ) label.resize(label.size()+10);
			label[n_label] = new QLabel( branch->getCurrentLink()->getLeafKey(),this );
			CHECK_PTR(label[n_label]);
			label[n_label]->setFixedSize( maxlabelwidth,labelheight );
			layout->addWidget( label[n_label],layout_row,0 );
			n_label++;

			const char* validat = branch->getCurrentLink()->getLeaf()->getValidatorType();
			if( !validat )	validat = "empty";
			if( !strcasecmp( validat,"yes" ) ) {
				if( (int)combo.size() < n_combo+1 ) combo.resize(combo.size()+10);
				combo[n_combo] = new JKQComboBox( FALSE,this,branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(combo[n_combo]);
				combo[n_combo]->insertStrList( YES,Nyes );
				combo[n_combo]->setFixedSize( maxeditwidth,labelheight );
				combo[n_combo]->setCurrentText( branch->getCurrentLink()->getLeafData() );
				connect( combo[n_combo],SIGNAL(activated(const QString&)),						
							combo[n_combo],SLOT(slotTextChanged(const QString&)) );
				connect( combo[n_combo],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( combo[n_combo],layout_row,1 );
				n_combo++;
			} else if( !strcasecmp( validat,"color" ) ) {
				if( (int)combo.size() < n_combo+1 ) combo.resize(combo.size()+10);
				combo[n_combo] = new JKQComboBox( FALSE,this,branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(combo[n_combo]);
				combo[n_combo]->insertStrList( JCOLORS,Njcolors );
				combo[n_combo]->setFixedSize( maxeditwidth,labelheight );
				combo[n_combo]->setCurrentText( branch->getCurrentLink()->getLeafData() );
				connect( combo[n_combo],SIGNAL(activated(const QString&)),
					combo[n_combo],SLOT(slotTextChanged(const QString&)) );
				connect( combo[n_combo],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( combo[n_combo],layout_row,1 );
				n_combo++;
			} else if( !strcasecmp( validat,"refer" ) ) {
				if( (int)combo.size() < n_combo+1 ) combo.resize(combo.size()+10);
				combo[n_combo] = new JKQComboBox( FALSE,this,branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(combo[n_combo]);
				combo[n_combo]->insertStrList( REFER,Nrefer );
				combo[n_combo]->setFixedSize( maxeditwidth,labelheight );
				combo[n_combo]->setCurrentText( branch->getCurrentLink()->getLeafData() );
				connect( combo[n_combo],SIGNAL(activated(const QString&)),
					combo[n_combo],SLOT(slotTextChanged(const QString&)) );
				connect( combo[n_combo],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( combo[n_combo],layout_row,1 );
				n_combo++;
			} else {	
				if( (int)edit.size() < n_edit+1 ) edit.resize(edit.size()+10);
				edit[n_edit] = new JKQLineEdit( this,branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(edit[n_edit]);
				edit[n_edit]->setFixedSize( maxeditwidth,labelheight );
				edit[n_edit]->setText( branch->getCurrentLink()->getLeafData() );

				if( !strcasecmp( validat,"float" ) ) {
//					edit[n_edit]->setValidator( floatvalidator );
				} else if( !strcasecmp( validat,"time" ) ) {
//					edit[n_edit]->setValidator( timevalidator );
				} else if ( !strcasecmp( validat,"degree") ) {
//					edit[n_edit]->setValidator( degreevalidator );
				}
				connect( edit[n_edit],SIGNAL( textChanged(const QString&) ),
						edit[n_edit],SLOT( slotTextChanged(const QString&) ) );
				connect( edit[n_edit],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( edit[n_edit],layout_row,1 );
				n_edit++;
			}
			layout_row++;		

		}
		branch->toNext();
	}

//	n_label--; n_edit--;n_combo--;
	layout->expand( layout_row+1,num_layout_cols );

	okbutton->setMinimumSize( okbutton->sizeHint() );
	okbutton->setMaximumSize( maxlabelwidth,okbutton->sizeHint().height() );
	layout->addWidget( okbutton,layout_row,0 );	

	cancelbutton->setMinimumSize( cancelbutton->sizeHint() );
	cancelbutton->setMaximumSize( maxeditwidth,cancelbutton->sizeHint().height() );
	layout->addWidget( cancelbutton,layout_row,1 );	

	layout_row++;
	layout->expand( layout_row+1,num_layout_cols );
	restorebutton->setMinimumSize( restorebutton->sizeHint() );
	restorebutton->setMaximumSize( maxlabelwidth,restorebutton->sizeHint().height() );
	layout->addMultiCellWidget( restorebutton,layout_row,layout_row,0,1,AlignCenter );	

	w_height = (labelheight+border) * ( layout_row+1 ) + border; 
	move( ( qApp->desktop()->width()-width() )/2,
					( qApp->desktop()->height()-w_height )/2 );
	emit signalShowStatus( "Section dialog",4000 );
}

void BranchDialog::slotRestored() {
	branch->copyBranch( bak_branch,branch );
	int i;
	for( i=0;i<n_edit;i++ ) {
		edit[i]->setText( edit[i]->getLeafText() );
	}
	for( i=0;i<n_combo;i++ ) {
		combo[i]->setCurrentText( combo[i]->getLeafText() );
	}
	emit signalShowStatus( "Restored",2000 );
}

BranchDialog::~BranchDialog() {
	delete layout;
	delete cancelbutton;
	delete okbutton;
	delete restorebutton;
	int i;
	for( i=0;i<n_label;i++ ) {
		delete label[i];
	}
	for( i=0;i<n_edit;i++ ) {
		delete edit[i];
	}
	for( i=0;i<n_combo;i++ ) {
		delete combo[i];
	}

//	delete floatvalidator;
//	delete timevalidator;
//	delete degreevalidator;

	debug( "BranchDialog: Destructor \n" );
}

CFGDialog::CFGDialog( QPopupMenu* menu, int* _item_id,
					QArray<QString*> menu_help ) {
	CFGBranch *branch, *s_branch, *s_s_branch, *bak_branch;

	debug( "CFG Dalog: start of Constructor" ); 
	work_tree = _deeptree.linkTree();

	n_section_item = -1;
	n_s_section_item = -1;
	n_sub_item = -1;
	int item_id = *_item_id;

	section_item.resize(10);
	s_section_item.resize(10);
	sub_item.resize(10);

	work_tree->toFirst();
	while( work_tree->getCurrentLink() ) {
		if( work_tree->getCurrentLink()->getId() == 'b' ) {
			branch = work_tree->getCurrentLink()->getBranch();
			if( branch->getNSubBranch() ) { // ..,     =>   
				n_section_item++;
				if( (int)section_item.size() < n_section_item+1 ) 
					section_item.resize(section_item.size()+10);

				section_item[n_section_item] = new QPopupMenu;
				CHECK_PTR( section_item[n_section_item] );	
				menu->insertItem( branch->getName(),section_item[n_section_item],item_id );
				if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString( branch->getHelp()->data() );
				item_id++;

				branch->toFirst();
				while( branch->getCurrentLink() ) {
					if( branch->getCurrentLink()->getId() == 'b' ) {
						s_branch = branch->getCurrentLink()->getBranch();
						if( s_branch->getNSubBranch() ) {	//    s_branch
							n_s_section_item++;
							if( (int)s_section_item.size() < n_s_section_item+1 ) 
								s_section_item.resize(s_section_item.size()+10);
							s_section_item[n_s_section_item] = new QPopupMenu;
							CHECK_PTR( s_section_item[n_s_section_item] );	
							section_item[n_section_item]->insertItem( s_branch->getName(),
								s_section_item[n_s_section_item],item_id );							
							if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
							menu_help[item_id] = new QString( s_branch->getHelp()->data() );
							item_id++;

							s_branch->toFirst();
							while( s_branch->getCurrentLink() ) {
								if( s_branch->getCurrentLink()->getId() == 'b' ) {
									s_s_branch = s_branch->getCurrentLink()->getBranch();
									if( !(s_s_branch->getNSubBranch()) ) { //         
										n_sub_item++;
										if( (int)sub_item.size() < n_sub_item+1 ) 
											sub_item.resize(sub_item.size()+10);
										bak_branch = tree->findBranch( branch->getName() )->
															findBranch( s_branch->getName() )->
																findBranch( s_s_branch->getName() );
										sub_item[n_sub_item] = new SubItem( s_s_branch, bak_branch );
										CHECK_PTR(sub_item[n_sub_item]);
										connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
											this,SIGNAL(signalShowStatus(const char*,int)) );
//										connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//											this,SLOT(slotApply()) );
										s_section_item[n_s_section_item]->insertItem( s_s_branch->getName(),sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id ); //  
										if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
										menu_help[item_id] = new QString( s_s_branch->getHelp()->data() );
										item_id++;
									}																		
								}
								s_branch->toNext();
							}
						} else {	//     s_branch =>  
							n_sub_item++;
							if( (int)sub_item.size() < n_sub_item+1 ) 
									sub_item.resize(sub_item.size()+10);
							bak_branch = tree->findBranch( branch->getName() )->findBranch( s_branch->getName() );
							sub_item[n_sub_item] = new SubItem( s_branch, bak_branch );
							CHECK_PTR(sub_item[n_sub_item]);
							connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
								this,SIGNAL(signalShowStatus(const char*,int)) );
//							connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//											this,SLOT(slotApply()) );
							section_item[n_section_item]->insertItem( s_branch->getName(),
								sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id );
							if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
							menu_help[item_id] = new QString( s_branch->getHelp()->data() );
							item_id++;
						}
					}
					branch->toNext();
				}
			} else {									//				 
				n_sub_item++;
				if( (int)sub_item.size() < n_sub_item+1 ) 
					sub_item.resize(sub_item.size()+10);
				bak_branch = tree->findBranch( branch->getName() );
				sub_item[n_sub_item] = new SubItem( branch, bak_branch );
				CHECK_PTR(sub_item[n_sub_item]);
				connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
					this,SIGNAL(signalShowStatus(const char*,int)) );
//				connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//								this,SLOT(slotApply()) );
				menu->insertItem( branch->getName(),
					sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id );
				if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString( branch->getHelp()->data() );
				item_id++;
			}
		}
		work_tree->toNext();
	}
	menu->insertSeparator();
	menu->insertItem( "Save CFG",this,SLOT(slotSaveFile()),0,item_id );
	if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Save CFG" );
	item_id++;
	*_item_id = item_id;
//	connect( &devmanager, SIGNAL(signalStartCounting()),
//				this, SLOT(slotSetDisabled()) );
//	connect( &devmanager, SIGNAL(signalEndCounting()),
//				this, SLOT(slotSetEnabled()) );
//	tree->saved();
//	connect( tree, SIGNAL(changed()),
//				this, SLOT(slotCfgChanged()) );
	debug( "CFG Dalog: end of Constructor" ); 
}

CFGDialog::~CFGDialog() {
	int i;
	for( i=0;i<=n_section_item;i++ ) {
		delete section_item[i];
	}
	for( i=0;i<=n_s_section_item;i++ ) {
		delete s_section_item[i];
	}
	for( i=0;i<=n_sub_item;i++ ) {
		delete sub_item[i];
	}
	debug( "CFG Dialog: Destructor" );
}

//void CFGDialog::slotCfgChanged( void ) {
//	_cfgChanged = 1;
//}
//
void CFGDialog::setEnabled( void ) {
	int i;
	for( i=0;i<=n_sub_item;i++ ) {
		sub_item[i]->setEnabled();
	}
}

void CFGDialog::setDisabled( void ) {
	int i;
	for( i=0;i<=n_sub_item;i++ ) {
		sub_item[i]->setDisabled();
	}
}

//void CFGDialog::slotApply( void ) {
//	tree->copyTree( work_tree,tree );
//}
//
void CFGDialog::slotSaveFile( void ) {
	if( !tree->isChanged() ) {
		emit signalShowStatus( "CFG-file not changed so no update needed",2000 );
		return;
	}

	QMessageBox mb( "Warning","Save changes to CFG-file?",
		QMessageBox::Warning,QMessageBox::Yes,
		QMessageBox::No, 0 );
	int ret = mb.exec();
	if( ret != QMessageBox::Yes ) {
		emit signalShowStatus( "CFG-file not saved",2000 );
		return;
	}

	QString backCfg = cfgFile;
	backCfg.append( ".back" );
	rename( cfgFile,backCfg.data() );
	ret = tree->saveToFile( cfgFile );
	
	if( ret == 0 ) emit signalShowStatus( "CFG-file saved",2000 );
	else emit signalShowStatus( "CFG-file not saved",2000 );
}
