#ifndef DEVICE_H
#define DEVICE_H

#define QT_THREAD_SUPPORT

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <qthread.h>
#include <qdatetime.h>
#include <signal.h>
#include <qobject.h>
#include <qpoint.h>
#include "tree.h"
#include "star.h"
#include "counter.h"
#include "light.h"
#include "motor.h"
#include "power.h"
#include "viewstar.h"
#include "scind.hxx"
#include "scan.hxx"
//#include "jkerror.h"
//#include "atmos.h"

#define S2MS			1000.0	// sec to msec
#define MKS2MS			(0.001)	// mksec to msec 
#define MS2MKS			1000L	// msec to mksec 
#define TM_PREC			(1.5)	// time precision for count timeout interval
#define	HOUR_DATE_CHANG		12	// local hour, smena daty

#define HEAD_SIZE		5	// size of header of block (bytes) 
// Count modes:

#define SCENARIO_MODE			0
#define EXCHANGE_TEST_MODE		1
#define DETECTOR_TEST_MODE		2
#define STATISTIC_TEST_MODE		3
#define INITIAL_TESTS_MODE		4
#define DETECTOR_COUNTING_MODE		5
#define DETECTOR_STATISTIC_MODE		6
#define BACKGROUND_CORR_MODE		7
#define CENTERING_CORR_MODE		8
#define FLUX_ESTIMATION_MODE		19
// work:
#define FOCUS_CONTROL_MODE		9
#define STAR_CENTERING_MODE		10
#define NORMAL_MODE			11
#define GENERALIZED_MODE		12
#define BACKGROUND_MEASUR_MODE		13
// FINISH:
#define END_OF_PROGRAM			25

typedef double matrix43[MAX_N_COUNTERS][3];
//typedef void(*PF)();

void timeout_action( void );
void *read_status( void *arg );
void *read_data( void );
void terminate_program( void );
void* count_cycle( void );
void initThreads( void );

class DevManager : public QObject {
	Q_OBJECT

protected:
	void timerEvent( QTimerEvent* );

private:
	Counter *masterCounter;
	Counter *slaveCounter1;
	Counter *slaveCounter2;
	Counter *slaveCounter3;

	Counter *testCounter;

	scan_t scan[MAX_N_COUNTERS];

	Counter counters[MAX_N_COUNTERS];
	Light light;
	Motor motor;
	Power power;


	int	_pooling;

	int	_temperPooling;
	int	_overLightPooling;
	int	_buttonsPooling;
	int	_baseTimeTimerPooling;

	int	_baseTimeTimer;
	int	_temperTimer;
	int	_hvTimer;
	int	_buttonsTimer;

	int	_testOverLight_flag;
	int	_processButtons_flag;
	int	_getTemper_flag;

	bool isScenario;
	bool isUserStop;
	int _counterExist;
	int _lightExist;
	int _motorExist;
	int _powerExist;

	const char* _name_A;
	const char* _name_B;
	const char* _name_C;
	const char* _name_D;

	char _moduleA;
	char _moduleB;
	char _moduleC;
	char _moduleD;

	int _total_test_err1,_total_test_err2,_total_test_err3,_total_test_err4; 
	double _zShiftNormal,_zShiftGeneral,_zShiftGeneralSecond;

// Files & Dir-s
//	double		_tmZone;
	int		_countsSave;		//	save to file or not
	int		_logSave;		//	save to file or not
	QString		_logDirName;
	QString		_dataDirName;

	double	_normalExposition;
	double	_generalExposition;
	double	_focusExposition;
	double	_centerExposition;
	int		_focusScanStart,_focusScanFinish;
	int		_centerScanStart,_centerScanFinish;
	double	_normalBaseTime;
	double	_generalBaseTime,_generalSecondBaseTime;
	double	_focusBaseTime;
	double	_centerBaseTime;
	double	_temperature;

	double	_temperPoolingTime;
	double	_buttonPoolingTime;
	double	_highPoolingTime;

	int		_aperturesMode;
	int		_illumMode;
	int		_shiftMode;
	int		_doorMode;

	int		_dataBlockSize;
	int		_dataBlockSize0;
	int		_normalBlockNumber;
	int		_generalBlockNumber,_generalSecondBlockNumber;
	int		_focusBlockNumber;
	int		_centerBlockNumber;

// Initial & current:
	int _inputDoor;
	int _initialStop;
	int _illumValue;
	int _illumination;
	int _light;	// on/off
	int _buttons;
	int _buttonMode;
	int _highVoltage;
	int _hvLevel;
	int _safety;
	QArray<double> _discrLevels;

// Parking:
	int _inputDoorPark;
	int _buttonsPark;
	int _hvPark;
	int _illumPark;
	int _lightPark;	// on/off

// Tests:
	int		_exchangeTest, _statisticTest, _detectorTest;

	double	_testVAmpl;

	int			_nStatisticTestLight;
	QArray<double>	_statisticTestLight;
	int			_nDetectorCountingLight;
	QArray<double>	_detectorCountingLight;
	int			_nDetectorCountingVoltage;
	QArray<int>	_detectorCountingVoltage;
	int			_nDetectorCountingDiscrimination;
	QArray<double>	_detectorCountingDiscrimination;
	int				_nDetectorStatisticLight;
	QArray<double>	_detectorStatisticLight;
	double			_detectorTestLight;	

	int		_testApert;
	int		_normalApert,_generalApert,_generalSecondApert,_focusApert;
	int		_centerApert,_backgroundApert,_countApert,_statisticApert;

	int		_backgroundApShift;

	double	_exchangeTestTime,_detectorTestTime;
	double	_detectorCountingTime,_detectorStatisticTime;
	double	_backgroundTime;
	double	_normalAccumTime;
	double	_generalAccumTime;
	double	_estimationAccumTime;
	double	_focusAccumTime;
	double	_centerAccumTime;
	double	_fluxTolerance;
	double	_nonPoisTolerance;
	QArray<double>	_testCounts;

	QArray<double> _backgroundCorrectionCoefficients;
	QArray<double> _backgrounds;
	int	_centerDeltaX0, _centerDeltaY0;
	QArray<double> _nonPoisson;
	QArray<double> _nonLinearity;

	QTime _startBaseTime;

	QArray<char>		currentScenario;
	QArray<QString*>	scenarioName;
	QArray<QString*>	scenarioScript;
	int	_nScenario;
	int	_currentScenarioNumber;

	int	_stopScenario;
	int	_pauseScenario;

	matrix43 *coef;
	QPoint	*delta;

	int _nCorrected[MAX_N_COUNTERS];

private:

	void	startCounting( void );
	void	endCounting( void );
	void	disablePooling( void );
	void	enablePooling( void );
	void	startTemperPooling( void );
	void	stopTemperPooling( void );	
	void	startOverLightPooling( void );
	void	stopOverLightPooling( void );
	void	startButtonsPooling( void );
	void	stopButtonsPooling( void );
	void	setBrightness( int,int );
	void	changeBrightness( int );
	int		changeAperture( int, int );

	int		setExpos( double,int );
	int		setBlSize( int,int );
	int 		calcBlockNumber(double& baseTime, double exposition);
	int 		calcExpositionNumber( int _blockNumber );
	int		setBlockNumber( int,int&,int );
	int		setVoltage( int,int,bool _wait=true );
	int		getVoltage( void ) {return( power.getVoltage() );}
	int		setLight( double,int, bool wait=true );
	double		getLight( void ) {return( light.getLight() );}
	int		setLightOn( int );
	int		setLightOff( int );
	int		setDiscriminations( QArray<double>&, int );
	void		getDiscriminations( QArray<double>& );
	int		synchAt( int,int );
	int		shiftAt( int,int );

	void		openProgressor( const QString& );	//
	void		closeProgressor( void );		//
	
	void		openMode( const QString&,double );	//
	void		closeMode( void );		//

	void		openScenario( const QString&,double );	//
	void		closeScenario( void );		//

	int		runNormal( void );				//!
	int		runGeneralized( void );			//!
	int		runFluxEstimation( void );		//!
	int		scanAperture( double, double, int, double,
				int, int, const QString&, bool, bool );
	int		runFocusControl( void );		//!
	int		runStarCentering( void );		//!
	int		runExchangeTest( void );		//!
	int		runDetectorTest( void );		//!
	int		runStatisticTest( void );		//!
	int		runDetectorStatisticMeasurement( void );	//!
	int		runDetectorCountingMeasurement( void );		//!
	int		runBackgroundCorrection( void );			//!
	int		runBackgroundMeasurement( void );			//!
	int		runCenterCorrection( void );				//!

	void	terminateCountMode( int );		//!

public:
	DevManager( void );		//
	~DevManager();

	void testExeption( int );

	int	calcTimeoutInterval( double,int );
	void	startDevTimer( int );
	void	stopDevTimer( void );
	void	waitData( void );
	bool	testUserStop( void );
	int	catchAppendix( void );
	void	outCountResult( void );

	int	resetCounters( int );
	void	resetDriverBlockCounter( void );

	int		isCounterExist( void ) {return( _counterExist );}
	int		isLightExist( void ) {return( _lightExist );}
	int		isMotorExist( void ) {return( _motorExist );}
	int		isPowerExist( void ) {return( _powerExist );}
	void	create( void );		//
	int		testShutter( Counter* );	//
	int		testIllum( void );		//
	int		testButtons( void );	//
	int		testMotorStatus( int );	//
	int		testPowerStatus( int );	//
	Counter* searchSlave( int&,int&,int&,int&,Counter*,int );	//
	int		setInduceRegime( int );	//
	int		resetTestCounter( void );	//
	int		switchTestCounter( void );	//
	Counter* getTestCounter( void ) { return( testCounter ); } //
	void 	resetTestData( void );	//
	int	runTestOneCounter( void );	//
	int	runTestAllCounters( void );	//
	int	runAllCounters( void );	//
	void	searchTestErrorsOneCounter( void );//
	void	searchTestErrorsAllCounters( void );//
	double	resumeTestOneCounter( void );	//
	double	resumeTestAllCounters( void );	//
	void	sendToScreen( const QString& text );
/*
	void	sendToScreen( const QString&,
				const QString& = NULL,
				const QString& = NULL,
				int=0, int=0,
				bool=true, bool=true );
*/
	void	sendFluxHead( bool clear,
				int nHeadRow,
				const char* headRow1,
				const char* headRow2 = "",
				const char* headRow3 = "" );
	int	accumResume( QTime midAccumTime, bool isgen );
	int	makeAccumResultScind( QTime midAccumTime, bool isgen );
	int	makeAccumResultAtmos( QTime midAccumTime, bool isgen );
	int	showAccumResultScind( bool isgen );
	void	showAccumResultAtmos( int row, bool isgen );
	void	fluxToTable( int nRows,
				int chan,
				double val1,
				double val2 = 0,
				double val3 = 0 );
	void	sendTable( int,int,int,int,bool=false,bool=false );

	void	prepareFiles( void );
	void	loadConst( void );

	void	putData( void );
	bool	cleanData( int expositionNumber, long *addr = 0 );

	void	setStartBaseTime( void );
	QTime	setMidTime( QTime startTime );

	int		isIllumOn( void ) {return( light.isIllumOn() ); }
	int		isHVOn( void );
	int		isButtonsOn( void ) {return( light.isButtonsOn() ); }
	int		isSafetyOn( void ) {return( power.isSafetyOn() ); }
	int		isDoorOpen( void ) {return( motor.isDoorOpen() ); }
	int		currentAperture( void ) { return( motor.currentAperture() ); }
	int		getCurrentBrightness( void ) { return( light.getCurrentBrightness() ); }
	int		getStopCorrect( void ) {return( motor.getStopCorrect() ); }
	int		getStopCorrectNew( void ) {return( motor.getStopCorrectNew() ); }
	int		getNStops( void ) { return( motor.getNStops() ); }
	int		getScenarioNumber( void ) {return( _currentScenarioNumber );}
	QString&	getScenarioName( int number );
	int		getMaxScenarioNumber( void ) {return( _nScenario );}
	QString	getStopName( int );
	void	startAdjustAperture( void );
	void	acceptAdjustAperture( int );
	void	acceptBackgroundCorrection( matrix43* );
	void	acceptCenterCorrection( QPoint* );
	void	getBackgroundCorrectionCoefficients( matrix43* );
	void	getCenterCorrectionCoefficients( QPoint* );

	int	testOverLight( int );
	void	getTemper( int );
	void	processButtons( int );

//	void	countAborted( void );			//

	void	statistic_test_mode( void );	//!
	void	detector_test_mode( void );		//!
	void	exchange_test_mode( void );		//!
	void	detector_counting_mode( void );	//!
	void	detector_statistic_mode( void );//!
	void	background_corr_mode( void );	//!
	void	background_measur_mode( void );	//!
	void	centering_corr_mode( void );	//!
	void	focus_control_mode( void );		//!
	void	star_centering_mode( void );	//!
	void	normal_mode( void );			//!
	void	flux_estimation_mode( void );	//!
	void	generalized_mode( void );		//!
	void	initial_tests_mode( void );	//!
	double	DevManager::getAccumTime( char mode );
	void	scenario_mode( void );	//


public slots:

	void	init( void );		//
	void	park( void );

	int	stopCounters( int );
	void	stopCounting( void );
	void	stopScenario( void );
	void	playScenario( void );
	void	setPause( void );
	void	test( void );
	void	change( void );
	int	turnDoor( const QString&, int, bool _backApert=true );
	int	turnDoor( int, int, bool _backApert=true );
	void	turnHV( int,int );
	void	turnIllum( const QString&, int );
	void	turnIllum( int, int );
	void	turnIllumOn( void );
	void	turnIllumOff( void );
	int		shiftAperture( int,int );
	int		setAperture( const QString&, int, bool _hard=false );
	int		setAperture( int, int, bool _hard=false );
	void	setBrightnessUp( void );
	void	setBrightnessDown( void );
	void	turnButtons( const QString&,int );
	void	turnButtons( int,int );
	void	turnSafety( const QString&,int );
	void	turnSafety( int,int );
	void	nullSet( void );
	int		fineShift( int );
	int		movePlus( void );
	int		moveMinus( void );

	void	exchangeTest( void );
	void	detectorTest( void );
	void	statisticTest( void );
	void	initialTests( void );
	char	decodeMode( char mode );
	void	setCurrentScenario( int number );
	void	detectorCountingMeasurement( void );
	void	detectorStatisticMeasurement( void );
	void	initTests( void );
	void	backgroundCorrection( matrix43* );
	void	backgroundMeasurement( void );
	void	fluxEstimation( void );
	void	centerCorrection( QPoint* );
	void	normal( void );
	void	generalized( void );
	void	focusControl( void );
	void	starCentering( void );
	void	setProgress( double );		//
	void	setRemainTimeMode( double );		//
	void	setRemainTimeScenario( double );		//

/*
signals:

	void	signalIllumination( int );
	void	signalHV( int );
	void	signalButtons( int );
	void	signalDoor( int );
	void	signalAperture( int );
	void	signalBrightness( int );
	void	signalTemperature( double );
	void	signalStartCounting( void );
	void	signalEndCounting( void );
	void	signalSendCorrection();
	void	signalToScreen( const QString& );
	void	signalProgress( const QString&,int );
	void	signalMode( const QString&,int );
	void	signalShowMessage( const char*,int );
*/
};

#endif
