
#include "graph.h"

extern CFGTree* tree;

QPoint wDev( double x,double y, Scale* s ) {
	return( QPoint( (int)( (x-s->xmin)*s->sx ), 
				(int)( (y-s->ymin)*s->sy ) ) );
}

int wDevX( double x, Scale* s ) {
	return( (int)( (x - s->xmin) * s->sx ) );
}

int wDevY( double y, Scale* s ) {
	return( (int)( (y - s->ymin) * s->sy ) );
}

double devWX( int x, Scale* s ) {
	return( (double)x / s->sx + s->xmin );
}

double devWY( int y, Scale* s ) {
	return( (double)y / s->sy + s->ymin );
}

GraphWindow::GraphWindow( QWidget* parent=0,const char* name=0 )
			: QWidget( parent,name ) {
	curve1 = 0;

	loadConst();
	setCaption( "Graph" );

//	setBackgroundMode( NoBackground );
	setBackgroundColor( _background  );

	connect( tree, SIGNAL( changed() ),
			this,SLOT( change() ) );
	connect( qApp->mainWidget(),SIGNAL( signalDrawCurve(DataArray*,DataArray*) ),
		this,SLOT( draw(DataArray*,DataArray*) ) );
//	int disp_width = qApp->desktop()->width();
//	int disp_height = qApp->desktop()->height();

        setGeometry( _winX, _winY, _winWidth, _winHeight );
//	setGeometry( disp_width/2,0,
//				disp_width/2-10,disp_height-50 );
}

GraphWindow::~GraphWindow( void ) {
}

void GraphWindow::loadConst( void ) {
	const char* _geometry = tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
		findLeaf( "WindowGeometry" )->getData()->data();
	if( !strcasecmp( _geometry,"AUTO" ) ) {
		_winX = qApp->desktop()->width()/2;
		_winY = 0;
		_winWidth = qApp->desktop()->width()/2 - 10;
		_winHeight = qApp->desktop()->height() - 50;
	} else {
		int key = sscanf( _geometry,"%d%d%d%d", &_winX, &_winY,
			&_winWidth, &_winHeight );
		if( key != 4 ) {
			debug( "GraphWindow: Invalid geometry!!!" );
			_winX = qApp->desktop()->width()/2;
			_winY = 0;
			_winWidth = qApp->desktop()->width()/2 - 10;
			_winHeight = qApp->desktop()->height() - 50;
		}
	}
	_background = tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
		findLeaf( "Background")->getData()->data();
	_axesColor = tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
		findLeaf( "AxesColor")->getData()->data();
	_fontSize = tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
		findLeaf("FontSize")->getData()->toInt();
	_pointSize = tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
		findLeaf( "PointSize" )->getData()->toInt();
	_font = QFont( "Courier",_fontSize );

	int _show1,_show2,_show3,_show4;
	_color1 = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A" )->
		findLeaf( "Color" )->getData()->data();
	const char* yes = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A" )->
		findLeaf( "ShowPicture" )->getData()->data();
	if( strcasecmp( yes,"yes" ) == 0 ) _show1 = 1;
		else _show1 = 0;
	_frac1 = _show1 * tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A" )->
		findLeaf( "Size" )->getData()->toInt();

	_color2 = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "AD" )->
		findLeaf( "Color" )->getData()->data();
	yes = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "AD" )->
		findLeaf( "ShowPicture" )->getData()->data();
	if( strcasecmp( yes,"yes" ) == 0 ) _show2 = 1;
		else _show2 = 0;
	_frac2 = _show2 * tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "AD" )->
		findLeaf( "Size" )->getData()->toInt();

	_color3 = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A-0.5" )->
		findLeaf( "Color" )->getData()->data();
	yes = tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A-0.5" )->
		findLeaf( "ShowPicture" )->getData()->data();
	if( strcasecmp( yes,"yes" ) == 0 ) _show3 = 1;
		else _show3 = 0;
	_frac3 = _show3 * tree->findBranch( "Display" )->
		findBranch( "Indices" )->findBranch( "A-0.5" )->
		findLeaf( "Size" )->getData()->toInt();

	_color4 = tree->findBranch( "Display" )->
		findBranch( "Integrals" )->findBranch( "Isoplanatic" )->
		findLeaf( "Color" )->getData()->data();
	yes = tree->findBranch( "Display" )->
		findBranch( "Integrals" )->findBranch( "Isoplanatic" )->
		findLeaf( "ShowPicture" )->getData()->data();
	if( strcasecmp( yes,"yes" ) == 0 ) _show4 = 1;
		else _show4 = 0;
	_frac4 = _show4 * tree->findBranch( "Display" )->
		findBranch( "Integrals" )->findBranch( "Isoplanatic" )->
		findLeaf( "Size" )->getData()->toInt();

	_total = _frac1 + _frac2 + _frac3 + _frac4;
}

void GraphWindow::change( void ) {
	loadConst();

	setGeometry( _winX+10, _winY+68, _winWidth, _winHeight );
//	Pochemu +10 i +68 NE ZNAYU !!!!!

	setBackgroundColor( _background );
	if( curve1 ) reDraw();
}

void GraphWindow::drawAxisX( QPainter *p, Scale *s, int nx, int top=0 ) {

	int i;
	QPoint b;
	QString txt;

	QPen oldpen = p->pen();
	p->setPen( _axesColor );

	int wl = p->window().left();
	int wr = p->window().right();
	int wt = p->window().top();
	int wb = p->window().bottom();

	int tw = p->fontMetrics().width( "wwwwwww" )+10; 
	int th = p->fontMetrics().height();
//	int swy = (int)( 1000.0 / 15.0 );
	QPoint p1 = p->xFormDev( QPoint(0,0) ); 
	QPoint p2 = p->xFormDev( QPoint(0,0+0.7*(float)th) ); 
	int swy = abs( p1.y() - p2.y() );

	double ddx = fabs( s->xmax-s->xmin ); 
	double sign = ddx / fabs( ddx );
	ddx = fabs( ddx );
	double log10x = log10( ddx );
	double floorlog10x = floor( log10x );
	double dx = sign * pow( 10, log10x-floorlog10x+1 ) / (double)nx;
//	debug( "ddx=%f floorlog10x=%f",ddx,floorlog10x );
//	debug( "dx=%f",dx ); 	

	double minfabs = fabs(dx-1);
	double _dx = 1;
	if( fabs(dx-2) < minfabs ) { minfabs = fabs(dx-2); _dx=2; }
	if( fabs(dx-5) < minfabs ) { minfabs = fabs(dx-5); _dx=5; }
	if( fabs(dx-10) < minfabs ) { minfabs = fabs(dx-10); _dx=10; }
	if( fabs(dx-20) < minfabs ) { minfabs = fabs(dx-20); _dx=20; }
	if( fabs(dx-25) < minfabs ) { minfabs = fabs(dx-25); _dx=25; }
	dx = _dx;
//	debug( "dx=%f",dx );

	double ddwx = dx*pow(10,floorlog10x-1);
	int dwx = (int)( ddwx * (s->sx) );
	int wl0 = wDevX( ddwx * ceil(s->xmin/ddwx), s );

	if( top ) p->drawLine( wl,wt,wr,wt );
		else p->drawLine( wl,wb,wr,wb );
	for( i=wl0; i<=wr; i+=dwx ) {
		p->setViewXForm( true );
		if( top ) {
			p->drawLine( i,wt,i,wt-swy );
			b = p->xForm( QPoint(i,wt) );
		}else {
			p->drawLine( i,wb,i,wb+swy );
			b = p->xForm( QPoint(i,wb) );
		}
		p->setViewXForm( false );
		txt.sprintf( "%.2f", devWX(i,s) );
		tw = p->fontMetrics().width( txt.data() );
		if( top ) p->drawText( b.x()-tw/2,b.y()-th/2,txt.data() );
			else p->drawText( b.x()-tw/2,b.y()+th,txt.data() );
	}
	p->setViewXForm( true );
	p->setPen( oldpen );
}

void GraphWindow::drawAxisX( QPainter *p ) {

	QPen oldpen = p->pen();
	p->setPen( _axesColor );

	int wl = p->window().left();
	int wr = p->window().right();
//	int wt = p->window().top();
	int wb = p->window().bottom();

	p->drawLine( wl,wb,wr,wb );
	p->setPen( oldpen );
}

void GraphWindow::drawAxisY( QPainter *p, Scale *s, int ny, 
				int left, const char* name=""  ) {
	int i;
	QPoint b;
	QString txt;

	QPen oldpen = p->pen();
	p->setPen( _axesColor );

	int wl = p->window().left();
	int wr = p->window().right();
	int wt = p->window().top();
	int wb = p->window().bottom();

	int tw = p->fontMetrics().width( "wwwwwww" )+10; 
	int th = p->fontMetrics().height();
	int th2 =(int)((double)th/3.0);
//	int swx = (int)( 1000.0 / 50.0 );

	QPoint p1 = p->xFormDev( QPoint(0,0) ); 
	QPoint p2 = p->xFormDev( QPoint(0+0.7*(float)th,0) ); 
	int swx = abs( p1.x() - p2.x() );

	double ddy = fabs( s->ymax-s->ymin ); 
	double sign = ddy / fabs( ddy );
	ddy = fabs( ddy );
	double log10y = log10( ddy );
	double floorlog10y = floor( log10y );
	double dy = sign * pow( 10, log10y-floorlog10y+1 ) / (double)ny;

	double minfabs = fabs(dy-1);
	double _dy = 1;
	if( fabs(dy-2) < minfabs ) { minfabs = fabs(dy-2); _dy=2; }
	if( fabs(dy-5) < minfabs ) { minfabs = fabs(dy-5); _dy=5; }
	if( fabs(dy-10) < minfabs ) { minfabs = fabs(dy-10); _dy=10; }
	if( fabs(dy-20) < minfabs ) { minfabs = fabs(dy-20); _dy=20; }
	if( fabs(dy-25) < minfabs ) { minfabs = fabs(dy-25); _dy=25; }
	dy = _dy;

	double ddwy = dy*pow(10,floorlog10y-1);
	int dwy = (int)( ddwy * (s->sy) );
	int wb0 = wDevY( ddwy * ceil(s->ymin/ddwy), s );

	p->setViewXForm( true );
	if( left ) p->drawLine( wl,wt,wl,wb );
		else p->drawLine( wr,wt,wr,wb );
	for( i=wb0; i<=wt; i+=dwy ) {
		p->setViewXForm( true );
		if( left ) p->drawLine( wl,i,wl+swx,i );
			else p->drawLine( wr,i,wr-swx,i );
		b = p->xForm( QPoint(wl,i) );
		p->setViewXForm( false );
		if( left ) { txt.sprintf( "%5.2f ", devWY(i,s) );
			tw = p->fontMetrics().width( txt.data() );
			p->drawText( b.x()-tw,b.y()+th2,txt.data() );
		}
	}
	if( left ) {
		p->setViewXForm( true );
		b = p->xForm( QPoint(wl,wt) );
		p->setViewXForm( false );
		tw = p->fontMetrics().width( "  " );
		p->drawText( b.x()+tw,b.y()+1.8*(float)th,name );
	}
	p->setViewXForm( true );
	p->setPen( oldpen );
}

void GraphWindow::draw( DataArray* _x, DataArray* _y ) {

	x = _x; y = _y;
	curve1 = 1;

	reDraw();
}

void GraphWindow::drawCurve( QPainter* wp, QPainter* bp,
	QColor color, Scale* s, int wt, int wr ) {
	int i;

//	wp->setBrush( color );
	bp->setBrush( color );
//	wp->setPen( color );
	bp->setPen( color );

//	wp->setWindow( 0,wt,wr+1,-1-wt );
	bp->setWindow( 0,wt,wr+1,-1-wt );

	QPoint es1 = bp->xFormDev( QPoint(10,10) ); 
	QPoint es2 = bp->xFormDev( QPoint(10+_pointSize,10+_pointSize) ); 
	int ax = abs( es1.x() - es2.x() );
	int ay = abs( es1.y() - es2.y() );

	if( ay == 0 ) ax = 1;
	for( i=0; i<=x->getNumber(); i++ ) {
//		wp->drawEllipse( wDevX( x->get(i),s )-ax,
//			wDevY( y->get(i),s )-ay,2*ax,2*ay );
		bp->drawEllipse( wDevX( x->get(i),s )-ax,
			wDevY( y->get(i),s )-ay,2*ax,2*ay );
	}	
}

void GraphWindow::reDraw( void ) {
	Scale s;

	QPainter windowpainter;
	QPainter bufferpainter;

//	windowpainter.begin( this );
	bufferpainter.begin( &_buffer );

	_buffer.fill( _background );

//	windowpainter.setFont( _font );
	bufferpainter.setFont( _font );

	int wr = 1000;
	int wt = 1000;

	s.xmin = x->getDataMin();
	s.xmax = x->getDataMax();
	s.ymin = y->getDataMin();
	s.ymax = y->getDataMax();

	s.sx = (double)wr / ( s.xmax - s.xmin ); 
	s.sy = (double)wt / ( s.ymax - s.ymin );

	int nx = 10;
	int ny = 5;

	int border_x = 20;
	int border_y = (int)( (float)bufferpainter.fontMetrics().height() );
	int view_left = bufferpainter.fontMetrics().width( "wwwwwww" )+10;
	int view_top = (int)( (float)bufferpainter.fontMetrics().height() * 2.0 );
	int view_height = (int)( (float)(height()-view_top-border_y)/_total
				-border_y );

// --------------------- Curve1 -------------------------

//	windowpainter.setViewport( view_left,view_top,
//		width()-view_left-border_x,view_height );	
	bufferpainter.setViewport( view_left,view_top,
		width()-view_left-border_x,view_height );
//	windowpainter.setWindow( 0,wt,wr+1,-1-wt );
	bufferpainter.setWindow( 0,wt,wr+1,-1-wt );
//	drawAxisX( &windowpainter,&s,nx,1 );
	drawAxisX( &bufferpainter,&s,nx,1 );

	if( _frac1 ) {
//		windowpainter.setViewport( view_left,view_top,
//			width()-view_left-border_x,view_height*_frac1 );	
		bufferpainter.setViewport( view_left,view_top,
			width()-view_left-border_x,view_height*_frac1 );
		view_top += (int)((float)view_height*_frac1) + border_y;

		drawCurve( &windowpainter, &bufferpainter, _color1,&s,wt,wr );

//		drawAxisX( &windowpainter );
//		drawAxisY( &windowpainter,&s,ny,1,"A" );
//		drawAxisY( &windowpainter,&s,ny,0 );
		drawAxisX( &bufferpainter );
		drawAxisY( &bufferpainter,&s,ny,1,"A" );
		drawAxisY( &bufferpainter,&s,ny,0 );

	}

// --------------------- Curve2 -------------------------

	if( _frac2 ) {
//		windowpainter.setViewport( view_left,view_top,
//			width()-view_left-border_x,view_height*_frac2 );
		bufferpainter.setViewport( view_left,view_top,
			width()-view_left-border_x,view_height*_frac2 );
		view_top += (int)((float)view_height*_frac2) + border_y;

		drawCurve( &windowpainter, &bufferpainter, _color2,&s,wt,wr );

//		drawAxisX( &windowpainter );
//		drawAxisY( &windowpainter,&s,ny,1,"AD" );
//		drawAxisY( &windowpainter,&s,ny,0 );
		drawAxisX( &bufferpainter );
		drawAxisY( &bufferpainter,&s,ny,1,"AD" );
		drawAxisY( &bufferpainter,&s,ny,0 );
	}
// --------------------- Curve3 -------------------------

	if( _frac3 ) {
//		windowpainter.setViewport( view_left,view_top,
//			width()-view_left-border_x,view_height*_frac3 );
		bufferpainter.setViewport( view_left,view_top,
			width()-view_left-border_x,view_height*_frac3 );
		view_top += (int)((float)view_height*_frac3) + border_y;

		drawCurve( &windowpainter, &bufferpainter, _color3,&s,wt,wr );

//		drawAxisX( &windowpainter );
//		drawAxisY( &windowpainter,&s,ny,1,"A-0.5" );
//		drawAxisY( &windowpainter,&s,ny,0 );
		drawAxisX( &bufferpainter );
		drawAxisY( &bufferpainter,&s,ny,1,"A-0.5" );
		drawAxisY( &bufferpainter,&s,ny,0 );
	}
// --------------------- Curve4 -------------------------

	if( _frac4 ) {
//		windowpainter.setViewport( view_left,view_top,
//			width()-view_left-border_x,view_height*_frac4 );
		bufferpainter.setViewport( view_left,view_top,
			width()-view_left-border_x,view_height*_frac4 );
		view_top += (int)((float)view_height*_frac4) + border_y;

		drawCurve( &windowpainter, &bufferpainter, _color4,&s,wt,wr );

//		drawAxisY( &windowpainter,&s,ny,1,"Isoplan" );
//		drawAxisY( &windowpainter,&s,ny,0 );
		drawAxisY( &bufferpainter,&s,ny,1,"Isoplan" );
		drawAxisY( &bufferpainter,&s,ny,0 );
	}

//	drawAxisX( &windowpainter,&s,nx );
	drawAxisX( &bufferpainter,&s,nx );


//	windowpainter.end();
	bufferpainter.end();
	repaint();
}

void GraphWindow::setPoint( QPoint _p ) {

	QPainter windowpainter;
	QPainter bufferpainter;

	windowpainter.begin( this );
	bufferpainter.begin( &_buffer );

	windowpainter.setPen( _color1 );
	bufferpainter.setPen( _color1 );

	windowpainter.end();
	bufferpainter.end();
}

void GraphWindow::paintEvent( QPaintEvent* e ) {

	bitBlt( this, 0, 0, &_buffer );
}

void GraphWindow::resizeEvent( QResizeEvent* e ) {
	int _chng = 0;
	if( _winWidth != e->size().width() ) {
		_winWidth = e->size().width();
		_chng = 1;
	}
	if( _winHeight != e->size().height() ) {
		_winHeight = e->size().height();
		_chng = 1;
	}
	debug( "Graph::ResizeEvent: %d %d chng=%d",
			_winWidth,_winHeight,_chng );

	if( _chng ) {
		QString _geometry;
		_geometry.sprintf( "%d %d %d %d",_winX,_winY,_winWidth,_winHeight );
		tree->findBranch( "Preferences" )->
			findBranch("Graphic window settings")->
		findLeaf( "WindowGeometry" )->replaceData(_geometry);
		tree->setChanged();
	}	
	_buffer.resize( e->size() );
	_buffer.fill( _background );
	if( curve1 ) reDraw();
}
/*
void GraphWindow::moveEvent( QMoveEvent* e ) {
	_winX = e->pos().x();
	_winY = e->pos().y();

	QString _geometry;
	_geometry.sprintf( "%d %d %d %d",_winX,_winY,_winWidth,_winHeight );
	tree->findBranch( "Preferences" )->
		findBranch("Graphic window settings")->
	findLeaf( "WindowGeometry" )->replaceData(_geometry);
	tree->setChanged();
	debug( "MoveEvent: %d %d",_winX,_winY );
}
*/
void GraphWindow::mousePressEvent( QMouseEvent* e ) {
}

void GraphWindow::mouseMoveEvent( QMouseEvent* e ) {
}
