#include "light.h"
#include "devplus.h"
#include "tools.h"

Light::Light( void ) {
	_name = "Lighters & Buttons";
	_currentBrightness = 0;
	_illumination = -1;
	_buttons = -1;
	_currentLight = 0;
	ready = FALSE;
}

Light::~Light() {
}

void Light::loadConst( void ) {
	debug( "light: LoadConst" );
	_module = readCharConst( _name,"Common","Module" );
	_setIllum = readCharConst( _name,"Command","SetIllum" );
	_getIllum = readCharConst( _name,"Command","GetIllum" );
	_setLight = readCharConst( _name,"Command","SetLight" );
	_getLight = readCharConst( _name,"Command","GetLight" );
	_getCrc = readCharConst( _name,"Command","GetCrc" );
	_getStatus = readCharConst( _name,"Command","GetStatus" );
	_setVAmpl = readCharConst( _name,"Command","SetVAmpl" );
	_getVAmpl = readCharConst( _name,"Command","GetVAmpl" );
	_setBMode = readCharConst( _name,"Command","SetBMode" );
	_getBMode = readCharConst( _name,"Command","GetBMode" );
	_setParam = readCharConst( _name,"Command","SetParam" );
	_getParam = readCharConst( _name,"Command","GetParam" );
	_illumOn = readCharConst( _name,"Command","IllumOn" );
	_illumOff = readCharConst( _name,"Command","IllumOff" );
	_lightOn = readCharConst( _name,"Command","LightOn" );
	_lightOff = readCharConst( _name,"Command","LightOff" );
	_varyOn = readCharConst( _name,"Command","VaryOn" );
	_varyOff = readCharConst( _name,"Command","VaryOff" );
	_buttLock = readCharConst( _name,"Command","ButtLock" );
	_buttUnLock = readCharConst( _name,"Command","ButtUnLock" );
	_getIdent = readCharConst( _name,"Command","GetIdent" );
	_getConst = readCharConst( _name,"Command","GetConst" );
	_reset = readCharConst( _name,"Command","Reset" );

	 const char* _id = readStrConst( _name,"Common","Identification" );
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	ready = TRUE;
}

int Light::get( char _module, char _command, const char* _name,
		const char* _warning, unsigned char** p_respbuf, int dialog ) {
	QString str;
	int i;
	int ret;
	int resp;

	for( i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = sendCommand( _module,_command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		jkwait( DEVICE_WAIT_INTERVAL );
	}
	if( (*p_respbuf)[0] == DSC ) {
		str.sprintf( "%s disconnected!",_name );
		ret = -1;
	} else {
		str.sprintf( "%s: %s",_name,_warning );
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(ret);
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Light::set( char _module, char _command, const char* _name,
		const char* _warning, int _dialog ) {
	QString str;
	int resp;
	int i;
	unsigned char *respbuf;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Light::set( char _module, char _command, char _data, const char* _name,
			const char* _warning, int _dialog ) {
	QString str;
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,_data,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}	
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Light::set( char _module, char _command, char _data_low, char _data_high,
			const char* _name, const char* _warning, int _dialog ) {
	QString str;
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,_data_low,_data_high,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}	
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}


int Light::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	debug( "Light::getIdent:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getIdent, str.data(),
			"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
			debug( "Light: %X %X %X %X",respbuf[1],respbuf[2],
					respbuf[3],respbuf[4] );	
			if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "Module \"%s\": Invalid identification: %X %X %X %X!",
						_name,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);

			}
		}
	}
}

int Light::reset( void ) {
	QString str;
	int resp;
	unsigned char* respbuf;
	int i;
	int ret=0;

	debug( "Light::reset:" );
	int cycle = 1;
	_currentBrightness = 0;
	_illumination = -1;
	_buttons = -1;
	_currentLight = -1;
	while( cycle ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module, _reset, &respbuf );
			if( resp == -2 ) {	// disconnect
				return(1);
			}
		}
		str.sprintf( "Module \"%s\": Error in Reset!",_name );
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: cycle=0; ret=0; break;
			case  QMessageBox::Retry : break;
		}
	}
	return( ret );
}

int Light::getConst( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "Module \"%s\": Error in getConst!",_name );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::testCrc( void ) {
	QString str;
	unsigned char *respbuf;

	debug( "testCrc:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ){
					str.sprintf( "Module \"%s\": EEPROM corrupted!",_name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Light::testButtons( void ) {
	uchar _status;
	if( getStatus( _status ) != 1 ) return(0);
	if( _status & BUTT_LOCK ) _buttons = 0;
	else _buttons = 1;
	return(1);
}

int Light::testIllum( void ) {
	uchar _status;
	if( getStatus( _status ) != 1 ) return(0);
	if( _status & ILLUM_ON ) _illumination = 1;
	else _illumination = 0;
// !!!	getIllum();
	return(1);
}

int Light::getStatus( uchar& _status ) {
	QString str;
	unsigned char *respbuf;

	debug( "getStatus:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getStatus, str.data(),
				"Error in getStatus!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Invalid status!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { _status = respbuf[1]; return(1); }
			}
		}
	}
}

int Light::turnIllum( const QString& _on, int dialog ) {
	int _on_int = 0;
	if( operator==( _on.upper(),"ON" ) ) _on_int = 1;
	if( operator==( _on.upper(),"OFF" ) ) _on_int = 0;
	return( turnIllum( _on_int,dialog ) );
}

int Light::turnIllum( int _on, int dialog ) {
	int ret;
	if( _on ) ret = setIllumOn( dialog );
	else ret = setIllumOff( dialog );
	return( ret );
}

int Light::setIllumOn( int dialog ) {
	int ret=1;
	QString str;

	debug( "setIllumOn:" );
	if( _illumination != 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_illumOn,str.data(),
			"Error in setIllumOn!",dialog );
		if( ret == 1 ) _illumination = 1;
	}
	return( ret );
}

int Light::setIllumOff( int dialog ) {
	int ret=1;
	QString str;

	debug( "setIllumOff:" );
	if( _illumination != 0 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_illumOff,str.data(),
			"Error in setIllumOff!",dialog );
		if( ret == 1 ) _illumination = 0;
	}
	return( ret );
}

int Light::turnLight( int _on, int dialog ) {
	int ret;
	if( _on ) ret = setLightOn( dialog );
	else ret = setLightOff( dialog );
	return( ret );
}

int Light::setLightOn( int dialog ) {
	debug( "setLightOn:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_lightOn,str.data(),
		"Error in setLightOn!", dialog ) );
}

int Light::setLightOff( int dialog ) {
	debug( "setLightOff:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_lightOff,str.data(),
		"Error in setLightOff!", dialog ) );
}

int Light::setVaryOn( int dialog ) {
	debug( "setVaryOn:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_varyOn,str.data(),
		"Error in setVaryOn!", dialog ) );
}

int Light::setVaryOff( int dialog ) {
	debug( "setVaryOff:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_varyOff,str.data(),
		"Error in setVaryOff!", dialog ) );
}

int Light::turnButtons( const QString& _on, int dialog ) {
	int _on_int = 0;
	if( operator==( _on.upper(),"ON" ) ) _on_int = 1;
	if( operator==( _on.upper(),"OFF" ) ) _on_int = 0;
	return( turnButtons( _on_int,dialog ) );
}

int Light::turnButtons( int _on, int dialog ) {
	int ret;
	if( _on ) ret = setButtUnLock( dialog );
	else ret = setButtLock( dialog );
	return( ret );
}

int Light::setButtLock( int dialog ) {
	int ret=1;
	debug( "setButtLock:" );
	QString str;
	if( _buttons != 0 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_buttLock,str.data(),
			"Error in setButtLock!", dialog );
		if( ret == 1 ) _buttons = 0;
	}
	return( ret );
}

int Light::setButtUnLock( int dialog ) {
	int ret=1;
	debug( "setUnButtLock:" );
	QString str;
	if( _buttons != 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		ret = set( _module,_buttUnLock,str.data(),
			"Error in setButtUnLock!", dialog );
		if( ret == 1 ) _buttons = 1;
	}
	return( ret );
}

int Light::setIllum( int _il, int dialog ) {
	QString str;
	int _il_;
	unsigned char* respbuf;

	if( (_il < MIN_ILL_VALUE ) || (_il > MAX_ILL_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid illumination value %d outside [%d..%d]!",
			_name,_il,MIN_ILL_VALUE,MAX_ILL_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setIllum, (char)_il,
				str.data(),"Error in setIllum!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getIllum,str.data(),
				"Error in getIllum!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_il_ = (int)respbuf[1]; //(double)respbuf[1]/255.0;
					debug( "_il=%d _il_=%d",_il,_il_ );
					if( _il == _il_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setIllum: %d != %d!",
							_name,_il,_il_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setIllum: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setLight( double _light, int dialog ) {
	QString str;
	int _light_int_;
	unsigned char* respbuf;

	if( (_light < MIN_LIGHT_VALUE ) || (_light > MAX_LIGHT_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid light value %.2f outside [%.2f..%.2f]!",
			_name,_light,MIN_LIGHT_VALUE,MAX_LIGHT_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	int _light_int = doubleTo255int( _light );
	while( 1 ) {
		_currentLight = -1;
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setLight, (char)_light_int,
				str.data(),"Error in setLight!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getLight,str.data(),
				"Error in getLight!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_light_int_ = (int)respbuf[1];
					debug( "_light=%d _light_=%d",_light_int,_light_int_ );
					if( _light_int == _light_int_ ) {
						_currentLight = _light;
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setLight: %d != %d!",
							_name,_light_int,_light_int_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setLight: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setVAmpl( double _dl, int dialog ) {
	QString str;
	unsigned char* respbuf;

	if( (_dl < MIN_VAMPL_VALUE ) || (_dl > MAX_VAMPL_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid vampl value %.2f outside [%.2f..%.2f]!",
			_name,_dl,MIN_VAMPL_VALUE,MAX_VAMPL_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	int _dl_int = doubleTo255int( _dl );

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setVAmpl, (char)_dl_int,
				str.data(),"Error in setVampl!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getVAmpl,str.data(),
				"Error in getVampl!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					int _dl_int_ = (int)respbuf[1];
					debug( "_dl=%d _dl_=%d",_dl_int,_dl_int_ );
					if( _dl_int == _dl_int_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setVampl: %d != %d!",
							_name,_dl_int,_dl_int_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setVampl: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::getBMode( int& _mode, int _dialog ) {
	QString str;
	unsigned char *respbuf;

//	debug( "getBMode:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getBMode, str.data(),
				"Error in getBMode!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !_dialog ) return(0);
					str.sprintf( "Module \"%s\": Error in getBMode! Numb. of ret. par. != 2!", _name );
				} else {
					_mode = (int)respbuf[1];
//					debug( "MODE = %d",_mode );
					if( (_mode < MIN_BMODE ) || (_mode > MAX_BMODE ) ) {
						if( !_dialog ) return(0);
						str.sprintf( "Module \"%s\": Invalid mode: %d outside [%d..%d]!",
						_name,_mode,MIN_BMODE,MAX_BMODE );
					} else {
						_mode = _mode % N_BMODES;
//						debug( "MODE = %d",_mode );
						return(1);
					}
				}
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : continue;
				}
			}
		}
	}
}

int Light::setBMode( int _mode, int dialog ) {
	QString str;
	int _mode_;
	unsigned char* respbuf;

	if( (_mode < MIN_BMODE ) || (_mode > MAX_BMODE ) ) {
		if( !dialog ) return(0);
		str.sprintf( "Module \"%s\": Invalid mode %d outside [%d..%d]!",
			_name,_mode,MIN_BMODE,MAX_BMODE );
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setBMode, (char)_mode,
				str.data(),"Error in setBMode!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getBMode,str.data(),
				"Error in getBMode!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_mode_ = (int)respbuf[1];
					debug( "_mode=%d _mode_=%d",_mode,_mode_ );
					if( _mode == _mode_ ) return(1);
					else {
						str.sprintf( "Module \"%s\": Error in setBMode: %d != %d!",
							_name,_mode,_mode_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setBMode: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::getParam( int& _param, int _dialog ) {
	QString str;
	unsigned char *respbuf;

//	debug( "getParam:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getParam, str.data(),
				"Error in getParam!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !_dialog ) return(0);
					str.sprintf( "Module \"%s\": Numb. of ret. par != 2!", _name );
				} else {
					_param = (int)respbuf[1];
//					debug( "PARAM = %d",_param );
					if( (_param < MIN_PARAM_VALUE ) || (_param > MAX_PARAM_VALUE ) ) {
						if( !_dialog ) return(0);
						str.sprintf( "Module \"%s\": Invalid param: %d outside [%d..%d]!",
						_name,_param,MIN_PARAM_VALUE,MAX_PARAM_VALUE );
					} else return(1);
				}
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : continue;
				}
			}
		}
	}
}

int Light::setParam( int _value, int dialog ) {
	QString str;
	int _value_;
	unsigned char* respbuf;

	if( (_value < MIN_PARAM_VALUE ) || (_value > MAX_PARAM_VALUE ) ) {
		str.sprintf( "Module \"%s\": Invalid param. value %d outside [%d..%d]!",
			_name,_value,MIN_PARAM_VALUE,MAX_PARAM_VALUE );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setParam, (char)_value,
				str.data(),"Error in setParam!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getParam,str.data(),
				"Error in getParam!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_value_ = (int)respbuf[1];
					debug( "_value=%d _value_=%d",_value,_value_ );
					if( _value == _value_ ) return(1);
					else {
						str.sprintf( "Module \"%s\": Error in setParam: %d != %d!",
							_name,_value,_value_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setParam: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Light::setBrightnessUp( int dialog ) {
	int _illum;
	if( _currentBrightness < MAX_BRIGHT_PARAM ) _currentBrightness++;
	else _currentBrightness = MAX_BRIGHT_PARAM;
	if( _currentBrightness == 0 ) _illum = 0;
	else _illum = (int) floor( pow( 2.0,
		(8.0*(double)_currentBrightness / 15.01) ) + 0.5 );
	debug( "BrightnessUp:%d",_illum );
	return( setIllum( _illum,dialog ) );
}

int Light::setBrightnessDown( int dialog ) {
	int _illum;
	if( _currentBrightness > MIN_BRIGHT_PARAM ) _currentBrightness--;
	else _currentBrightness = MIN_BRIGHT_PARAM;
	if( _currentBrightness == 0 ) _illum = 0;
	else _illum = (int) floor( pow( 2.0,
		(8.0*(double)_currentBrightness / 15.01) ) + 0.5 );
	debug( "BrightnessDown:%d",_illum );
	return( setIllum( _illum,dialog ) );
}

int Light::setBrightness( int _brightness, int dialog ) {
	int _illum;
	if( _brightness < MIN_BRIGHT_PARAM ) _brightness = MIN_BRIGHT_PARAM;
	if( _brightness > MAX_BRIGHT_PARAM ) _brightness = MAX_BRIGHT_PARAM;
	_currentBrightness = _brightness;
	if( _currentBrightness == 0 ) _illum = 0;
	else _illum = (int) floor( pow( 2.0,
		(8.0*(double)_currentBrightness / 15.01) ) + 0.5 );
	debug( "setBrightness:%d",_illum );
	return( setIllum( _illum,dialog ) );
}
