#ifndef LIGHT_H
#define LIGHT_H

#include <unistd.h>
#include <math.h>
#include <qstring.h>
#include "tree.h"

#define MIN_ILL_VALUE	0
#define MAX_ILL_VALUE	255
#define MIN_LIGHT_VALUE	0.0
#define MAX_LIGHT_VALUE 1.0
#define MIN_VAMPL_VALUE	0.0
#define MAX_VAMPL_VALUE 1.0
#define MIN_BMODE	0
#define MAX_BMODE	7
#define N_BMODES	4
#define MIN_PARAM_VALUE	0
#define MAX_PARAM_VALUE	255
#define MIN_BRIGHT_PARAM	0
#define MAX_BRIGHT_PARAM	15


// Status of Light:

#define BUTT_LOCK	0x01
#define BMODE		0x02
#define BPLUS		0x04
#define BMINUS		0x08
#define LIGHT_ON	0x10
#define ILLUM_ON	0x20
#define VARY_ON		0x40
#define AUTO		0x80

class Light {
private:
	bool	ready;

	const char* _name;	//
	char _module;		//
	char _setIllum;		//
	char _getIllum;		//
	char _setLight;		//
	char _getLight;		//
	char _getCrc;		//
	char _getStatus;	//
	char _setVAmpl;		//
	char _getVAmpl;		//
	char _setBMode;		//
	char _getBMode;		//
	char _setParam;		//
	char _getParam;		//
	char _illumOn;		//
	char _illumOff;		//
	char _lightOn;		//
	char _lightOff;		//
	char _varyOn;		//
	char _varyOff;		//
	char _buttLock;		//
	char _buttUnLock;	//
	char _getIdent;		//
	int	_nIdent, _ident1,_ident2,_ident3,_ident4;
	char _getConst;		//
	int	_const1,_const2,_const3,_const4;
	char _reset;		//

	int _currentBrightness;
	int _illumination;
	int _buttons;
	double _currentLight;

public:
	Light( void );
	~Light();
	void	loadConst( void );
	bool	isReady( void ) { return( ready ); }

	int	get( char, char, const char*, const char*, unsigned  char**, int );
	int	set( char, char, const char*, const char*,int );
	int	set( char, char, char, const char*, const char*,int );
	int	set( char, char, char, char, const char*, const char*,int );

	int reset( void );
	int getIdent( void );
	int getConst( void );
	int testCrc( void );
	int getStatus( uchar& );
	int turnIllum( const QString&,int );
	int turnIllum( int,int );
	int setIllumOn( int );
	int setIllumOff( int );
	int turnLight( int, int );
	int setLightOn( int );
	int setLightOff( int );
	int setVaryOn( int );
	int setVaryOff( int );
	int turnButtons( const QString&,int );
	int turnButtons( int, int );
	int setButtLock( int );
	int setButtUnLock( int );
	int setGetIdent( void );
	int setIllum( int,int );
	int setLight( double,int );
	int setVAmpl( double,int );
	int getBMode( int&,int );
	int setBMode( int,int );
	int getParam( int&,int );
	int setParam( int,int );

	int setBrightness( int,int );
	int setBrightnessUp( int );
	int setBrightnessDown( int );

	int isIllumOn( void ) { return(_illumination); }
	int isButtonsOn( void ) { return(_buttons); }
	int getCurrentBrightness( void ) { return( _currentBrightness ); }
	double getLight( void ) { return( _currentLight ); }

	int testButtons( void );
	int testIllum( void );
};

#endif
