#define QT_THREAD_SUPPORT

#include <qapplication.h>
#include <qthread.h>
#include <qfont.h> 
#include <qdatetime.h>
#include "names.h"
#include "device.h"
#include "tools.h"
#include "mywidget.h"
#include "deeptree.h"
#include "maptree.h"
#include "graph.h"
#include "jkexport.h"

class DataThread : public QThread {
public:
	virtual void run();
};

class CountThread : public QThread {
public:
	virtual void run();
};

void DataThread::run() { 
    read_data();
}

void CountThread::run( void ) {
    count_cycle();
}

bool		ExportText::isChanged = false;
QString		ExportText::text;

bool		ExportTable::clear=false;
bool		ExportTable::adjust=false;
bool		ExportTable::isChanged=false;
int		ExportTable::startRow = 0;
int		ExportTable::nRows = MAX_ROW;
int		ExportTable::startCol = 0;
int		ExportTable::nCols = MAX_COL;
QString		ExportTable::table[MAX_ROW][MAX_COL];

bool		StateExport::isStartCounting = false;
bool		StateExport::isEndCounting = false;
bool		StateExport::isChanged = false;
//bool		StateExport::isModeChanged = false;
DeviceState	StateExport::state;
bool		StateExport::isCorrectionReady = false;

bool		InfoExport::isMessage = false;
QString		InfoExport::message;
int		InfoExport::ms;	// skol'ko emu svetit'sia, ms
bool		InfoExport::isProgressChanged = false;
Progress	InfoExport::progress;
bool		InfoExport::isModeChanged = false;
Progress	InfoExport::mode;
bool		InfoExport::isScenarioChanged = false;
bool		InfoExport::isScenario = false;
Progress	InfoExport::scenario;

QMutex countMutex;
QMutex dataMutex;
QMutex scenarioMutex;
QMutex stateExportMutex;
QMutex infoExportMutex;

DevManager	devmanager;
Star		star;
ExportTable	table;
FILE		*out_file=NULL;

CFGLeaf	leaf_empty;
CFGBranch branch_empty;
Link link_empty_leaf;

DeepTree	deeptree;
CFGTree		*tree;		//  
MapTree		maptree_object;
CFGTree		*maptree;
double		_tmZone = 0;
QTime		_programStartTime;

int main( int argc, char ** argv ) {
	DataThread dataThread;
	CountThread countThread;

	initThreads();

	debug( "===== data: try lock..." );
	dataMutex.lock();
	dataThread.start();
	debug( "===== count: try lock..." );
	countMutex.lock();
	debug( "===== count: locked..." );
	countThread.start();

//	----- Tree:

	leaf_empty.initLeaf("Empty");
	branch_empty.initBranch("Empty");
	link_empty_leaf.initLink('l',&leaf_empty,&branch_empty);

	tree = deeptree.linkTree();	//		  
	maptree = maptree_object.linkMapTree();

	QApplication qapp( argc, argv );
//	qapp.setFont( QFont("Helvetica",18) );

	if( tree->readCfg( cfgFile ) ) myQAppExit(0);
	int _fsize = tree->findBranch( "Preferences" )->
		findBranch( "Text window settings" ) ->
		findLeaf( "FontSizeLarge" )->getData()->toInt();
	qapp.setFont( QFont("Helvetica",_fsize) );
	if( maptree->readCfg( deviceCfgFile ) ) myQAppExit(0);

	devmanager.create();
	devmanager.prepareFiles();

	star.loadConst();
	star.init();

	devmanager.loadConst();

	MyWidget w;
	qapp.setMainWidget( &w );
	GraphWindow gw;
	gw.show();
	w.show();

	debug( " gw.show(), w.show()" );

	devmanager.init();

	devmanager.initialTests();
	debug( "devmanager.initialTests" );

	return qapp.exec();
}
