
#include "motor.h"
#include "devplus.h"
#include "tools.h"

extern CFGTree* tree;

QString _undefinedAperture = "Undefined";

Motor::Motor( void ) {
	_name = "Apertures";
	_currentAperture = -1;
	_currentDoor = -1;
	debug( "CurrentDoor=%d",_currentDoor );
	ready = FALSE;
}

Motor::~Motor() {
}

void Motor::loadConst( void ) {
	debug( "motor: LoadConst" );
	_module = readCharConst( _name,"Common","Module" );

	_getStatus = readCharConst( _name,"Command","GetStatus" );
	_getSensors = readCharConst( _name,"Command","GetSensors" );
	_getCrc = readCharConst( _name,"Command","GetCrc" );
	_getPosition = readCharConst( _name,"Command","GetPosition" );

	_moveTo = readCharConst( _name,"Command","MoveTo" );
	_shiftAt = readCharConst( _name,"Command","ShiftAt" );
	_synchAt = readCharConst( _name,"Command","SynchAt" );
	_stopMotion = readCharConst( _name,"Command","StopMotion" );
	_loadStops = readCharConst( _name,"Command","LoadStops" );
	_getStops = readCharConst( _name,"Command","GetStops" );
	_setAperture = readCharConst( _name,"Command","SetAperture" );
	_getAperture = readCharConst( _name,"Command","GetAperture" );
	_setStep = readCharConst( _name,"Command","SetStep" );
	_getStep = readCharConst( _name,"Command","GetStep" );
	_testMotion = readCharConst( _name,"Command","TestMotion" );
	_nullSet = readCharConst( _name,"Command","NullSet" );
	_openDoor = readCharConst( _name,"Command","OpenDoor" );
	_closeDoor = readCharConst( _name,"Command","CloseDoor" );
	_powerDown = readCharConst( _name,"Command","PowerDown" );
	_getIdent = readCharConst( _name,"Command","GetIdent" );
	_getConst = readCharConst( _name,"Command","GetConst" );
	_reset = readCharConst( _name,"Command","Reset" );

	 const char* _id = readStrConst( _name,"Common","Identification" );
	_nIdent = sscanf( _id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	_nStops = 0;

	_stop[_nStops] = 1;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop1",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 2;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop2",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 3;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop3",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 4;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop4",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 5;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop5",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 6;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop6",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 7;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop7",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	_stop[_nStops] = 8;
	_stopCorrect[_nStops] = (char)leafDataToCharInt( "turbina.cfg","Devices",
		_name,"Stop8",_stopName[_nStops],_stopFoclen[_nStops] );
	if( operator!=(_stopName[_nStops].upper(),"SPARE") ) _nStops++;

	debug( "_nStops=%d,Stop3=%s focus[Stop3]=%f",
				_nStops,_stopName[2].data(),_stopFoclen[2] );
	ready = TRUE;
}

int Motor::get( char _module, char _command, const char* _name,
		const char* _warning, unsigned char** p_respbuf, int dialog ) {
	QString str;
	int i;
	int ret;
	int resp;

	for( i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = sendCommand( _module,_command,p_respbuf );
		if( resp == 1 ) return(1);	// data
		jkwait( DEVICE_WAIT_INTERVAL );
	}
	if( (*p_respbuf)[0] == DSC ) {
		str.sprintf( "%s disconnected!",_name );
		ret = -1;
	} else {
		str.sprintf( "%s: %s",_name,_warning );
		ret = -2;
	}
	if( !dialog ) return(ret);
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(ret);
		case  QMessageBox::Ignore: return(ret);
		case  QMessageBox::Retry : return(0);
		default : return(0);
	}
}

int Motor::set( char _module, char _command, const char* _name,
		const char* _warning, int _dialog ) {
	QString str;
	int resp;
	int i;
	unsigned char *respbuf;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Motor::set( char _module, char _command, char _data, const char* _name,
			const char* _warning, int _dialog ) {
	QString str;
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,_data,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}	
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}

int Motor::set( char _module, char _command, char _data_low, char _data_high,
			const char* _name, const char* _warning, int _dialog ) {
	QString str;
	int resp;
	unsigned char *respbuf;
	int i;

	while( 1 ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module,_command,_data_low,_data_high,&respbuf );
			if( respbuf[0] == ACY ) return(1);
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		if( !_dialog ) return(0); 	
		if( respbuf[0] == DSC ) {
			str.sprintf( "%s disconnected!",_name );
		} else {
			str.sprintf( "%s: %s",_name,_warning );
		}	
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1); 
			case  QMessageBox::Ignore: return(0); 
			case  QMessageBox::Retry : continue; 
		}
	}
}


int Motor::getIdent( void ) {
	QString str;
	unsigned char *respbuf;

	debug( "Motor::getIdent:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getIdent, str.data(),
			"Error in getIdent!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
			debug( "Motor: %X %X %X %X",respbuf[1],respbuf[2],
					respbuf[3],respbuf[4] );	
			if( (respbuf[0] != _nIdent+1 ) ||
				(_ident1 != respbuf[1]) || (_ident2 != respbuf[2]) ||
				(_ident3 != respbuf[3]) || (_ident4 != respbuf[4]) ) {
					str.sprintf( "Module \"%s\": Invalid identification: %X %X %X %X!",
						_name,respbuf[1],respbuf[2],respbuf[3],respbuf[4] );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);

			}
		}
	}
}

int Motor::reset( void ) {
	QString str;
	int resp;
	unsigned char* respbuf;
	int i;
	int ret=0;

	debug( "Motor::reset:" );
	int cycle = 1;
	_currentAperture = -1;
	_currentDoor = -1;
	while( cycle ) {
		for( i=0; i<N_REPEAT_COMMAND; i++ ) {
			resp = sendCommand( _module, _reset, &respbuf );
			if( resp == -2 ) {	// disconnect
				return(1);
			}
		}
		str.sprintf( "Module \"%s\": Error in Reset!",_name );
		switch ( showWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: cycle=0; ret=0; break;
			case  QMessageBox::Retry : break;
		}
	}
	return( ret );
}

int Motor::getConst( void ) {
	QString str;
	unsigned char *respbuf;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getConst, str.data(),
				"Error in getConst!",&respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 5 ) {
					_const1 = (int)respbuf[1];
					_const2 = (int)respbuf[2];
					_const3 = (int)respbuf[3];
					_const4 = (int)respbuf[4];
					return(1);
				}
				str.sprintf( "Module \"%s\": Error in getConst!",_name );
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::testCrc( void ) {
	QString str;
	unsigned char *respbuf;

	debug( "testCrc:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getCrc, str.data(),
				"Error in getCrc!", &respbuf,1 ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( (respbuf[0] != 2) || (respbuf[1] != 0) ){
					str.sprintf( "Module \"%s\": EEPROM corrupted!",_name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : break;
					}
				} else return(1);
			}
		}
	}
}

int Motor::getStatus( uchar& _status, int dialog ) {
	QString str;
	unsigned char *respbuf;

	debug( "getStatus:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getStatus, str.data(),
				"Error in getStatus!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid status!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { _status = respbuf[1]; return(1); }
			}
		}
	}
}

int Motor::getSensors( uchar& _status, int dialog ) {
	QString str;
	unsigned char *respbuf;

	debug( "getSensors:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getSensors, str.data(),
				"Error in getSensors!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid status of sensors!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { _status = respbuf[1]; return(1); }
			}
		}
	}
}

int Motor::getPosition( int& _position, int dialog ) {
	QString str;
	unsigned char *respbuf;
	uchar _low;
	uchar _high;

	debug( "getPosition:" );
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		switch ( get( _module, _getPosition , str.data(),
				"Error in getPosition!",&respbuf,dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 3 ) {
					if( !dialog ) return(0);
					str.sprintf( "Module \"%s\": Invalid Position!", _name );
					switch ( showWarning( str ) ) {
						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Retry : continue;
					}
				} else  { 
					_low = respbuf[1];
					_high = respbuf[2];
					_position = (int)((_high << 8)+_low);
					return(1);
				}
			}
		}
	}
}

int Motor::setStep( uchar _step, int dialog ) {
	QString str;
	uchar _step_;
	unsigned char* respbuf;

	if( (_step < MIN_STEP ) || (_step > MAX_STEP ) ) {
		str.sprintf( "Module \"%s\": Invalid step value %d outside [%d..%d]!",
			_name,_step,MIN_STEP,MAX_STEP );
		if( !dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setStep, _step,
				str.data(),"Error in setStep!", dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getStep,str.data(),
				"Error in getStep!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == 2 ) {
					_step_ = respbuf[1];
					debug( "_step=%d _step_=%d",_step,_step_ );
					if( _step == _step_ ) {
						return(1);
					} else {
						str.sprintf( "Module \"%s\": Error in setStep: %d != %d!",
							_name,_step,_step_ );
					}
				} else {
					str.sprintf( "Module \"%s\": Error in setStep: Numb. of ret. val. != 2!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::loadStops( int dialog ) {
	QString str;
	char _stopCorrect_[MAX_N_STOPS];
	unsigned char* respbuf;
	int ret;
	char i;
	int j,_nApp;
	int abort = 0;

	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		for( i=0; i<_nStops; i++ ) {
			ret = set( _module,_loadStops,_stop[i],_stopCorrect[i],
				str.data(),"Error in loadStops!", dialog );
			if( ret == 0 ) return(0);
		}

		switch( get( _module,_getStops,str.data(),
				"Error in getStops!",&respbuf, dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] == MAX_N_STOPS+1 ) {
					for( i=0; i<MAX_N_STOPS; i++ ) {
						_nApp = -1;
						for( j=0; j<_nStops; j++ ) {
							if( i == _stop[j] ) {
								_nApp = j;
								break;
							}
						}
						if( _nApp > -1 ) {
							_stopCorrect_[_nApp] = respbuf[i+1];
							if( _stopCorrect[_nApp] != _stopCorrect_[_nApp] ) {
								str.sprintf( "Module \"%s\": Error in loadStops, aperture N %d: %d != %d!",
								_name,i,_stopCorrect[_nApp],_stopCorrect_[_nApp] );
								abort=1;
								break;
							}
						}
					}
					if( !abort ) return(1);
				} else {
					str.sprintf( "Module \"%s\": Error in loadStops: Numb. of ret. val. != MAX_N_STOPS+1!",
						_name );
				}
				if( !dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
}

int Motor::getApertureNumber( const QString& _apName, int dialog ) {
	QString str;
	int i;

	for( i=0; i<_nStops; i++ ) {
		if( operator==( _apName.upper(),_stopName[i].upper() ) ) {
			return( i );
		}
	}
	str.sprintf( "Module \"%s\": Invalid aperture %s!",
			_name,_apName.data() );
	if( !dialog ) return(-1);
	switch ( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(-1);
		default : return(-1);
	}
}


double Motor::getFoclen( const QString& _apName,int _dialog ) {
	QString str;
	int _aperture;

	_aperture = getApertureNumber( _apName,_dialog );
	if( _aperture >= 0 ) return( getFoclen(_aperture ) );
	return(0);
}

double Motor::getFoclen( int _aperture, int _dialog=0 ) {
	if( (_aperture < 0 ) || (_aperture >= _nStops ) ) {
		return( 0 );
	}
	return( _stopFoclen[ _aperture ] );
}

int Motor::setAperture( const QString& _apName, int _dialog,
			bool _hard=FALSE ) {
	QString str;
	int _aperture;

	_aperture = getApertureNumber( _apName,_dialog );
	if( _aperture >= 0 ) return( setAperture(_aperture,_dialog,_hard) );
	return(0);
}

int Motor::setAperture( int _aperture, int _dialog,
			bool _hard=FALSE ) {
	QString str;
	unsigned char* respbuf;
	char _stop_;
	int i;

//	debug( "setAperture: Stop%d",_stop[_aperture] );
	if( !_hard ) { if( _aperture == _currentAperture ) return(1); }
	if( (_aperture < 0 ) || (_aperture >= _nStops ) ) {
		str.sprintf( "Module \"%s\": Invalid aperture number %d outside [0..%d]!",
			_name,_aperture,_nStops-1 );
		if( !_dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	_currentAperture = -1;
RETRY:
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setAperture, _stop[_aperture],
				str.data(),"Error in setAperture!", _dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getAperture,str.data(),
				"Error in getAperture!",&respbuf, _dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Error in getAperture: Numb. of ret. val. != 2!",
						_name );
				} else {
					_stop_ = (char)respbuf[1];
					if( _stop[_aperture] == _stop_ ) goto CONT;
 					else {
						str.sprintf( "Module \"%s\": Error in setAperture: %d != %d!",
						_name,_stop[_aperture],_stop_ );
					}
				}
				if( !_dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
CONT:		
	int iii;
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		iii = testMotion( _dialog );
//		debug( "TestMotion ret: %d",iii );
		switch( iii ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				_currentAperture = _aperture;
				return(1);
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
//		debug( "sleep" );
	}
	stopMotion( _dialog );
	if( !_dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::setStop( int _absolute_stop_number, int _dialog ) {
	QString str;
	unsigned char* respbuf;
	int _stop_;
	int i;

	debug( "setStop: Stop%d",_absolute_stop_number );
	if( (_absolute_stop_number < 0 ) || (_absolute_stop_number >= MAX_N_STOPS ) ) {
		str.sprintf( "Module \"%s\": Invalid absolute stop number %d outside [0..%d]!",
			_name,_absolute_stop_number,MAX_N_STOPS-1 );
		if( !_dialog ) return(0);
		switch ( showStrongWarning( str ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: return(0);
			default : return(0);
		}
	}

	_currentAperture = -1;
RETRY:
	while( 1 ) {
		str.sprintf( "Module \"%s\"",_name );
		if( set( _module,_setAperture, (char)_absolute_stop_number,
				str.data(),"Error in setStop!", _dialog ) == 0 ) {
			return(0);
		}
		switch( get( _module,_getAperture,str.data(),
				"Error in getAperture!",&respbuf, _dialog ) ) {
			case  0 : continue;
			case -1 : return(-1);
			case -2 : return(-2);
			case  1 : {
				if( respbuf[0] != 2 ) {
					str.sprintf( "Module \"%s\": Error in getAperture: Numb. of ret. val. != 2!",
						_name );
				} else {
					_stop_ = (int)respbuf[1];
					if( _stop_ == _absolute_stop_number ) goto CONT;
 					else {
						str.sprintf( "Module \"%s\": Error in setStop: %d != %d!",
						_name,_absolute_stop_number,_stop_ );
					}
				}
				if( !_dialog ) return(0);
				switch ( showWarning( str ) ) {
					case  QMessageBox::Abort : myQAppExit(1);
					case  QMessageBox::Ignore: return(0);
					case  QMessageBox::Retry : break;
				}
			}
		}
	}
CONT:
	int iii,j;
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		iii = testMotion( _dialog );
//		debug( "TestMotion ret: %d",iii );
		switch( iii ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				for( j=0; j<_nStops; j++ ) {
					if( _stop_ == _stop[j] ) {
	 					_currentAperture = j;
						break;
					}
				}
				debug( "Current Aperture = %d",_currentAperture );
				return(1);
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
//		debug( "sleep" );
	}
	stopMotion( _dialog );
	if( !_dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::moveTo( int _position, int dialog ) {
	QString str;
	int _high,_low;
	int _position_;
	int i;
	int ret;

	debug( "moveTo:%d",_position );
	_high = _position >> 8;
	_low = _position - ( _high << 8 );

RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_moveTo, _low, _high,
			str.data(),"Error in moveTo!", dialog ) == 0 ) {
		return(0);
	}
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				goto CONT;
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
//		debug( "sleep" );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);

CONT:
	ret = getPosition( _position_,dialog );
	if( ret == 1 ) {
		if( _position_ == _position ) return(1);
		else {
			if( !dialog ) return(0);
			str.sprintf( "Module \"%s\": Error in moveTo: %d != %d",
				_name,_position,_position_ );
			switch ( showWarning( str ) ) {
				case	QMessageBox::Abort : myQAppExit(1);
				case	QMessageBox::Ignore: return(0);
				case	QMessageBox::Retry : goto RETRY;
			}
		}
	}
	return(0);
}

int Motor::synchAt( char _shift, int dialog ) {
	QString str;

	debug( "synchAt:%d",_shift );

	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_synchAt, _shift,
			str.data(),"Error in synchAt!", dialog ) == 0 ) {
		return(0);
	}
	return(1);
}

int Motor::shiftAt( char _shift, int dialog ) {
	QString str;
	int i;

	debug( "shiftAt:%d",_shift );

RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_shiftAt, _shift,
			str.data(),"Error in shiftAt!", dialog ) == 0 ) {
		return(0);
	}
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : return(1);
		}
		jkwait( DEVICE_WAIT_INTERVAL );
//		debug( "sleep" );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::testMotion( int dialog ) {
	QString str;
	int resp;
	int i;
	int ret;
	unsigned char *respbuf;

//	debug( "TestMotion:" );
	for( i=0; i<N_REPEAT_COMMAND; i++ ) {
		resp = sendCommand( _module,_testMotion,&respbuf, FALSE );
		if( respbuf[0] == ACY ) return(1);
		if( respbuf[0] == ACW ) return(2);
		jkwait( DEVICE_WAIT_INTERVAL );
	}
	if( respbuf[0] == DSC ) {
		str.sprintf( "%s disconnected!",_name );
		ret = -1;
	} else {
		str.sprintf( "Module %s: Error in testMotion!",_name );
		ret = -2;
	}
	if( !dialog ) return( ret ); 	
	switch ( showWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1); 
		case  QMessageBox::Ignore: return(ret); 
		case  QMessageBox::Retry : return(0); 
	}
	return(0);
}

int Motor::turnDoor( const QString& _open, int dialog ) {
	int _on = 0;
	
	if( operator==(_open.upper(),"OPEN" ) ) _on=1;
	if( operator==(_open.upper(),"CLOSE" ) ) _on=0;
	debug( "Motor::turnDoor %d",_on );
	return( turnDoor( _on,dialog ) );
}

int Motor::turnDoor( int _on, int _dialog ) {
	debug( "Motor::turnDoor %d",_on );
	if( _on == ON ) return( openDoor( _dialog ) );
	if( _on == OFF ) return( closeDoor( _dialog ) );
	if( !_dialog ) return(0);
	QString str = "Invalid door parametr!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default: return(0);
	}
}

int Motor::openDoor( int dialog ) {
	QString str;
	int iii,i;

	debug( "openDoor:" );
	debug( "CurrentDoor=%d",_currentDoor );
	if( _currentDoor == OPEN ) return(1);
	_currentDoor = -1;
	if( testDoor( dialog ) == OPEN ) { _currentDoor=OPEN; return(1); }
	_currentAperture = -1;
RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_openDoor, str.data(),
		"Error in openDoor!", dialog ) == 0 ) {
		return(0);
	}
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		iii = testMotion( dialog );
		switch( iii ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				_currentDoor = testDoor( dialog );
				debug( "CurrentDoor=%d",_currentDoor );
				if( _currentDoor == OPEN ) { return(1); }
				else {
					if( !dialog ) return(0);
					str = "Input Door is not opened!";
					switch ( showStrongWarning( str,"Ignore","Retry" ) ) {
//						case  QMessageBox::Abort : myQAppExit(1);
						case  QMessageBox::Ignore: return(0); 
						case  QMessageBox::Abort : goto RETRY; 
					}
				}
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::closeDoor( int dialog ) {
	QString str;
	int i;

	debug( "closeDoor:" );
	debug( "CurrentDoor=%d",_currentDoor );
	if( _currentDoor == CLOSE ) return(1);
	_currentDoor = -1;
	if( testDoor( dialog ) == CLOSE ) { _currentDoor = CLOSE; return(1); }
	_currentAperture = -1;
RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_closeDoor, str.data(),
		"Error in closeDoor!", dialog ) == 0 ) {
		return(0);
	}
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
				_currentDoor = testDoor( dialog );
				debug( "CurrentDoor=%d",_currentDoor );
				if( _currentDoor == CLOSE ) { return(1); }
				else {
					if( !dialog ) return(0);
					str = "Input Door is not closed!";
					switch ( showStrongWarning( str,"Ignore","Retry" ) ) {
//						case  QMessageBox::Abort : myQAppExit(1); 
						case  QMessageBox::Ignore: return(0);
						case  QMessageBox::Abort : goto RETRY;
					}
				}
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::testDoor( int dialog ) {
	uchar _status;
	int ret;
	ret = getSensors( _status,dialog );
	if( ret != 1 ) return( INDEFINITE );
	if( _status & DOOR ) return( CLOSE );
	else return( OPEN );
}

int Motor::stopMotion( int dialog ) {
	debug( "stopMotion:" );
	QString str;
	str.sprintf( "Module \"%s\"",_name );
	return( set( _module,_stopMotion,str.data(),
		"Error in stopMotion!",dialog ) );
}

int Motor::testInitStatus( int dialog ) {
	QString str;
	uchar _status;
	int ret = 1;

	while( 1 ) {
		if( getStatus( _status,dialog ) != 1 ) return( INDEFINITE );
		_currentDoor = CLOSE;
		if( !(_status & DOOR_CLOSE) ){
			_currentDoor = OPEN;
			if( !dialog ) {ret=0; goto CONT; }
			str = "Input Door is not closed!";
			switch ( showWarning( str ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: ret=0; break;
				case  QMessageBox::Retry : continue; 
			}
		}
CONT:
		if( !(_status & ZERO_POINT) ){
			if( !dialog ) return(0);
			str = "Null-Point in not defined!";
			switch ( showWarning( str ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: ret=0; break; 
				case  QMessageBox::Retry : continue; 
			}
		}
		debug( "PPPPPPP Test Init !!!!!!!!!!!!!!!!!!!!!!! CurrentDoor=%d",_currentDoor );
		return(ret);
	}
}

int Motor::nullSet( int dialog ) {
	QString str;
	int i;

	debug( "nullSet:" );
	_currentAperture = -1;
	_currentDoor = -1;
RETRY:
	str.sprintf( "Module \"%s\"",_name );
	if( set( _module,_nullSet, str.data(),
		"Error in nullSet!", dialog ) == 0 ) {
		return(0);
	}
	for( i=0; i<MAX_N_INTERVALS; i++ ) {
		switch( testMotion( dialog ) ) {
			case  0 : goto RETRY;
			case -1 : return(-1);
			case -2 : return(-2);
			case  2 : break;
			case  1 : {
//				iii = testNull( dialog );
				return(1);
			}
		}
		jkwait( DEVICE_WAIT_INTERVAL );
//		debug( "sleep" );
	}
	stopMotion( dialog );
	if( !dialog ) return(0);
	str = "Infinite Motion!";
	switch( showStrongWarning( str ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
	return(0);
}

int Motor::movePlus( int dialog ) {
	char _shift = 1;
	debug( "Move Plus:" );
	if( _currentAperture < 0 ) return(0);
//	int _position;
//	int ret = getPosition( _position,dialog );
//	if( ret != 1 ) return(0);
//	debug( "_position=%d",_position );
//	if( _position >= MAX_POSITION ) return(0);
	if( _stopCorrect_new[_currentAperture] >= MAX_CORRECTION ) return(0);
	(_stopCorrect_new[_currentAperture])++;
	debug( "StopCorrect:%d",_stopCorrect_new[_currentAperture] );
	return( shiftAt( _shift,dialog ) );
}

int Motor::moveMinus( int dialog ) {
	char _shift = -1;
	debug( "Move Minus:" );
	if( _currentAperture < 0 ) return(0);
//	int _position;
//	int ret = getPosition( _position,dialog );
//	if( ret != 1 ) return(0);
//	debug( "_position=%d",_position );
//	if( _position <= MIN_POSITION ) return(0);
	if( _stopCorrect_new[_currentAperture] <= MIN_CORRECTION ) return(0);
	(_stopCorrect_new[_currentAperture])--;
	debug( "StopCorrect:%d",_stopCorrect_new[_currentAperture] );
	return( shiftAt( _shift,dialog ) );
}

int Motor::shiftAperture( int _shift, int dialog ) {
	debug( "Shift Aperture: %d", _shift );
	int _position;
	int ret = getPosition( _position,dialog );
	if( ret != 1 ) return(0);
	debug( "_position=%d",_position );
	_currentAperture = -1;
	return( moveTo( _position+_shift,dialog ) );
}

QString Motor::getStopName( int _stopNumber ) {
	if( (_stopNumber < 0 ) || (_stopNumber >= _nStops ) ) {
		return( _undefinedAperture );
	}
	return( _stopName[_stopNumber] );
}

int Motor::getStopCorrect( void ) {
	debug( "Motor: currentApert = %d",_currentAperture ); 
	return( (int)_stopCorrect[ _currentAperture ] );
}

int Motor::getStopCorrectNew( void ) {
	debug( "Motor: currentApert = %d",_currentAperture ); 
	return( (int)_stopCorrect_new[ _currentAperture ] );
}

void Motor::startCorrection( void ) {
	int i;
	debug( "Motor: Start Correction" );
	for( i=0; i<_nStops; i++ ) {
		_stopCorrect_new[i] = _stopCorrect[i];
	}
}

void Motor::acceptCorrection( int _dialog ) {
	int i;
	debug( "Motor: Accept Correction" );
	for( i=0; i<_nStops; i++ ) {
		_stopCorrect[i] = _stopCorrect_new[i];
	}
	loadStops( _dialog );

// Copy to tree:
	QString _str,_num,_stopLeaf;
	CFGLeaf *leaf;
	for( i=0; i<_nStops; i++ ) {
//		_str = "";
		_str = _stopName[i];
		_str.append( "\t" );
		_num.setNum( _stopFoclen[i],'f',2 );
		_str.append( _num );
		_str.append( " " );
		_num.setNum( _stopCorrect[i] );
		_str.append( _num );
		_stopLeaf = "Stop";
		_num.setNum( _stop[i] );
//		_num.setNum( _stop[i]+1 );
		_stopLeaf.append( _num );
//	debug( "_nStops=%d,Stop3=%s focus[Stop3]=%f",
//				_nStops,_stopName[2].data(),_stopFoclen[2] );
// lll
		debug( " ________ _str = %s leaf=%s",_str.data(),
			_stopLeaf.data() );
		leaf = tree->findBranch( "Devices" )->findBranch( _name )
			->findLeaf( _stopLeaf.data() );
		if( leaf != &leaf_empty ) {
			leaf->replaceData( _str );
			tree->setChanged();
		}
	}
}
