#include "mywidget.h"
#include "device.h"
#include "devplus.h"
#include "tools.h"
#include <qpixmap.h>
#include "diaphragm.xpm"
#include "buttons.xpm"

extern DevManager	devmanager;
extern Star		star;
extern CFGTree*		tree;
extern QMutex		scenarioMutex;
extern QMutex		infoExportMutex;
extern QMutex		stateExportMutex;

static const char* Undefin	= "?";
static const char* Open		= "Open";
static const char* Close	= "Close";
static const char* On		= "On";
static const char* Off		= "Off";

// ToolTips:
static const char* diaphragmText	= "Star in diaphragm";
//static const char* groupButtText 	= "Main buttons";
static const char* stopText 		= "Stop Scenario";
static const char* pauseText		= "Pause";
static const char* playText			= "Press to run scenario";
static const char* navigateText		= "Press to show star info";
static const char* selectText		= "Press to select new star";  
static const char* apertText		= "Current Aperture";
static const char* illumText		= "Illumination";
static const char* doorText			= "Input Door";
static const char* hvText			= "High Voltage";
static const char* tmprText			= "Temperature, C";
static const char* starText			= "Observed Star";
static const char* zenitText		= "Air Mass";
//static const char* haText			= "Hour Angle";
//static const char* raText			= "RA";
//static const char* decText			= "Declination";
//static const char* magText			= "V-magnitude";
//static const char* spText				= "Spectrum";
//static const char* duplText			= "Duplicity";
static const char* scenarioText		= "Current Scenario";
static const char* modeText			= "Current Mode";
static const char* modeTimeText		= "Mode Remained Time (sec)";
static const char* scenarioTimeText	= "Scenario Remained Time (sec)";
static const char* clockText		= "Local Time";

DataArray curve_x(100);
DataArray curve_y(100);

DataArray::DataArray( int _size ) : QArray<double>( _size ) {
	_number = -1;
}

DataArray::~DataArray() {
}

int DataArray::put( double _data ) {
	_number++;
	if( (int)size() < _number+1 ) resize( size()+10 );
	at( _number ) = _data;
	return( _number );
}

double DataArray::get( int i ) {
	return at(i);
}

void DataArray::searchMinMax( void ) {
	uint i;

	_min = at(0);
	_max = at(0);
	for( i=1; i<(uint)_number; i++ ) {
		if( at(i) < _min ) _min = at(i);
		if( at(i) > _max ) _max = at(i);
	}
	if( _min == _max ) _max = _min+1;
}

MyWidget::MyWidget( QWidget *parent, const char *name )
					: QMainWindow( parent, name )	{
	isTimerEventLocked = false;
	isScenario = false;
	existTalkDialog = 0;
	existAdjustDialog = 0;
	existIllumDialog = 0;
	existSelectStarDialog = 0;
	existShowStarDialog = 0;
	existBackCorrectionDialog = 0;
	existCenterCorrectionDialog = 0;
	_updateHA = 1;
	mainView = NULL;

	mainView = new QJKMainView( this,"MainView" );
	CHECK_PTR( mainView );

	debug( "LoadConst:" );
	loadConst();

	mainView->setFont( QFont("Helvetica",_fontSizeSmall),TRUE );
	mainView->fitToFont();
	setCentralWidget( mainView );



	progressor = new QProgressBar( 100,statusBar() );
	CHECK_PTR( progressor );
	statusBar()->addWidget( progressor,0,TRUE );
	progressor->hide();

	toolBarDevice = new QToolBar( this );
	CHECK_PTR( toolBarDevice );
	addToolBar( toolBarDevice,"Device status",Bottom,TRUE );
	toolBarDevice->setHorizontalStretchable( TRUE );

	toolBarObject = new QToolBar( this );
	CHECK_PTR( toolBarObject );
	addToolBar( toolBarObject,"Object status",Top,TRUE );
	toolBarObject->setHorizontalStretchable( TRUE );

	toolBarProgram = new QToolBar( this );
	CHECK_PTR( toolBarProgram );
	addToolBar( toolBarProgram,"Program status",Top,TRUE );
	toolBarProgram->setHorizontalStretchable( TRUE );

	toolBarButtons = new QToolBar( this );
	CHECK_PTR( toolBarButtons );
	addToolBar( toolBarButtons,"Program status",Left,TRUE );
	toolBarButtons->setHorizontalStretchable( TRUE );

	labelApert = new QLabel( "WorkAperture",toolBarDevice );
	CHECK_PTR( labelApert );
	labelApert->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelApert->setAlignment( AlignCenter );
	_labelApertSize = labelApert->sizeHint();
	labelApert->setFixedSize(_labelApertSize );
	QToolTip::add( labelApert,apertText );

	labelDoor = new QLabel( "Close",toolBarDevice );
	CHECK_PTR( labelDoor );
	labelDoor->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelDoor->setAlignment( AlignCenter );
	labelDoor->setFixedSize( labelDoor->sizeHint() );
	QToolTip::add( labelDoor,doorText );

	labelIllum = new QLabel( "Off",toolBarDevice );
	CHECK_PTR( labelIllum );
	labelIllum->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelIllum->setAlignment( AlignCenter );
	labelIllum->setFixedSize( labelIllum->sizeHint() );
	QToolTip::add( labelIllum,illumText );

	labelHV = new QLabel( "Off",toolBarDevice );
	CHECK_PTR( labelHV );
	labelHV->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelHV->setAlignment( AlignCenter );
	labelHV->setFixedSize( labelHV->sizeHint() );
	QToolTip::add( labelHV,hvText );

	labelTmpr = new QLabel( "-99.9",toolBarDevice );
	CHECK_PTR( labelTmpr );
	labelTmpr->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelTmpr->setAlignment( AlignCenter );
	labelTmpr->setFixedSize( labelTmpr->sizeHint() );
	QToolTip::add( labelTmpr,tmprText );
	toolBarDevice->setStretchableWidget( labelTmpr );

	toolBarDevice->addSeparator();

	labelDiaphragm = new QLabel( toolBarObject );
	CHECK_PTR( labelDiaphragm );
	labelDiaphragm->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelDiaphragm->setPixmap( cc );
	labelDiaphragm->setAlignment( AlignCenter );
	QToolTip::add( labelDiaphragm, diaphragmText );

	labelStar = new QLabel( "  Alpha Lyr  ",toolBarObject );
	CHECK_PTR( labelStar );
	labelStar->setText( star.getName() );
	labelStar->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelStar->setAlignment( AlignCenter );
	labelStar->setFixedSize( labelStar->sizeHint() );
	QToolTip::add( labelStar,starText );
	toolBarObject->setStretchableWidget(labelStar);

	labelZenit = new QLabel( "99.99",toolBarObject );
	CHECK_PTR( labelZenit );
	labelZenit->setText( star.getAirMass() );
	labelZenit->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelZenit->setAlignment( AlignCenter );
	labelZenit->setFixedSize( labelZenit->sizeHint() );
	QToolTip::add( labelZenit,zenitText );

	digitalclock = new DigitalClock( toolBarObject );
	CHECK_PTR( digitalclock );
	digitalclock->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	QToolTip::add( digitalclock,clockText );
	toolBarObject->addSeparator();

	labelMode = new QLabel( "    Generalized    ",toolBarProgram );
	CHECK_PTR( labelMode );
	labelMode->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelMode->setAlignment( AlignCenter );
	labelMode->setFixedSize( labelMode->sizeHint() );
	QToolTip::add( labelMode,modeText );

	labelModeTime = new QLabel( " 00:00 ",toolBarProgram );
	CHECK_PTR( labelModeTime );
	labelModeTime->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelModeTime->setAlignment( AlignCenter );
	labelModeTime->setFixedSize( labelModeTime->sizeHint() );
	QToolTip::add( labelModeTime,modeTimeText );

	labelScenario = new QLabel( devmanager.getScenarioName(
				devmanager.getScenarioNumber() ),
				toolBarProgram );
	CHECK_PTR( labelScenario );
	labelScenario->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelScenario->setAlignment( AlignCenter );
	labelScenario->setFixedSize( labelScenario->sizeHint() );
	QToolTip::add( labelScenario,scenarioText );	

	labelScenarioTime = new QLabel( " 00:00:00 ",toolBarProgram );
	CHECK_PTR( labelScenarioTime );
	labelScenarioTime->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelScenarioTime->setAlignment( AlignCenter );
	labelScenarioTime->setFixedSize( labelScenarioTime->sizeHint() );
	QToolTip::add( labelScenarioTime,scenarioTimeText );

	toolBarProgram->setStretchableWidget(labelScenario);

//	QIconSet navigateIconSet;
	navigateIconSet.setPixmap( navigate,QIconSet::Automatic,QIconSet::Normal );
	navigateIconSet.setPixmap( navigateDisabled,QIconSet::Automatic,QIconSet::Disabled );

//	QIconSet selectStarIconSet;
	selectStarIconSet.setPixmap( selectStar,QIconSet::Automatic,QIconSet::Normal );
	selectStarIconSet.setPixmap( selectStarDisabled,QIconSet::Automatic,QIconSet::Disabled );

//	QIconSet playIconSet;
	playIconSet.setPixmap( play,QIconSet::Automatic,QIconSet::Normal );
//	playIconSet.setPixmap( play,QIconSet::Automatic,QIconSet::Active );
	playIconSet.setPixmap( playDisabled,QIconSet::Automatic,QIconSet::Disabled );

//	QIconSet stopIconSet;
	stopIconSet.setPixmap( setStop,QIconSet::Automatic,QIconSet::Normal );
//	stopIconSet.setPixmap( setStop,QIconSet::Automatic,QIconSet::Active );
	stopIconSet.setPixmap( setStopDisabled,QIconSet::Automatic,QIconSet::Disabled );

//	QIconSet pauseIconSet;
	pauseIconSet.setPixmap( setPause,QIconSet::Automatic,QIconSet::Normal );
	pauseIconSet.setPixmap( setPauseDisabled,QIconSet::Automatic,QIconSet::Disabled );

	navigateButton = new QToolButton( navigateIconSet,navigateText,"",
		this,SLOT( slotShowStarDialog() ),toolBarButtons );
	CHECK_PTR(navigateButton);
	navigateButton->setUsesBigPixmap( TRUE );
	navigateButton->setAutoRaise( FALSE );

	selectButton = new QToolButton( selectStarIconSet,selectText,"",
		this,SLOT( slotSelectStarDialog() ),toolBarButtons );
	CHECK_PTR(selectButton);
	QToolTip::add( selectButton,selectText );
	selectButton->setUsesBigPixmap( TRUE );
	selectButton->setAutoRaise( FALSE );

	playButtonGroup = new QButtonGroup( this );
	CHECK_PTR( playButtonGroup );
	playButtonGroup->setExclusive( TRUE );

	playButton = new QToolButton( playIconSet, playText,"",
			this, SLOT( playScenario() ), toolBarButtons );
	CHECK_PTR(playButton);
	playButtonGroup->insert( playButton,1 );
	playButton->setToggleButton( TRUE );
	playButton->setUsesBigPixmap( TRUE );
	playButton->setAutoRaise( FALSE );

	stopButton = new QToolButton( stopIconSet, stopText, "",
			this, SLOT( stopScenario() ), toolBarButtons );
	CHECK_PTR( stopButton );
	playButtonGroup->insert( stopButton,2 );
	stopButton->setToggleButton( TRUE );
	stopButton->setUsesBigPixmap( TRUE );
	stopButton->setAutoRaise( FALSE );
	stopButton->setDisabled( TRUE );
//	stopButton->setDisabled( FALSE );

	pauseButton = new QToolButton( pauseIconSet, pauseText,"",
		this, SLOT( togglePause() ), toolBarButtons );
	CHECK_PTR(pauseButton);
	pauseButton->setUsesBigPixmap( TRUE );
	pauseButton->setAutoRaise( FALSE );
	pauseButton->setToggleButton( TRUE );
	pauseButton->setDisabled( TRUE );

	toolBarProgram->addSeparator();

	item_id = 0;
	menu_help.resize(100);
	n_dis = 0;
	disabl_count.resize(100);

	file = new QPopupMenu;
	CHECK_PTR( file );

	file->insertItem( "Select star...",
		this,SLOT( slotSelectStarDialog() ),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Select star..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	file->insertItem( "Show star...",
		this,SLOT( slotShowStarDialog() ),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Show star..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	file->insertItem( "&Open DB", this, SLOT(slotOpenDB()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Open DB" );
	item_id++;

	file->insertSeparator();
	file->insertItem( "Exit", this, SLOT(myQuit()),CTRL+Key_Q,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Exit" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	help = new QPopupMenu;
	CHECK_PTR( help );
	help->insertItem( "Help", this, SLOT(slotHelp()),CTRL+Key_H,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Help" );
	item_id++;
		
	measure = new QPopupMenu;
	CHECK_PTR( measure );

	measure->insertItem( "Normal", &devmanager, SLOT(normal()),
						0, item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Normal..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertItem( "Generalized", &devmanager, SLOT(generalized()),
						0, item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Generalized..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertItem( "Star centering", &devmanager,SLOT(starCentering()),
									0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Star centering..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertItem( "Focus control",
				&devmanager,SLOT(focusControl()),
									0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Focus control..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertItem( "Background",
				&devmanager,SLOT(backgroundMeasurement()),
									0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Background measurement..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertItem( "Flux Estimation",&devmanager,SLOT(fluxEstimation()),
						CTRL+Key_F,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Flux Estimation..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	measure->insertSeparator();

	scenario_set = new QPopupMenu;
	CHECK_PTR( scenario_set );
	measure->insertItem( "Select Scenario",scenario_set,
			item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Select Scenario" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

 	int nscen = devmanager.getMaxScenarioNumber();;
	scen_id.resize( nscen );
	for( int i=0; i<nscen; i++ ) {
		scenario_set->insertItem( devmanager.getScenarioName(i),
			item_id );
		scen_id[i] = item_id;
		if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
		menu_help[item_id] = new QString( devmanager.getScenarioName(i) );
		item_id++;
		debug( "insertScenario %s",devmanager.getScenarioName(i).data() );
	}
	slotScenario( devmanager.getScenarioNumber() );
	connect( scenario_set, SIGNAL(activated(int)),
		this, SLOT(slotSetScenario(int)) );

	measure->insertSeparator();

	measure->insertItem( "Run scenario",
				this,SLOT(playScenario()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( playText );
	play_item = item_id;
	item_id++;

	measure->insertItem( "Stop scenario",
				this,SLOT(stopScenario()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( stopText );
	stop_item = item_id;
	menuBar()->setItemEnabled( stop_item, FALSE );
	item_id++;

	measure->insertItem( "Pause scenario",
				this,SLOT(togglePauseMenu()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( pauseText );
	pause_item = item_id;
	menuBar()->setItemEnabled( pause_item, FALSE );
	item_id++;

 	tests = new QPopupMenu;
	CHECK_PTR( tests );

	tests->insertItem( "Exchange Test",&devmanager,SLOT(exchangeTest()),
						0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Run full exchange test" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tests->insertItem( "Detectors Test",&devmanager,SLOT(detectorTest()),
						0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Run detectors test" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tests->insertItem( "Statistic Test",&devmanager,SLOT(statisticTest()),
						0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Run statistic test" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools = new QPopupMenu;
	CHECK_PTR( tools );

	tools->insertItem( "Initialization", &devmanager,
			SLOT(init()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Initialization" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools->insertItem( "Detectors Counting measurement",&devmanager,SLOT(detectorCountingMeasurement()),
						0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Detectors Counting measurement..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools->insertItem( "Detectors Statistics measurement",&devmanager,SLOT(detectorStatisticMeasurement()),
						0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Detectors Statistics measurement..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools->insertItem( "Background correction...",
			this,SLOT(slotBackCorrectionDialog()),
									0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Background correction..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools->insertItem( "Centering correction...",
			this,SLOT(slotCenterCorrectionDialog()),
								0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Centering correction..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	tools->insertItem( "Adjust Aperture...",
		this,SLOT( slotAdjustDialog() ),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Adjust Aperture..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	functions = new QPopupMenu;
	CHECK_PTR( functions );
	tools->insertItem( "Functions",functions,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Functions" );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	functions->insertItem( "",
		this, SLOT( turnDoor() ),0,item_id );
	door_id = item_id;
//	slotTurnDoor( devmanager.isDoorOpen() );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Door Open/Close" );
	item_id++;

	functions->insertItem( "",
			this,SLOT( turnHV() ),0,item_id );
	hv_id = item_id;
//	slotTurnHV( devmanager.isHVOn() );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "HV On/Off" );
	item_id++;

	functions->insertItem( "",
		this,SLOT( turnButtons() ),0,item_id );
	buttons_id = item_id;
//	slotTurnButtons( devmanager.isButtonsOn() );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Buttons On/Off" );
	item_id++;

	functions->insertItem( "",
		this, SLOT( turnIllum() ),0,item_id );
	illum_id = item_id;
//	slotTurnIllum( devmanager.isIllumOn() );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Illumintaion On/Off" );
	item_id++;

	functions->insertSeparator();

	functions->insertItem( "Set Illumination level...",
		this,SLOT( slotIllumDialog() ),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Set Illumination level" );
	item_id++;

	aperture_set = new QPopupMenu;
	CHECK_PTR( aperture_set );
	functions->insertItem( "Set &Aperture",aperture_set,
			item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Set Aperture" );
	item_id++;

	int ns = devmanager.getNStops();
	apert_id.resize( ns );
	for( int i=0; i<ns; i++ ) {
		aperture_set->insertItem( devmanager.getStopName(i),
			item_id );
		apert_id[i] = item_id;
		if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
		menu_help[item_id] = new QString( devmanager.getStopName(i) );
		item_id++;
		debug( "insertAperture %s",devmanager.getStopName(i).data() );
	}
	slotAperture( devmanager.currentAperture() );
	connect( aperture_set, SIGNAL(activated(int)),
		this, SLOT(setAperture(int)) );

	functions->insertItem( "Set Null-Point",
		&devmanager,SLOT( nullSet() ),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Set Null-Point..." );
	item_id++;

	tools->insertSeparator();

	tools->insertItem( "Talk",this,SLOT(slotTalkDialog()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Talk (debug mode)" );
	item_id++;

	tools->insertItem( "Break Accumulation",&devmanager,SLOT(stopCounting()),
						CTRL+Key_S,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Break Accumulation..." );
	break_item = item_id;
	menuBar()->setItemEnabled( break_item, FALSE );
	item_id++;

	tools->insertItem( "Parking", &devmanager,
			SLOT(park()),0,item_id );
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Parking..." );
	if( disabl_count.size() <= (uint)n_dis ) disabl_count.resize(disabl_count.size()+10);
	disabl_count[n_dis] = item_id;
	n_dis++;
	item_id++;

	debug( "config" );
	config = new QPopupMenu;
	CHECK_PTR( config );
	cnfg = new CFGDialog( config,&item_id,menu_help );

	menuBar()->insertItem( "&File",file );
	menuBar()->insertItem( "&Measurements",measure );
	menuBar()->insertItem( "T&ests",tests );
	menuBar()->insertItem( "&Tools",tools );

	menuBar()->insertItem( "&Config",config );
	menuBar()->insertSeparator();
	menuBar()->insertItem( "&Help",help );

	connect( menuBar(),SIGNAL(highlighted(int)),
				this,SLOT(slotHighlighted(int)) );
	connect( cnfg,SIGNAL(signalShowStatus(const char*,int)),
				this,SLOT(slotShowMessage(const char*,int)) );
//	connect( &devmanager,SIGNAL( signalShowMessage(const char*,int) ),
//				this,SLOT(slotShowMessage(const char*,int)) );
	connect( tree, SIGNAL( changed() ), this,SLOT( change() ) );


	normalTimer = startTimer( 1000 );	// 1 sec

	setGeometry( _winX, _winY, _winWidth, _winHeight );
	statusBar()->message( "Ready",2000 );
	repaint();
}

void MyWidget::myQuit( void ) {
	debug( "mywidget::myQuit..." );
	devmanager.park();
	qApp->quit();
}

void MyWidget::playScenario( void ) {
	devmanager.playScenario();
	showPlayScenario();
}

void MyWidget::showPlayScenario( void ) {
	playButton->setOn( TRUE );
	playButton->setDisabled( TRUE );

	menuBar()->setItemEnabled( play_item, FALSE );

	stopButton->setOn( FALSE );
	stopButton->setDisabled( FALSE );

	menuBar()->setItemEnabled( stop_item, TRUE );
	pauseButton->setDisabled( FALSE );
	menuBar()->setItemEnabled( pause_item, TRUE );
}

void MyWidget::stopScenario( void ) {
	devmanager.stopScenario();
	showStopScenario();
}

void MyWidget::showStopScenario( void ) {
	playButton->setOn( FALSE );
	playButton->setDisabled( FALSE );

	menuBar()->setItemEnabled( play_item, TRUE );

	stopButton->setOn( TRUE );
	stopButton->setDisabled( TRUE );

	menuBar()->setItemEnabled( stop_item, FALSE );
	pauseButton->setDisabled( TRUE );
	menuBar()->setItemEnabled( pause_item, FALSE );
}

void MyWidget::togglePauseMenu( void ) {
	pauseButton->toggle();
	togglePause();
}

void MyWidget::togglePause( void ) {
	debug( "&&&&&&&&&&&&&&&&&&&&&&&&&&&&& toggle Pause!" );
	if( pauseButton->isOn() ) {
		debug( "&&&&&&&&&& pauseOn!" );
		stopButton->setDisabled( TRUE );
		menuBar()->setItemEnabled( stop_item, FALSE );
		devmanager.setPause();
	} else {
		debug( "&&&&&&&&&& pauseOff!" );
		stopButton->setDisabled( FALSE );
		menuBar()->setItemEnabled( stop_item, TRUE );
		scenarioMutex.unlock();
	}
	debug( "&&&&&&&&&& end toggle" );
}

void MyWidget::slotTurnIllum( int _on ) {
	debug( "Illum On/off: %d",_on );
	switch ( _on ) {
		case  ON : functions->changeItem( illum_id, 
			"Turn Illumintaion\tOff" );
			labelIllum->setText( On );
			break;
		case  OFF : functions->changeItem( illum_id,
			"Turn Illumintaion\tOn" );
			labelIllum->setText( Off );
			break;
		case -1 : functions->changeItem( illum_id,
			"Turn Illumintaion\tOn/Off" );
			labelIllum->setText( Undefin );
			break;
	}
}

void MyWidget::slotTemperature( double _temperature ) {
	QString str;
	str.setNum( _temperature,'f',1 );
	labelTmpr->setText( str );
}

void MyWidget::turnIllum( void ) {
	if( devmanager.isIllumOn() ) {
		devmanager.turnIllum(0,1);
	} else {
		devmanager.turnIllum(1,1);
	}
}

void MyWidget::slotTurnHV( int _on ) {
	debug( "HV On/off: %d",_on );
	switch( _on ) {
		case  ON : functions->changeItem( hv_id,
			"Turn HV\tOff" );
			labelHV->setText( On );
			break;
		case  OFF : functions->changeItem( hv_id,
			"Turn HV\tOn" );
			labelHV->setText( Off );
			break;
		case -1 : functions->changeItem( hv_id,
			"Turn HV\tOn/Off" );
			labelHV->setText( Undefin );
			break;
	}
}

void MyWidget::turnHV( void ) {
	int _on = devmanager.isHVOn();
	if( _on == ON ) {
		devmanager.turnHV(OFF,1);
	} else {
		devmanager.turnHV(ON,1);
	}
}

void MyWidget::slotTurnButtons( int _on ) {
	debug( "Buttons On/off: %d",_on );
	switch( _on ) {
		case  1 : functions->changeItem( buttons_id,
			"Buttons\tLock" ); break;
		case  0 : functions->changeItem( buttons_id,
			"Buttons\tUnLock" ); break;
		case -1 : functions->changeItem( buttons_id,
			"Buttons\tLock/UnLock" ); break;
	}
}

void MyWidget::turnButtons( void ) {
	if( devmanager.isButtonsOn() ) {
		devmanager.turnButtons(0,1);
	} else {
		devmanager.turnButtons(1,1);
	}
}

void MyWidget::slotTurnDoor( int _open ) {
	debug( "Set Door On/off: %d",_open );
	switch( _open ) {
		case  OPEN : functions->changeItem( door_id,
			"Door\tClose" ); 
			labelDoor->setText( Open );
			break;
		case  CLOSE : functions->changeItem( door_id,
			"Door\tOpen" );
			labelDoor->setText( Close );
			break;
		case -1 : functions->changeItem( door_id,
			"Door\tOpen/Close" );
			labelDoor->setText( Undefin );
			break;
	}
}

void MyWidget::turnDoor( void ) {
	if( devmanager.isDoorOpen() == OPEN ) {
		devmanager.turnDoor(CLOSE,1);
	} else {
		devmanager.turnDoor(OPEN,1);
	}
}

void MyWidget::slotScenario( int number ) {
	uint i;
	debug( "Scenario N: %d",number );

	labelScenario->setText( devmanager.getScenarioName(number) );

	for( i=0; i<scen_id.size(); i++ ) {
		menuBar()->setItemEnabled( scen_id[i], true );
	}
	if( (number < 0) || 
		(number >= (int)scen_id.size() ) ) return;
	menuBar()->setItemEnabled( scen_id[number], false );
}

void MyWidget::slotSetScenario( int _id ) {
	uint i;
	for( i=0; i<scen_id.size(); i++ ) {
		if( _id == scen_id[i] ) {
			debug( "MyWidget: set Scenario %d",i );
			devmanager.setCurrentScenario(i);
			labelScenario->
				setText(devmanager.getScenarioName(i) );
			return;
		}
	}
}

void MyWidget::slotAperture( int _stopNumber ) {
	uint i;

	labelApert->setText( devmanager.getStopName(_stopNumber) );

	for( i=0; i<apert_id.size(); i++ ) {
		menuBar()->setItemEnabled( apert_id[i], TRUE );
	}
	if( (_stopNumber < 0) || 
		(_stopNumber >= (int)apert_id.size() ) ) return;
	menuBar()->setItemEnabled( apert_id[_stopNumber], FALSE );
}

void MyWidget::setAperture( int _id ) {
	uint i;
	for( i=0; i<apert_id.size(); i++ ) {
		if( _id == apert_id[i] ) {
			debug( "MyWidget: set Aperture %d",i );
			devmanager.setAperture(i,1);
			debug( "MyWidget::setAperture return" );
			return;
		}
	}
}

void MyWidget::setProgress( const QString& _text, double progress ) {
	if( progress == -1 ) openProgressor( _text );
	else if( progress == -2 ) closeProgressor();
	else progressor->setProgress( (int)progress );
}

void MyWidget::openProgressor( const QString& str ) {
	statusBar()->message( str );
	labelMode->setText( str );
	progressor->reset();
	if( !progressor->isVisible() ) progressor->show();
}

void MyWidget::closeProgressor( void ) {
	statusBar()->clear();
	if( progressor->isVisible() ) progressor->hide();
}

void MyWidget::setMode( const QString& _text, double _time_ ) {
	QString str;

	if( labelMode->text() != _text )
		labelMode->setText( _text );

	labelModeTime->setText( str.setNum(_time_) );
	if( isScenario ) {
		labelScenarioTime->setText( str.setNum(scenarioTime+_time_) );
	}
}

void MyWidget::setScenario( const QString& _text, double _time_ ) {
	QString str;

	if( labelScenario->text() != _text )
			labelScenario->setText( _text );
	scenarioTime = _time_;
	labelScenarioTime->setText( str.setNum(_time_) );
}

/*
void MyWidget::resizeEvent( QResizeEvent* e ) {

	int _chng = 0;
	if( _winWidth != e->size().width() ) {
		_winWidth = e->size().width();
		_chng = 1;
	}
	if( _winHeight != e->size().height() ) {
		_winHeight = e->size().height();
		_chng = 1;
	}
//	debug( "MyWidget::ResizeEvent: %d %d changd:%d",
//				_winWidth,_winHeight,_chng );
	if( _chng ) {
		QString _geometry;
		_geometry.sprintf( "%d %d %d %d",_winX,_winY,_winWidth,_winHeight );
		tree->findBranch( "Preferences" )->
			findBranch("Text window settings")->
		findLeaf( "WindowGeometry" )->replaceData(_geometry);
		tree->setChanged();
	}
}
*/

/*
void MyWidget::moveEvent( QMoveEvent* e ) {
	_winX = e->pos().x();
	_winY = e->pos().y();
	debug( "MyWidget::MoveEvent: %d %d",_winX,_winY );

	QString _geometry;
	_geometry.sprintf( "%d %d %d %d",_winX,_winY,_winWidth,_winHeight );
	tree->findBranch( "Preferences" )->
		findBranch("Text window settings")->
	findLeaf( "WindowGeometry" )->replaceData(_geometry);
	tree->setChanged();
}
*/
void MyWidget::change( void ) {
	loadConst();
//	setGeometry( _winX+10, _winY+68, _winWidth, _winHeight );
//   Pochemu +10 i +68 NE ZNAYU !!!!!

	labelApert->setFixedSize( labelApert->sizeHint() );
	labelDoor->setFixedSize( labelDoor->sizeHint() );
	labelIllum->setFixedSize( labelIllum->sizeHint() );
	labelHV->setFixedSize( labelHV->sizeHint() );
	labelTmpr->setFixedSize( labelTmpr->sizeHint() );
	labelZenit->setFixedSize( labelZenit->sizeHint() );
	labelStar->setFixedSize( labelStar->sizeHint() );
	labelScenario->setFixedSize( labelScenario->sizeHint() );
	labelMode->setFixedSize( labelMode->sizeHint() );

//	setFixedSize( size() );
//	resize( width()+1,height() );
//	resize( width()-1,height() );
}

void MyWidget::loadConst( void ) {
	int key;
	const char* _geometry = tree->findBranch( "Preferences" )->
		findBranch("Text window settings")->
		findLeaf( "WindowGeometry" )->getData()->data();
	if( !strcasecmp( _geometry,"AUTO" ) ) {
		_winX = 0;
		_winY = 0;
		_winWidth = qApp->desktop()->width()/2;
		_winHeight = qApp->desktop()->height()-50;
	} else {
		key = sscanf( _geometry,"%d%d%d%d", &_winX, &_winY,
			&_winWidth, &_winHeight );
		debug( "_________ GEOMETRY: %d %d %d %d",_winX, _winY,
			_winWidth, _winHeight );
		if( key != 4 ) {
			debug( "TextWindow: Invalid geometry!!!" );
			_winX = 0;
			_winY = 0;
			_winWidth = qApp->desktop()->width()/2;
			_winHeight = qApp->desktop()->height()-50;
		}
	}

	_fontSizeLarge = tree->findBranch( "Preferences" )->
		findBranch("Text window settings")->
		findLeaf( "FontSizeLarge" )->getData()->toInt();
	_fontSizeSmall = tree->findBranch( "Preferences" )->
		findBranch("Text window settings")->
		findLeaf( "FontSizeSmall" )->getData()->toInt();
	qApp->setFont( QFont("Helvetica",_fontSizeLarge),TRUE );
	statusBar()->setFont( QFont("Helvetica",_fontSizeSmall),TRUE );
	mainView->setFont( QFont("Helvetica",_fontSizeSmall),TRUE );
	mainView->fitToFont();

	QString* str = tree->findBranch("General")->
		findBranch("Inputs")->findLeaf("CoordinateSystem")->getData();
	if( str->upper() == "RA" ) _showHA = 0;
	else _showHA = 1;
}

void MyWidget::slotDisableItems( void ) {
	int i;
	for( i=0; i<n_dis; i++ ) {
		menuBar()->setItemEnabled( disabl_count[i], FALSE );
	}
	cnfg->setDisabled();
	selectButton->setDisabled( TRUE );
	navigateButton->setDisabled( TRUE );
	playButton->setDisabled( TRUE );
	menuBar()->setItemEnabled( break_item, TRUE );
	if( existBackCorrectionDialog ) backCorrectionDialog->setButtonsEnabled( FALSE );
	if( existCenterCorrectionDialog ) centerCorrectionDialog->setButtonsEnabled( FALSE );
	_updateHA = 0;
}

void MyWidget::slotEnableItems( void ) {
	int i;
	debug( "slot enable Items" );
	for( i=0; i<n_dis; i++ ) {
		menuBar()->setItemEnabled( disabl_count[i], TRUE );
	}
	cnfg->setEnabled();
	selectButton->setDisabled( FALSE );
	navigateButton->setDisabled( FALSE );
	if( !isScenario ) playButton->setDisabled( FALSE );
	menuBar()->setItemEnabled( break_item, FALSE );
	if( existBackCorrectionDialog ) backCorrectionDialog->setButtonsEnabled( TRUE );
	if( existCenterCorrectionDialog ) centerCorrectionDialog->setButtonsEnabled( TRUE );
	_updateHA = 1;
	debug( "end of slotEnableItems" );
}

MyWidget::~MyWidget() {
	debug( "................. MyWidget: destructor..." );
	killTimer( normalTimer );

	debug( "....delete toolBarDevice" );
	delete toolBarDevice;
//	debug( "....deletetoolBarDeviceHead " );
//	delete toolBarDeviceHead;
	debug( "....deletetoolBarObject " );
	delete toolBarObject;
	debug( "....delete toolBarProgram" );
	delete toolBarProgram;
	debug( "....delete toolBarButtons" );
	delete toolBarButtons;

	debug( "....delete mainView" );
	delete mainView;
	debug( "....delete progressor" );
	delete progressor;
/*
	debug( "....delete labelApert" );
	delete labelApert;
	debug( "....delete labelApertHead" );
	delete labelApertHead;
	debug( "....delete labelIllum" );
	delete labelIllum;
	debug( "....delete labelIllumHead" );
	delete labelIllumHead;
	debug( "....delete labelHV" );
	delete labelHV;
	debug( "....delete labelHVHead" );
	delete labelHVHead;
	debug( "....delete labelDoor" );
	delete labelDoor;
	debug( "....delete labelDoorHead" );
	delete labelDoorHead;
	debug( "....delete labelTmpr" );
	delete labelTmpr;
	debug( "....delete labelTmprHead" );
	delete labelTmprHead;

	debug( "....delete labelStar" );
	delete labelStar;
	debug( "....delete labelDiaphragm" );
	delete labelDiaphragm;
	debug( "....delete labelZenit" );
	delete labelZenit;
	debug( "....delete digitalclock" );
	delete digitalclock;

	debug( "....delete navigateButton" );
	delete navigateButton;
	debug( "....delete selectButton" );
	delete selectButton;

	debug( "....delete labelScenario" );
	delete labelScenario;
	debug( "....delete labelScenarioTime" );
	delete labelScenarioTime;
	debug( "....delete labelMode" );
	delete labelMode;
	debug( "....delete labelModeTime" );
	delete labelModeTime;
*/
	debug( "....delete playButtonGroup" );
	delete playButtonGroup;
/*
	debug( "....delete stopButton" );
	delete stopButton;
	debug( "....delete pauseButton" );
	delete pauseButton;
	debug( "....delete playButton" );
	delete playButton;
*/
	debug( "....delete file" );
	delete file;
	debug( "....delete measure" );
	delete measure;
	debug( "....delete tests" );
	delete tests;
	debug( "....delete help" );
	delete help;
	debug( "....delete config" );
	delete config;
	debug( "....delete tools" );
	delete tools;
	debug( "....delete functions" );
	delete functions;
	debug( "....delete aperture_set" );
	delete aperture_set;
	debug( "....delete scenario_set" );
	delete scenario_set;
	debug( "....delete cnfg" );
	delete cnfg;

	int i;
	debug( "....delete menu_help" );
	for( i=0; i<item_id; i++ ) {
		delete menu_help[i];
	}
	if( existTalkDialog ) {
		debug( "....delete talkDialog" );
		delete talkDialog;
		existTalkDialog = 0;
	}
	if( existAdjustDialog ) {
		debug( "....delete adjustDialog" );
		delete adjustDialog;
		existAdjustDialog = 0;
	}
	if( existSelectStarDialog ) {
		debug( "....delete selectStarDialog" );
		delete selectStarDialog;
		existSelectStarDialog = 0;
	}
	if( existShowStarDialog ) {
		debug( "....delete showStarDialog" );
		delete showStarDialog;
		existShowStarDialog = 0;
	}
	if( existIllumDialog ) {
		debug( "....delete illumDialog" );
		delete illumDialog;
		existIllumDialog = 0;
	}
	if( existBackCorrectionDialog ) {
		debug( "....delete backCorrectionDialog" );
		delete backCorrectionDialog;
		existBackCorrectionDialog = 0;
	}
	if( existCenterCorrectionDialog ) {
		debug( "....delete centerCorrectionDialog" );
		delete centerCorrectionDialog;
		existCenterCorrectionDialog = 0;
	}
}

void MyWidget::slotHighlighted( int id ) {
	if( id >= 0 ) {
		statusBar()->message( menu_help[id]->data(),4000 );
	}
}

void MyWidget::slotShowMessage( const char* _text, int ms ) {
	debug( "start slotShowMessage %s %d",_text,ms );
	if( ms > 0 ) { statusBar()->message( _text,ms ); return; }
	if( ms < 0 ) { statusBar()->message( _text ); return; }
	if( ms == 0 ) { statusBar()->clear(); return; }
	debug( "end slotShowMessage %s %d",_text,ms );
}

void MyWidget::slotShowMessage( const char* _text  ) {
	statusBar()->message( _text );
	debug( "MyWidget::slotShowMessage: %s",_text );
}

void MyWidget::slotHelp() {
	QMessageBox::about( this, "help","Help me!" );
}

void MyWidget::slotOpenDB() {
	QString fn = QFileDialog::getOpenFileName( 0,"*.dat",this );
	QString str,text;
	int loadError,row;
	double x,y;
	int i;

	if( !fn.isEmpty() ) {
		QFile f( fn );
		if( !f.open( IO_ReadOnly ) ) {
			debug( "Net takogo faila!" );
			return;
		}
		curve_x.reset(100);
		curve_y.reset(100);

		QTextStream t(&f);
		loadError = 0;
		row = 0;
		while ( !t.eof() && !loadError ) {
			str = t.readLine();
			row++;
			if( (str[0] != '*') && (!str.isEmpty()) ) {
				if( sscanf( str.data(),"%lf %lf",&x,&y ) != 2 ) {
					text.sprintf( "Loading error in row %d\n%s",
						row,str.data() );
					QMessageBox::critical( 0,"invalid",text.data() );
					statusBar()->message( "Loading aborted", 4000 );
					loadError = 1;
				}
				curve_x.put( x );
				curve_y.put( y );
			}
		}
		f.close();
		text = "Loaded file";
		text.operator+=( fn );
		if( !loadError ) statusBar()->message( text, 4000 );

		for( i=0; i<=curve_x.getNumber(); i++ ) {
			debug( "%d: %f %f", i,curve_x[i],curve_y[i] );
		}
		curve_x.searchMinMax();
		debug( "X: min = %f max = %f",curve_x.getDataMin(),curve_x.getDataMax() );
		curve_y.searchMinMax();
		debug( "Y: min = %f max = %f",curve_y.getDataMin(),curve_y.getDataMax() );
		emit signalDrawCurve( &curve_x, &curve_y );
		debug( "mywidget: emit signalDrawCurve" );
	} else {
		statusBar()->message( "Loading aborted", 2000 );
	}
}

void MyWidget::slotTalkDialog( void ) {
	if( !existTalkDialog ) {
		talkDialog = new TalkDialog();
		CHECK_PTR(talkDialog);
		existTalkDialog = 1;
	}

	talkDialog->show();
}

void MyWidget::slotAdjustDialog( void ) {
	QString str;
	if( !existAdjustDialog ) {
		adjustDialog = new AdjustDialog();
		CHECK_PTR(adjustDialog);
		existAdjustDialog = 1;
	}

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
	int _oldIllum = devmanager.isIllumOn();
	devmanager.turnIllumOn();
	devmanager.startAdjustAperture();
	adjustDialog->update();
	int ret = adjustDialog->exec();
	if( ret == QDialog::Accepted ) {
		devmanager.acceptAdjustAperture(1);
	}
	devmanager.setAperture( _oldApert,1,1 );
// !!! Nasil'stvenno stavim dazhe tu zhe aperturu, t.k. mogla byt'
// !!!sdvinuta na shagi!!!	
	devmanager.turnIllum( _oldIllum,1 );
}

void MyWidget::slotSelectStarDialog( void ) {
	QString str;
	if( !existSelectStarDialog ) {
		selectStarDialog = new SelectStarDialog();
		CHECK_PTR(selectStarDialog);
		existSelectStarDialog = 1;
	} else selectStarDialog->update();

	selectButton->setDisabled( TRUE );
	int ret = selectStarDialog->exec();
	if( ret == QDialog::Accepted ) {
		labelStar->setText( star.getName() );
		labelZenit->setText( star.getAirMass() );
	}
	selectButton->setDisabled( FALSE );
}

void MyWidget::slotShowStarDialog( void ) {
	QString str;
	if( !existShowStarDialog ) {
		showStarDialog = new ShowStarDialog( _showHA );
		CHECK_PTR(showStarDialog);
		existShowStarDialog = 1;
	} else showStarDialog->update( _showHA );

	navigateButton->setDisabled( TRUE );
	showStarDialog->exec();
	navigateButton->setDisabled( FALSE );
}

void MyWidget::timerEvent( QTimerEvent *e ) {
	if( isTimerEventLocked ) return;
	isTimerEventLocked = true;
//	infoExportMutex.lock();
	if( ExportText::isChanged ) {
		ExportText::isChanged = false;
		mainView->showTextLine();
	}
	if( ExportTable::isChanged ) {
		ExportTable::isChanged = FALSE;
		mainView->fillTable();
	}
	if( InfoExport::isProgressChanged ) {
		setProgress( InfoExport::progress.message,
				InfoExport::progress.progress );
		InfoExport::isProgressChanged = FALSE;
	}

	if( InfoExport::isMessage ) {
		InfoExport::isMessage = false;
		slotShowMessage( InfoExport::message.data(),
				InfoExport::ms );
	}
	if( InfoExport::isModeChanged ) {
		setMode( InfoExport::mode.message,
				InfoExport::mode.progress );
		InfoExport::isModeChanged = FALSE;
	}
	if( isScenario != InfoExport::isScenario ) { // scenario state is changed
		isScenario = InfoExport::isScenario;
		if( !isScenario ) showStopScenario();
		else showPlayScenario();
	}

	if( InfoExport::isScenarioChanged ) {
		setScenario( InfoExport::scenario.message,
				InfoExport::scenario.progress );
		InfoExport::isScenarioChanged = FALSE;
	}

//	infoExportMutex.unlock();
	stateExportMutex.lock();
	if( StateExport::isStartCounting ) {
		StateExport::isStartCounting = FALSE;
		slotDisableItems();
	}
	if( StateExport::isEndCounting ) {
		StateExport::isEndCounting = FALSE;
		slotEnableItems();
		slotShowMessage( "End of count" );
	}
	if( StateExport::isChanged ) {
		StateExport::isChanged = FALSE;
		if( oldState.hv != StateExport::state.hv ) {
			oldState.hv = StateExport::state.hv;
			slotTurnHV( oldState.hv );
		}
		if( oldState.illum != StateExport::state.illum ) {
			oldState.illum = StateExport::state.illum;
			slotTurnIllum( oldState.illum );
		}
		if( oldState.door != StateExport::state.door ) {
			oldState.door = StateExport::state.door;
			slotTurnDoor( oldState.door );
		}
		if( oldState.buttons != StateExport::state.buttons ) {
			oldState.buttons = StateExport::state.buttons;
			slotTurnButtons( oldState.buttons );
		}
		if( oldState.aperture != StateExport::state.aperture ) {
			oldState.aperture = StateExport::state.aperture;
			slotAperture( oldState.aperture );
		}
		if( oldState.temperature != StateExport::state.temperature ) {
			oldState.temperature = StateExport::state.temperature;
			slotTemperature( oldState.temperature );
		}
	}
	stateExportMutex.unlock();

	digitalclock->showTime();
	if( _updateHA ) {
		star.updateCurrent();
		labelZenit->setText( star.getAirMass() );
		if( existShowStarDialog && _showHA ) showStarDialog->showHA();
	}
	isTimerEventLocked = false;
}

void MyWidget::slotIllumDialog( void ) {
	if( !existIllumDialog ) {
		illumDialog = new IllumDialog();
		CHECK_PTR(illumDialog);
		existIllumDialog = 1;
	}
	illumDialog->update();
	illumDialog->show();
}

void MyWidget::slotBackCorrectionDialog( void ) {
	QString str;

	debug( "Slot BackCorrection Dialog!!!" );
	int ret = QMessageBox::information( 0,"background",
			"Point to the dark sky","Continue","Cancel" );
	if( ret != 0 ) {
		slotShowMessage( "Canceled...",4000 );
		return;
	}

	if( !existBackCorrectionDialog ) {
		backCorrectionDialog = new BackCorrectionDialog();
		CHECK_PTR(backCorrectionDialog);
		existBackCorrectionDialog = 1;
	}
	backCorrectionDialog->update();

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
//	devmanager.startBackgroundCorrection();
	ret = backCorrectionDialog->exec();
	if( ret == QDialog::Accepted ) {
		slotShowMessage( "Accepted...",4000 );
	} else {
		slotShowMessage( "Canceled...",4000 );
	}
	backCorrectionDialog->finish();
	devmanager.setAperture( _oldApert,1 );
}

void MyWidget::slotCenterCorrectionDialog( void ) {
	QString str;

	debug( "Slot CenterCorrection Dialog!!!" );
	int ret = QMessageBox::information( 0,"centering",
			"Point to the star","Continue","Cancel" );
	if( ret != 0 ) {
		slotShowMessage( "Canceled...",4000 );
		return;
	}

	if( !existCenterCorrectionDialog ) {
		centerCorrectionDialog = new CenterCorrectionDialog();
		CHECK_PTR(centerCorrectionDialog);
		existCenterCorrectionDialog = 1;
	}
	centerCorrectionDialog->update();

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
//	devmanager.startCenterCorrection();
	ret = centerCorrectionDialog->exec();
	if( ret == QDialog::Accepted ) {
		slotShowMessage( "Accepted...",4000 );
	} else {
		slotShowMessage( "Canceled...",4000 );
	} 
	centerCorrectionDialog->finish();
	devmanager.setAperture( _oldApert,1 );
}

void MyWidget::keyPressEvent( QKeyEvent *k ) {
	switch( tolower(k->ascii()) ) {
		case '1': labelDiaphragm->setPixmap( ld ); break;
		case '2': labelDiaphragm->setPixmap( dd ); break;
		case '3': labelDiaphragm->setPixmap( rd ); break;
		case '4': labelDiaphragm->setPixmap( ll ); break;
		case '5': labelDiaphragm->setPixmap( cc ); break;
		case '6': labelDiaphragm->setPixmap( rr ); break;
		case '7': labelDiaphragm->setPixmap( lu ); break;
		case '8': labelDiaphragm->setPixmap( uu ); break;
		case '9': labelDiaphragm->setPixmap( ru ); break;
		case '0': labelDiaphragm->setPixmap( empt ); break;
	}
}