#ifndef MYWIDGET_H
#define MYWIDGET_H

#include <qmainwindow.h>
#include <qtoolbar.h>
#include <qbuttongroup.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qiconset.h>
#include <qtooltip.h>
#include <qprogressbar.h>
#include <qarray.h>

#include "widgets.h"
#include "tree.h"
#include "cfgdialog.h"
#include "viewstar.h"
#include "dataarray.h"
#include "jkexport.h"

class MyWidget : public QMainWindow {
    Q_OBJECT

public:
	MyWidget( QWidget *parent=0, const char *name=0 );
	~MyWidget( void );
private:

	void openProgressor( const QString& );
	void closeProgressor( void );

protected:
	void timerEvent( QTimerEvent* );
	void keyPressEvent( QKeyEvent * );
//	virtual void resizeEvent( QResizeEvent* );
//	virtual void moveEvent( QMoveEvent* );

private:
	int _fontSizeLarge;
	int _fontSizeSmall;
	int _winX;
	int _winY;
	int _winWidth;
	int _winHeight;
	int _showHA;
	int normalTimer;
	bool isTimerEventLocked;
	int _updateHA;

	DeviceState oldState;
	bool isScenario;
	double scenarioTime; // ostalos' do finisha scenariya

	QToolBar *toolBarDevice,*toolBarObject,*toolBarProgram,*toolBarButtons;
//	QToolBar *toolBarDeviceHead;
	DigitalClock* digitalclock;
	QLabel *labelApert, *labelIllum, *labelDoor, *labelHV;
//	QLabel *labelApertHead, *labelIllumHead, *labelDoorHead, *labelHVHead;
	QLabel *labelTmpr, *labelDiaphragm, *labelStar, *labelZenit;
	QLabel *labelScenario, *labelMode, *labelScenarioTime, *labelModeTime;

	QSize	_labelApertSize;
	QIconSet navigateIconSet,selectStarIconSet, playIconSet;
	QIconSet stopIconSet, pauseIconSet;
	
	QButtonGroup *playButtonGroup;
	QToolButton *playButton, *stopButton, *pauseButton;
	QToolButton *navigateButton;
	QToolButton *selectButton;

	QJKMainView* mainView;
	QProgressBar* progressor;
//	QLabel* progressLabel;
	QPopupMenu* file;
	QPopupMenu* measure;
	QPopupMenu* tests;
	QPopupMenu* help;
	QPopupMenu* config;
	QPopupMenu* tools;
	QPopupMenu* functions;
	QPopupMenu *aperture_set;
	QPopupMenu *scenario_set;
	QArray<int> apert_id;
	QArray<int> scen_id;
	int illum_id, door_id, buttons_id, hv_id;
	CFGDialog* cnfg;
// help:
	QArray<QString*> menu_help;
	int item_id;
	int stop_item;
	int play_item;
	int pause_item;
	int break_item;
	QArray<int> disabl_count;
	int n_dis;
	TalkDialog*				talkDialog;
	AdjustDialog*			adjustDialog;
	SelectStarDialog*		selectStarDialog;
	ShowStarDialog* 		showStarDialog;
	IllumDialog*			illumDialog;
	BackCorrectionDialog*	backCorrectionDialog;
	CenterCorrectionDialog* centerCorrectionDialog;
	int		existTalkDialog;
	int		existAdjustDialog;
	int		existSelectStarDialog;
	int		existShowStarDialog;
	int		existIllumDialog;
	int		existBackCorrectionDialog;
	int		existCenterCorrectionDialog;

	void loadConst( void );
	void showStopScenario( void );
	void showPlayScenario( void );

private slots:

	void myQuit( void );
	void change( void );
	void slotEnableItems( void );
	void slotDisableItems( void );
	void slotHighlighted( int );
	void slotShowMessage( const char* );
	void slotShowMessage( const char*,int );
	void slotHelp( void );
	void slotOpenDB( void );
	void slotTalkDialog( void );
	void turnIllum( void );
	void turnHV( void );
	void turnButtons( void );
	void turnDoor( void );
	void setAperture( int );
	void slotSetScenario( int );
	void slotAdjustDialog( void );
	void slotSelectStarDialog( void );
	void slotShowStarDialog( void );
	void slotIllumDialog( void );
	void slotBackCorrectionDialog( void );
	void slotCenterCorrectionDialog( void );
	void setProgress( const QString&, double );
	void setMode( const QString&, double );
	void setScenario( const QString&, double );

	void playScenario( void );
	void stopScenario( void );
	void togglePauseMenu( void );
	void togglePause( void );

	void slotTurnIllum( int );
	void slotTurnHV( int );
	void slotTurnButtons( int );
	void slotTurnDoor( int );
	void slotAperture( int );
	void slotScenario( int );
	void slotTemperature( double );

//public slots:
//	void slotToScreen( const QString& );

signals:
	void signalDrawCurve( DataArray*,DataArray* );
};

#endif