#include <math.h>
#include <qapplication.h>
#include "jkerror.h"
#include "tools.h"
#include "device.h"
#include "mywidget.h"
#include "graph.h"

extern DevManager devmanager;
extern MyWidget *w;
extern GraphWindow *gw;

extern CFGTree* tree;
extern CFGTree* maptree;
extern double _tmZone;
extern QTime _programStartTime;

QTime getCurrentUT( void ) {		// to UT
	QTime ut =  QTime::currentTime()
		.addSecs( int(-1.0*_tmZone*SEC_IN_HOUR) );
	return ut;
}

/**
	@brief  Return the concatenation of a character argument
	and QString
	@param  char    leading string (normally, a single character
	@param  string  string to append
	@return C-string as char.arg+Qstring (normally, the stamp for
	writing in mass-file 
*/ 

static const int HOUR_IN_DAY = 24;

const char* getStamp( const char * prefix, const QTime& time,
		const char* ext=NULL ) {
	static QString s;
	int hour = time.hour();
	if( time < _programStartTime ) {
		hour += HOUR_IN_DAY;
	}
	s.sprintf( "%02d:%02d:%02d",
		hour,time.minute(),time.second() );
	s.prepend( prefix );
	if( ext ) s += ext;
	return s.data();  
} 

void showError( CFGLeaf* leaf, const char* _error ) {
	QString text;

	text = ( _error );
	if( leaf ) {
		text.operator+=( leaf->getKey() );
		text.operator+=( "   " );
		text.operator+=( leaf->getData()->data() );
	}
	QMessageBox::critical( 0,"invalid",text.data() );
}

char readCharConst( const char* section, 
	const char* subsection,const char* key, FILE* out_f=NULL ) {

	QString* _data = maptree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	int _value;
	bool ok;
	int pos;
	if( (pos=_data->find( "x",0,FALSE )) != -1 ) {
		_value = _data->right( _data->length()-pos-1 ).toInt( &ok,16 );
	} else _value = _data->toInt( &ok,10 );
	if( !ok ) {
		QString warning;
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s",
			section,subsection, key );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
	return (char)_value;
}

char readCharConst( const char* section, 
	const char* subsection,const char* key,char min, char max,
			FILE *out_f=NULL ) {

	QString* _data = maptree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	int _value,pos;
	bool ok;
	if( (pos=_data->find( "x",0,FALSE )) != -1 ) {
		_value = _data->right( _data->length()-pos-1 ).toInt( &ok,16 );
	}
	else _value = _data->toInt( &ok,10 );

	if( ok && (_value >= min) && (_value <= max) ) {
		return (char)_value;
	}

	QString warning;
	if( !ok )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s",
			section,subsection, key );
	if( _value < min )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s\n%d<%d!",
			section,subsection, key, _value, min );
	if( _value > max )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s\n%d>%d!",
			section,subsection, key, _value, max );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return( (char)_value );
		default : return(0);
	}
	return(0);
}

int readIntConst( const char* section,
		const char* subsection,const char* key,
			FILE *out_f=NULL ) {

	QString* _data = maptree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	int _value=0;
	bool ok;
	int pos;
	if( (pos=_data->find( "x",0,FALSE )) != -1 ) {
		_value = _data->right( _data->length()-pos-1 ).toInt( &ok,16 );
	}
	else _value = _data->toInt( &ok,10 );

	if( !ok ) {
		QString warning;
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s",
			section,subsection, key );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
	return _value;
}

double readDoubleConst( const char* section, 
			const char* subsection,const char* key,
			FILE *out_f=NULL ) {

	QString* _data  = maptree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();

	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	bool ok;
	double _value = _data->toDouble( &ok );

	if( ok ) return (_value);

	QString warning;
	warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s",
		section,subsection, key );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return( _value );
		default : return(0);
	}
}

double readDoubleConst( const char* section, const char* key,
			FILE *out_f=NULL ) {

	QString* _data = maptree->findBranch( section )
			->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,key,
			_data->data() );
	}

	bool ok;
	double _value = _data->toDouble( &ok );

	if( ok ) return (_value);

	QString warning;
	warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s",
		section, key );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return( _value );
		default : return(0);
	}
}

double readDoubleConst( const char* section, 
	const char* subsection,const char* key,double min, double max,
			FILE *out_f=NULL) {

	QString* _data = maptree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	bool ok;
	double _value = _data->toDouble( &ok );

	if( ok && (_value >= min) && (_value <= max) ) {
		return (_value);
	}

	QString warning;
	if( !ok )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s",
			section,subsection, key );
	if( _value < min )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s\n%f<%f!",
			section,subsection, key, _value, min );
	if( _value > max )
		warning.sprintf( "Invalid input:\ndevice.cfg\\%s\\%s\\%s\n%f>%f!",
			section,subsection, key, _value, max );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return( _value );
		default : return(0);
	}
}


const char* readStrConst( const char* section, 
	const char* subsection,const char* key, FILE *out_f=NULL ) {

	CFGLeaf* leaf = maptree->findBranch( section )->
		findBranch( subsection )->findLeaf( key );
	const char* data = leaf->getData()->data();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			data );
	}
	return data;
}

int leafDataToInt( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {

	QString* _data = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}
	bool ok;
	int _value = _data->toInt( &ok );
	if( !ok ) {
		QString warning;
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s",
			file,section,subsection, key );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
	return( _value );
}

int leafDataToInt( const char* file,const char* section,
			const char* subsection, const char* key,
			int _min, int _max, FILE *out_f=NULL ) {

	QString* _data = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	bool ok;
	int _value = _data->toInt( &ok );
	if( ok && (_value >= _min) && (_value <= _max) ) {
		return(_value);
	}

	QString warning;
	if( !ok )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s",
			file,section,subsection, key );
	if( _value < _min )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s\n%d<%d!",
			file,section,subsection, key, _value, _min );
	if( _value > _max )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s\n%d>%d!",
			file,section,subsection, key, _value, _max );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(_value);
		default : return(0);
	}
}

double leafDataToDouble( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {

	QString* _data = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	bool ok;
	double _value = _data->toDouble( &ok );
	if( !ok ) {
		QString warning;
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s",
			file,section,subsection, key );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
	return( _value );
}

double leafDataToDouble( const char* file,const char* section,
		const char* subsection, const char* key,
		double _min, double _max, FILE *out_f=NULL ) {

	QString* _data = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			_data->data() );
	}

	bool ok;
	double _value = _data->toDouble( &ok );
	if( ok && (_value >= _min) && (_value <= _max) ) {
		return(_value);
	}

	QString warning;
	if( !ok )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s",
			file,section,subsection, key );
	if( _value < _min )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s\n%f<%f!",
			file,section,subsection, key, _value, _min );
	if( _value > _max )
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s\n%f>%f!",
			file,section,subsection, key, _value, _max );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(_value);
		default : return(0);
	}
}

static const QString invForm = "Invalid format";
static const QString outRange = "Out of Range";

double leafDataToTimeSec( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {

	QString str = *(tree->findBranch( section )
		->findBranch( subsection )->findLeaf( key )->getData());
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			str.data() );
	}
	str = str.simplifyWhiteSpace();
	int startPos = 0;
	int spacePos = 0;
	double sign = 1.0;
	bool ok;
	if( str[0] == '+' ) { sign = 1.0; startPos = 1; }
	else if( str[0] == '-' ) { sign = -1.0; startPos = 1; }
	else startPos = 0;
	spacePos = str.find( ' ',startPos );
	debug( "startPos = %d spacePos = %d",startPos,spacePos );
	if( spacePos < 0 ) throw LoadError( invForm );
	int hour = str.mid( startPos,spacePos-startPos ).toInt( &ok );
	if( !ok ) throw LoadError( invForm );
	if( hour < 0 ) throw LoadError( invForm );

	startPos = spacePos + 1;
	spacePos = str.find( ' ',startPos );
	debug( "startPos = %d spacePos = %d",startPos,spacePos );
	if( spacePos < 0 ) throw LoadError( invForm );
	int min = str.mid( startPos,spacePos-startPos ).toInt( &ok );
	if( !ok ) throw LoadError( invForm );
	if( min < 0 ) throw LoadError( invForm );

	startPos = spacePos + 1;
	debug( "startPos = %d",startPos );
	double sec = str.right( str.length()-startPos ).toDouble( &ok );
	if( !ok ) throw LoadError( invForm );
	if( sec < 0 ) throw LoadError( invForm );

	debug( "longitude= %d %d %f",hour,min,sec );
	sec += ( (double)hour*60.0 + (double)min )*60.0;
	if( sign < 0 ) sec = SEC_IN_DAY - sec;
	if( (sec < 0) || (sec > SEC_IN_DAY) ) throw LoadError( outRange );
	debug( "sec=%f",sec );
	return( sec );
}

double leafDataToDegSec( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {

	QString str = *(tree->findBranch( section )
		->findBranch( subsection )->findLeaf( key )->getData());
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			str.data() );
	}
	str = str.simplifyWhiteSpace();
	double sign = 1.0;
	int startPos = 0;
	int spacePos = 0;
	bool ok;
	if( str[0] == '+' ) { sign = 1.0; startPos = 1; }
	else if( str[0] == '-' ) { sign = -1.0; startPos = 1; }
	else startPos = 0;

	spacePos = str.find( ' ',startPos );
	if( spacePos < 0 ) throw LoadError( invForm );
	int deg = str.mid( startPos,spacePos-startPos ).toInt( &ok );
	if( !ok ) throw LoadError( invForm );

	startPos = spacePos + 1;
	spacePos = str.find( ' ',startPos );
	if( spacePos < 0 ) throw LoadError( invForm );
	int min = str.mid( startPos,spacePos-startPos ).toInt( &ok );
	if( !ok ) throw LoadError( invForm );
	if( min < 0 ) throw LoadError( invForm );

	startPos = spacePos + 1;
	debug( "startPos = %d",startPos );
	double sec = str.right( str.length()-startPos ).toDouble( &ok );
	if( !ok ) throw LoadError( invForm );
	if( sec < 0 ) throw LoadError( invForm );

	debug( "latitude = %.1f %d %d %f",sign,deg,min,sec );
	sec += ( (double)deg*60.0 + (double)min )*60.0;
	debug( "sec=%f",sign*sec );
	if( sec > SEC_IN_90 ) throw LoadError( outRange );
	return( sign*sec );
}

int leafDataOnOffToInt( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {

	QString* str = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			str->data() );
	}
	if( operator==( str->upper(),"ON" ) ) return(1);
	if( operator==( str->upper(),"OFF" ) ) return(0);
	if( operator==( str->upper(),"OPEN" ) ) return(1);
	if( operator==( str->upper(),"CLOSE" ) ) return(0);

	QString warning;
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s",
			file,section,subsection, key );
	switch ( showStrongWarning( warning ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
}

int leafDataToCharInt( const char* file,const char* section,
		const char* subsection, const char* key,
			QString& _name, double& _foclen,
			FILE *out_f=NULL ) {
	int _valueInt = 0;
	bool ok1 = FALSE, ok2 = FALSE;

	_foclen = 0;
	QString* p_str = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			p_str->data() );
	}

	p_str->replace( QRegExp(","),QString(" ") );
	p_str->replace( QRegExp("\""),QString(" ") );
	QString str = p_str->simplifyWhiteSpace();

	int pos1 = str.find( " " );
	if( pos1 != -1 ) {
		_name = str.left(pos1);
		if( operator==( _name.upper(),"SPARE" ) ) return(0);

		int pos2 = str.find( " ",pos1+1 );
		if( pos2 != -1 ) {
			_foclen = str.mid( pos1,pos2-pos1 ).toDouble( &ok1 );
			_valueInt = str.right( str.length()-pos2-1 ).toInt( &ok2 );
		}
	} else {
		if( operator==( str.upper(),"SPARE" ) ) {
			_name="Spare";
			return(0);
		}
 		ok1 = FALSE;
	}
	if( !ok1 || !ok2 ) {
		QString warning;
		warning.sprintf( "Invalid input:\n%s\\%s\\%s\\%s:\n%s",
			file,section,subsection, key,p_str->data() );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
	return( _valueInt );
}

int testFileName( QString& _fileName, int _dialog ) {
	QFile _file( _fileName );
	if( _file.exists() ) return(1);
	if( !_dialog ) return(0);
	QString warning;
	warning.sprintf( "File \"%s\" not found",_fileName.data() );
	switch( showStrongWarning( warning ) ) {
		case QMessageBox::Abort	: myQAppExit(1);
		case QMessageBox::Ignore: return(0);
		default	: return(0);
	}
}

int testDirName( QString& _dirName, int _dialog  ) {
	QDir _dir( _dirName );
	if( _dir.exists() ) return(1);
	if( !_dialog ) return(0);
	QString warning;
	warning.sprintf( "Directory \"%s\" not found",_dirName.data() );
	switch( showStrongWarning( warning ) ) {
		case QMessageBox::Abort	: myQAppExit(1);
		case QMessageBox::Ignore: return(0);
		default	: return(0);
	}
}

QString* leafDataToStr( const char* file,const char* section,
			const char* subsection, const char* key,
			FILE *out_f=NULL ) {
	QString* str = tree->findBranch( section )->findBranch( subsection )
		->findLeaf( key )->getData();
	if( out_f ) {
		fprintf( out_f,"%s %s\\%s\\%s=%s\n",
			getStamp( "P ",getCurrentUT() ),
			section,subsection,key,
			str->data() );
	}
	return( str );
}

int showWarning( QString& str ) {
	QMessageBox mb( "Warning",str,
		QMessageBox::Warning,QMessageBox::Retry,
		QMessageBox::Ignore,QMessageBox::Abort );
	mb.setButtonText( QMessageBox::Abort, "Exit" );
	mb.setPalette( QPalette( WARNING_BUTTON_COLOR, WARNING_BACK_COLOR ) );
	return( mb.exec() );
}

int showStrongWarning( QString& str, const char* _ignore="Ignore",
				const char* _abort="Exit" ) {
	debug( "Strong Warning: %s",str.data() );
	QMessageBox mb( "Warning",str,
		QMessageBox::Warning,
		QMessageBox::Ignore,QMessageBox::Abort,0 );
	mb.setButtonText( QMessageBox::Ignore, _ignore );
	mb.setButtonText( QMessageBox::Abort, _abort );
	mb.setPalette( QPalette( WARNING_BUTTON_COLOR, WARNING_BACK_COLOR ) );
	return( mb.exec() );
}

int showStrongMessage( QString& str, const char* _ok="OK" ) {
	debug( "Strong Message: %s",str.data() );
	QMessageBox mb( "Warning",str,
		QMessageBox::Warning,
		QMessageBox::Ok,0,0 );
	mb.setButtonText( QMessageBox::Ok, _ok );
	mb.setPalette( QPalette( WARNING_BUTTON_COLOR, WARNING_BACK_COLOR ) );
	return( mb.exec() );
}

void showFatalError( QString& str ) {
	QMessageBox mb( "Error",str,
		QMessageBox::Critical,
		QMessageBox::Abort,0,0 );
	mb.setButtonText( QMessageBox::Abort, "Exit" );
	mb.setPalette( QPalette( WARNING_BUTTON_COLOR, WARNING_BACK_COLOR ) );
	mb.exec();

//	debug( "Fatal: %s",str.data() );
	myQAppExit(1);
}

int doubleTo255int( double val ) {
	int int_val = (int)rint( 255.0 * val );
	if( (int_val == 0) && (val > 0) ) int_val = 1;
	if( int_val < 0 ) int_val = 0;
	if( int_val > 255 ) int_val = 255;
	return( int_val );
}

int calcNumberDoubleInString( QString* p_str, const char* path ) {
	int i;
	int pos;
	bool ok;
	QString warning;
	QString str;

	if( !p_str ) return(0);

	p_str->replace( QRegExp(","),QString(" ") );
	str = p_str->simplifyWhiteSpace();
	int number = str.contains( " " ) + 1;

	for( i=0; i<number; i++ ) {
		pos = str.find( " " );
		if( pos != -1 ) str.left(pos).toDouble( &ok );
		else str.toDouble( &ok );
		if( !ok ) {
		warning.sprintf( "Invalid input:\n%s",path );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
			}
		}
		pos = str.find( " " );
		if( pos != -1 ) str.remove( 0,(uint)++pos );
	}
	return( number );
}

int stringToDoubleArray( QString* p_str, QArray<double>& arr,
						const char* path ) {
	int i;
	int pos;
	bool ok;
	QString warning;
	QString str;

	if( !p_str ) return(0);

	p_str->replace( QRegExp(","),QString(" ") );
	str = p_str->simplifyWhiteSpace();
	int number = str.contains( " " ) + 1;
	arr.resize( number );

	for( i=0; i<number; i++ ) {
		pos = str.find( " " );
		if( pos != -1 ) arr[i] = str.left(pos).toDouble( &ok );
		else arr[i] = str.toDouble( &ok );
		if( !ok ) {
		warning.sprintf( "Invalid input:\n%s",path );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
			}
		}
		pos = str.find( " " );
		if( pos != -1 ) str.remove( 0,(uint)++pos );
//		debug( "%d: %f",i,arr[i] );
	}
	return( number );
}

int stringToDoubleArray( QString* p_str, QArray<double>& arr,
		const char* path, double _min, double _max ) {
	int i;
	int pos;
	bool ok;
	QString warning;
	QString str;
	int _abort = 0;

	if( !p_str ) return(0);

	p_str->replace( QRegExp(","),QString(" ") );
	str = p_str->simplifyWhiteSpace();
	int number = str.contains( " " ) + 1;
	arr.resize( number );

	for( i=0; i<number; i++ ) {
		pos = str.find( " " );
		if( pos != -1 ) arr[i] = str.left(pos).toDouble( &ok );
		else arr[i] = str.toDouble( &ok );
		if( !ok ) {
			warning.sprintf( "Invalid input:\n%s",path );
			_abort = 1;
		}
		if( arr[i] < _min ) {
			warning.sprintf( "Invalid input:\n%s: %f<%f!",
			path, arr[i], _min );
			_abort = 1;
		}
		if( arr[i] > _max ) {
			warning.sprintf( "Invalid input:\n%s: %f>%f!",
			path, arr[i], _max );
			_abort = 1;
		}
		if( _abort ) {
			switch ( showStrongWarning( warning ) ) {
				case  QMessageBox::Abort : myQAppExit(1);
				case  QMessageBox::Ignore: _abort = 0; break;
				default : _abort = 0; break;
			}
		}
		pos = str.find( " " );
		if( pos != -1 ) str.remove( 0,(uint)++pos );
//		debug( "%d: %f",i,arr[i] );
	}
	return( number );
}

int stringToIntArray( QString* p_str, QArray<int>& arr, const char* path ) {
	int i;
	int pos;
	bool ok;
	QString warning;
	QString str;
	int val_int;

	if( !p_str ) return(0);

	p_str->replace( QRegExp(","),QString(" ") );
	str = p_str->simplifyWhiteSpace();
	int number = str.contains( " " ) + 1;
	arr.resize( number );

	for( i=0; i<number; i++ ) {
		pos = str.find( " " );
		if( pos != -1 ) val_int = str.left(pos).toInt( &ok );
		else val_int = str.toInt( &ok );
		if( !ok ) {
		warning.sprintf( "Invalid input:\n%s",path );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
			}
		}
		pos = str.find( " " );
		if( pos != -1 ) str.remove( 0,(uint)++pos );
		arr[i] = val_int;
//		debug( "%d: %d",i,arr[i] );
	}
	return( number );
}

int stringTo255intArray( QString* p_str, QArray<int>& arr, const char* path ) {
	int i;
	int pos;
	bool ok;
	QString warning;
	QString str;
	double val_double;

// !!!
	if( !p_str ) return(0);

	p_str->replace( QRegExp(","),QString(" ") );
	str = p_str->simplifyWhiteSpace();
	int number = str.contains( " " ) + 1;
	arr.resize( number );

	for( i=0; i<number; i++ ) {
		pos = str.find( " " );
		if( pos != -1 ) val_double = str.left(pos).toDouble( &ok );
		else val_double = str.toDouble( &ok );
		if( !ok ) {
		warning.sprintf( "Invalid input:\n%s",path );
		switch ( showStrongWarning( warning ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
			}
		}
		pos = str.find( " " );
		if( pos != -1 ) str.remove( 0,(uint)++pos );
		arr[i] = doubleTo255int( val_double );
//		debug( "%d: %d",i,arr[i] );
	}
	return( number );
}

void myQAppExit( int retcode=0 ) {
// Ne rabotaet eto...
//	debug( "myQAppExit:" );
	retcode = 1;
	devmanager.park();
	exit( retcode );
}
