#ifndef TREE_H
#define TREE_H
#include <qobject.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qmessagebox.h>

class CFGTree;
class CFGBranch;
class CFGLeaf;

extern CFGLeaf leaf_empty;
extern CFGBranch branch_empty;

class Link {
private:
	Link* next;
	char id;	// 'l' or 'b' - leaf or branch
	void* data;	//  ,     CFGLeaf or CFGBranch

public:
//	Link( char _id, void* _data, CFGBranch* _branch );	
//	Link( char _id, void* _data, CFGTree* _tree );
	void initLink( char _id, void* _data, CFGBranch* _branch );	
	void initLink( char _id, void* _data, CFGTree* _tree );		

	virtual ~Link( void );
	const char* BranchName( void );
	const char* getLeafKey( void );
	char* getLeafData( void );
	char getId( void )		{ return id; }
	CFGBranch* getBranch( void ) {if(id=='b') return((CFGBranch*)data); return NULL; }

	CFGLeaf* getLeaf( void ) {if(id == 'l') return((CFGLeaf*)data); return NULL;}
	Link* getNext( void ) { return next; }
	void setNext( Link* _next ) { next = _next; }
};

class CFGLeaf {

private:
	const char *key;
	QString data;
	const char* validator_type;
	bool enable_change;
	QString help;

public:
//	CFGLeaf( const char* _key );
//	void initLeaf( const char* );
	void initLeaf( const char*, bool _enable_change_=TRUE );
	void initLeaf( const char*, const char*, bool _enable_change_=TRUE );
//	void initLeaf( const char*, const char*, bool );
	virtual ~CFGLeaf();

	bool isChangeEnabled( void ) { return enable_change; }
	const char* getKey( void ) { return key; }
	QString* getData( void ) { return &data; }
	QString* getHelp( void ) { return &help; }
	void setHelp( const char* );
	int replaceData( const QString& data_ );
	const char* getValidatorType() { return validator_type; }
};

class CFGBranch {

private:
	const char *name;
	Link *first_link; //    ,    link
	Link *last_link;
	Link *current_link;
	QString help;
	int n_leaf;		//         
	int n_subbranch; //     

public:
//	CFGBranch();
//	CFGBranch( const char* _name );
	void initBranch( const char* _name );
//	CFGBranch( const char* _name, Link* _first_link );

	virtual ~CFGBranch( void );
	void addLink( Link* );
	QString* getHelp( void ) { return &help; }
	void setHelp( const char* );
	CFGBranch* findBranch( const char* );
	CFGLeaf* findLeaf( const char* ); //     
	QString* findLeaf_str( const char* ); //       
	char* findLeaf_ch( const char* );
	int replaceLeaf( const char*,const char* );
	int setLeafHelp( const char*,const char* );
	void toFirst( void ) { current_link = first_link; }
	void toNext( void ) { current_link = current_link->getNext(); }
	Link* getCurrentLink( void ) { return current_link; }
	const char* getName( void ) { return name; }
	int getNLeaf( void ) { return n_leaf; }
	int getNSubBranch( void ) { return n_subbranch; }
	int copyBranch( CFGBranch*,CFGBranch* );
};                        

class CFGTree : public QObject {
	Q_OBJECT
private:
	Link* first_link;
	Link* current_link;
	Link* last_link;
	int _changed;

public:
	CFGTree( void );
//	CFGTree( Link* _first_link );
	virtual ~CFGTree( void );
	void addLink( Link* );
	CFGBranch* findBranch( const char* );
	void toFirst( void ) { current_link = first_link; } 
	void toNext( void ) { current_link = current_link->getNext(); }
	Link* getCurrentLink( void ) { return current_link; }
	const char* CurrentName( void ) { return current_link->getBranch()->getName(); }
	int copyTree( CFGTree*,CFGTree* );
	int readCfg ( const char* );
	int saveToFile( const char* );
	void emitSignalChanged( void );
	int isChanged( void ) {return( _changed ); }
	void setSaved( void ) {_changed = 0;}
	void setChanged( void ) {_changed = 1;}
signals:
	void changed( void );
};


#endif

