#ifndef WIDGETS_H
#define WIDGETS_H

#include <qpushbutton.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qlineedit.h>
#include <qmultilinedit.h>
#include <qcombobox.h>
#include <qtableview.h>
//#include <qtable.h>
#include <qtextview.h>
#include <qlcdnumber.h>
#include <qarray.h>

#include "device.h"
#include "viewstar.h"
#include "jkexport.h"

class QJKPushButton : public QPushButton {
public:
	QJKPushButton( const char*, QWidget*, const char* name=0 );
	~QJKPushButton();
protected:
	void focusInEvent( QFocusEvent* );
	void focusOutEvent( QFocusEvent* );
};

class QJKTable : public QTableView {
//private:
//	bool isColumnSizeChanged;
public:
	QJKTable( QWidget* parent=0,
			const char* name=0,
			int numRows=MAX_ROW,
			int numCols=MAX_COL,
			bool grid=false );
	~QJKTable();

	void setCellContent( int row, int col, const QString& );

	void fillTable( void );
	void fitToFont( void );

protected:
	void paintCell( QPainter*, int row, int col );
	virtual void resizeEvent( QResizeEvent* );
	
private:
	int indexOf( int row, int col ) const;
	QString* contents;
	bool showGrid; 

	void setCellGeometry( void );
};

class QJKTextView : public QTextView {
private:
	QString text;

public:
	QJKTextView( QWidget*, const char* name=0 );
	~QJKTextView();

	void addLine( const QString& );
};

class QJKMainView : public QWidget {
private:
	QJKTable *table;
	QLineEdit *textView;

	void setChildrenGeometry( void );
public:
	QJKMainView( QWidget* parent, const char* name=0 );
	~ QJKMainView();
	void fitToFont( void );
	void fillTable();
	void showTextLine();

protected:
	virtual void resizeEvent( QResizeEvent* );
};

class TalkDialog : public QDialog {
	Q_OBJECT
public:
	TalkDialog( void );
	~TalkDialog();
private:
	QVBoxLayout* layout;
	QLabel* label;
	QJKPushButton* closebutton;
	QComboBox* combo;
	QMultiLineEdit* edit;
private slots:
	void slotTalk( const QString& );
};

class AdjustDialog : public QDialog {
	Q_OBJECT
public:
	AdjustDialog( void );
	~AdjustDialog();
	void update( void );

private:
	int lastAperture;
	QGridLayout* layout;
	QLabel *label, *label_move, *label_apert, *label_correct;
	QLineEdit *result_window;
	QJKPushButton *acceptbutton, *cancelbutton;
	QJKPushButton *move_plus,*move_minus;
	QComboBox *combo_apert;

private slots:
	void movePlus( void );
	void moveMinus( void );
	void setCurrentItem( int );
	void setAperture( const QString& );
};

class ShowStarDialog : public QDialog {
public:
	ShowStarDialog( int );
	~ShowStarDialog();
	void update( int );
	void showHA( void );

private:
	int _showHA;
	QGridLayout* layout;
	QLabel *labelName,*labelHA,*labelDec,*labelMagV,*labelSp,*labelDupl;
	QJKPushButton *closeButton;
};

class SelectStarDialog : public QDialog {
	Q_OBJECT
public:
	SelectStarDialog( void );
	~SelectStarDialog();
	void update( void );

private:
	QGridLayout* layout;
	QLabel* label;
	ViewStarList* viewStarList;
	QJKPushButton *selectbutton, *cancelbutton;

private slots:
	void select( void );
	void select( QListViewItem* );
};

class IllumDialog : public QDialog {
	Q_OBJECT
public:
	IllumDialog( void );
	~IllumDialog();
	void update( void );

private:
	QGridLayout* layout;
	QLabel* label;
	QLineEdit* bright_window;
	QJKPushButton *bright_plus,*bright_minus, *closebutton;
	void setBrightness( void );

private slots:
	void setBrightnessUp( void );
	void setBrightnessDown( void );
};

class BackCorrectionDialog : public QDialog {
	Q_OBJECT

private:
	QGridLayout* layout;
	QLabel *label,*label_cf,*label_l,*label_r,*label_av;
	QLabel *label_a,*label_b,*label_c,*label_d;
	QLineEdit* edit_l[MAX_N_COUNTERS];
	QLineEdit* edit_r[MAX_N_COUNTERS];
	QLineEdit* edit_av[MAX_N_COUNTERS];
	QJKPushButton *acceptbutton, *cancelbutton;
	QJKPushButton *stopbutton, *repeatbutton;

	int timer;
	matrix43 coef;

public:
	BackCorrectionDialog( void );
	~BackCorrectionDialog();
	void update( void );
	void finish( void );
	void setButtonsEnabled( bool );

protected:
	void timerEvent( QTimerEvent* );

private slots:
	void acceptCorrection( void );
	void correct( void );
	void showResult( void );
};

class CenterCorrectionDialog : public QDialog {
	Q_OBJECT

private:
	QGridLayout* layout;
	QLabel *label,*label_x,*label_y;
	QLineEdit* edit_x;
	QLineEdit* edit_y;
	QJKPushButton *acceptbutton, *cancelbutton;
	QJKPushButton *stopbutton, *repeatbutton;

	int timer;
	QPoint	delta;

public:
	CenterCorrectionDialog( void );
	~CenterCorrectionDialog();
	void update( void );
	void finish( void );
	void setButtonsEnabled( bool );

protected:
	void timerEvent( QTimerEvent* );

private slots:
	void acceptCorrection( void );
	void correct( void );
	void showResult( void );
};

class DigitalClock : public QLCDNumber {
	Q_OBJECT
public:
	DigitalClock( QWidget* parent=0,const char *name=0 );
	~DigitalClock();
public slots:
	void showTime();
};

#endif