#define QT_THREAD_SUPPORT

#include <math.h>
#include <qthread.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "scind.hxx"
#include "device.h"
#include "names.h"
#include "devplus.h"
#include "tools.h"
#include "jkexport.h"
#include "nrutil.h"
#include "atmos.hxx"
#include "weif.hxx"
#include "unroll.h"

#include <mcua.h>
#include "qapplication.h"

static int count_cycle_condition	= 1;
static int data_cycle_condition		= 1;

static QString str_message, str_edit, str_edit1, str_edit2, str_add;

//static const char*	cfgFile		= "usr/lib/turbina/turbina.cfg";
//static const char*	dataPath	= "/usr/share/turbina/";
//static const char*	indexFileExt	= "mass";
//static const char*	logFileExt	= "log";
//static const char*	countsFileExt	= "cnt";

static const char*	nameA		= "A";
static const char*	nameB		= "B";
static const char*	nameC		= "C";
static const char*	nameD		= "D";

static const char*	scenarioDict	= "NGBCFDSX";

static const QString emptyLine		= "";
static const QString notClosed		= "Input Door is not closed!";
static const QString errAperture	= "Error in Set Aperture!";
static const QString errShiftApert	= "Error in Shift Aperture!";
static const QString errLight		= "Error in Set Control Light!";
static const QString errCounter		= "Error in Counter setting!";
static const QString errLevel		= "Error in set Discrimination!";
static const QString errVoltage		= "Error in set Voltage!";
static const QString errScan		= "Too many errors in scan!";
static const QString userStop		= "User's stop!";
static const QString abortProcess	= "Measurement aborted!";
static const QString successfully	= "It's OK";
static const QString fluxError		= "Invalid fluxes!";
static const QString nonPoisError	= "Invalid P-parameter!";
static const QString overLightText	= "Overlight! HV turned OFF!";
static const QString badIndex		= "Zero scintillation indices!";
static QString invalid			= "Invalid";

static const QString normalName		= "Normal ";
static const QString generalName	= "Generalized ";
static const QString detTestName	= "Detect Test ";
static const QString statTestName	= "Statis Test ";
static const QString estimName		= "Flux ";
static const QString indTestName	= "Individ Test ";
static const QString commonTestName	= "Exchang Test ";
static const QString detCountMeasName	= "Counting ";
static const QString detStatMeasName	= "Statis Meas ";
static const QString backCorrName	= "Backgr Cor";
static const QString backMeasName	= "Background ";
static const QString centerCorName	= "Center Cor ";
static const QString starCenterName	= "Centering ";
static const QString focusContrName	= "Focus ";

static char to_proc_file[11] = {'0','1',' ','0','2',' ','0','3',' ','0','4' };
static const char* DEV_SIGNAL_ABBR[] = {"ACK","NAK","NOD","ACN","ACY","ACW",
						"DSC","BUS","COL"};
static unsigned char DEV_SIGNAL_CODE[] = {ACK,NAK,NOD,ACN,ACY,ACW,
						DSC,BUS,COL};
static int N_DEV_SIGNAL = 9;

static int _count_mode = NORMAL_MODE;
static int _old_count_mode;

static FILE *srv_file=NULL, *counts_file=NULL;
extern FILE *out_file;

static int n_run = 0;

static int block_number = 512; // = 50;
static int max_block_size;	// yesli prishel udlinennyi paket, on ne
			// dolzhen byt' dlinneye etogo

extern DevManager	devmanager;
extern Star		star;
extern ExportTable	table;
extern CFGTree*		tree;
extern CFGTree*		maptree;
extern QTime		_programStartTime;

static int n_counters = 1;

static QString response;

QMutex userStopMutex;
extern QMutex countMutex;
extern QMutex dataMutex;
extern QMutex scenarioMutex;
extern QMutex infoExportMutex;
extern QMutex stateExportMutex;

static unsigned char respbuf[MSG_SIZE];
static unsigned char *msgdata;

static int id_out=0;
static int id_stat=0;
static int id_data=0;
static int id_proc=0;
static int count_timeout_flag = 0;
static int n_read_data;

void jkwait( int msec ) {

	infoExportMutex.lock();
	InfoExport::message = "Wait " +
		str_add.setNum( msec/S2MS ) + " sec...";
	InfoExport::ms = msec;
	InfoExport::isMessage = true;
	infoExportMutex.unlock();

	ulong mksec = ulong(msec*MS2MKS);
	usleep( mksec );
/*
	QTime t;
	t.start();
	while( t.elapsed() < msec );
*/
}

void timeout_action( void ) {
	count_timeout_flag = 1;
	dataMutex.unlock();
}

void *read_data( void ) { //  
	int size;

	debug( "+++++++++ read_data: Start..." );
	n_read_data = 0;

	int count_byte = 0;
	while( data_cycle_condition ) {
		while( n_read_data < n_counters*block_number ) {
			count_byte = read( id_data, msgdata,1 );
			if( count_byte == 0 ) {
				usleep( 10000 );
				continue;
			}
			size = (int)msgdata[0];
			if( size > max_block_size ) size = max_block_size;
			read( id_data, msgdata+1, size );

			devmanager.putData();
/*
			if( srv_file ) {
				for( int t=0; t<=size; t++ ) {
					fprintf( srv_file,"%02X ",msgdata[t] );
				}
				int _blnumber = (int)(msgdata[3] + (msgdata[4] << 8));
				fprintf( srv_file," %d %d\n",n_read_data,_blnumber );
			}
*/
			n_read_data++;
		}
		n_read_data = 0;
//		debug( "read_data: dataMutex try unlock..." );
		dataMutex.unlock();
//		debug( "read_data: dataMutex unlocked!" );
	}
	dataMutex.unlock();
	int res = 7;
	return( &res );
}

void DevManager::putData( void ) {
	Counter *currentCounter;
	int dnumb;
	int t;
	int size;
	int ok = 1;

	if( !isCounterExist() ) return;

	if( (char)msgdata[2] == _moduleA ) 
		currentCounter = &(counters[0]);
	else if( (char)msgdata[2] == _moduleB ) 
		currentCounter = &(counters[1]);
	else if( (char)msgdata[2] == _moduleC ) 
		currentCounter = &(counters[2]);
	else if( (char)msgdata[2] == _moduleD ) 
		currentCounter = &(counters[3]);
	else {
		currentCounter = NULL;
		if( srv_file ) fprintf( srv_file, "Unknown counter with address %X\n",msgdata[2] );
	}

	size=(int)msgdata[0];

// error 1:
	if( (size=(int)msgdata[0]) != (_dataBlockSize0-1) ) {
		ok = 0;
//		_size_errors++;
	}

	int _blnumber = (int)(msgdata[3] + (msgdata[4] << 8));
	if( currentCounter ) {
		dnumb = abs( _blnumber - currentCounter->blockCounter.get() );
		if( dnumb != 0 ) {
			if( srv_file ) fprintf( srv_file,"%s block %02X-1 %02X skipped\n",
				currentCounter->getName(),msgdata[3],msgdata[4] );
			currentCounter->blockCounter.set( _blnumber );
//			_number_errors += dnumb;
		}

		if( ok ) {
			if( size > (_dataBlockSize0-1) ) size = _dataBlockSize0 - 1;
			currentCounter->addBlockLabel( _blnumber );
			for( t=HEAD_SIZE; t<=size; t=t+2 ) {
				count_t val = msgdata[t]+(msgdata[t+1] << 8);
				currentCounter->putData( val );
			}
			currentCounter->blockCounter.incr();
		} else {
//			debug( "!!!!!!!!!!!!!!%s: Skip block !!!!!!!",
//				currentCounter->getName() );
			if( srv_file ) fprintf( srv_file,"%s Skip block %02X %02X\n",
				currentCounter->getName(),msgdata[3],msgdata[4] );
		}
	}
}

void* count_cycle( void ) {
	debug( "!!!!!!!!!!!!!!!!!!!!!! count_cycle: " );
	while( count_cycle_condition ) {
		debug( "count_cycle: try lockcount..." );
		countMutex.lock();
		debug( "count_cycle: lockcount" );
		switch ( _count_mode ) {
			case NORMAL_MODE             : devmanager.normal_mode(); break;
			case GENERALIZED_MODE        : devmanager.generalized_mode(); break;
			case EXCHANGE_TEST_MODE      : devmanager.exchange_test_mode(); break;
			case DETECTOR_TEST_MODE      : devmanager.detector_test_mode(); break;
			case STATISTIC_TEST_MODE     : devmanager.statistic_test_mode(); break;
			case INITIAL_TESTS_MODE      : devmanager.initial_tests_mode(); break;
			case DETECTOR_COUNTING_MODE  : devmanager.detector_counting_mode(); break;
			case DETECTOR_STATISTIC_MODE : devmanager.detector_statistic_mode(); break;
			case BACKGROUND_CORR_MODE    : devmanager.background_corr_mode(); break;
			case FLUX_ESTIMATION_MODE    : devmanager.flux_estimation_mode(); break;
			case CENTERING_CORR_MODE     : devmanager.centering_corr_mode(); break;
			case FOCUS_CONTROL_MODE      : devmanager.focus_control_mode(); break;
			case STAR_CENTERING_MODE     : devmanager.star_centering_mode(); break;
			case BACKGROUND_MEASUR_MODE  : devmanager.background_measur_mode(); break;
			case SCENARIO_MODE           : devmanager.scenario_mode(); break;
			default : debug( "!!!!!!!! Invalid _count_mode = %d while=%d",
					_count_mode,count_cycle_condition ); break;
		}
	}
	countMutex.unlock();
	debug( "count_cycle: unlocked..." );
	int res = 13;
	return( &res );
}


void DevManager::terminateCountMode( int success ) {
	endCounting();
//	_count_mode = _old_count_mode;
	debug( "DevManager::terminateCountMode: change mode: " );
}

void DevManager::exchange_test_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runExchangeTest();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::detector_test_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runDetectorTest();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::statistic_test_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runStatisticTest();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::detector_counting_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runDetectorCountingMeasurement();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::detector_statistic_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runDetectorStatisticMeasurement();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::background_corr_mode( void ) {
	int success = runBackgroundCorrection();
	terminateCountMode( success );
}

void DevManager::background_measur_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runBackgroundMeasurement();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::centering_corr_mode( void ) {
	int success = runCenterCorrection();
	terminateCountMode( success );
}

void DevManager::normal_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runNormal();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::flux_estimation_mode( void ) {
	int success = runFluxEstimation();
	terminateCountMode( success );
}

void DevManager::generalized_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runGeneralized();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::focus_control_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runFocusControl();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

void DevManager::star_centering_mode( void ) {
	int _oldAperture = currentAperture();
	int success = runStarCentering();
	setAperture( _oldAperture,0 );
	terminateCountMode( success );
}

double DevManager::getAccumTime( char mode ) {
	switch( mode ) {
		case NORMAL_MODE            : return _normalAccumTime;
		case GENERALIZED_MODE       : return _generalAccumTime;
		case FOCUS_CONTROL_MODE     : return _focusAccumTime;
		case STAR_CENTERING_MODE    : return _centerAccumTime;
		case BACKGROUND_MEASUR_MODE : return _backgroundTime*2;
		case BACKGROUND_CORR_MODE   : return _backgroundTime*4;

		case EXCHANGE_TEST_MODE     : return _exchangeTestTime;
		case DETECTOR_TEST_MODE     : return _detectorTestTime;
		case STATISTIC_TEST_MODE    : return _normalAccumTime
							+LIGHT_WAIT_INTERVAL/S2MS*
							_nStatisticTestLight;
		case DETECTOR_COUNTING_MODE	: 
				return( _detectorCountingTime*
					_nDetectorCountingDiscrimination*
					_nDetectorCountingLight*
					_nDetectorCountingVoltage
					+LIGHT_WAIT_INTERVAL/S2MS*
					_nDetectorCountingLight
					+HV_WAIT_INTERVAL/S2MS*
					_nDetectorCountingVoltage );
		case DETECTOR_STATISTIC_MODE	:
				return ( _detectorStatisticTime*
					_nDetectorStatisticLight
					+LIGHT_WAIT_INTERVAL/S2MS*
					_nDetectorStatisticLight );
		case FLUX_ESTIMATION_MODE	: return _estimationAccumTime;
		default : debug( "+++!!!!!!!! Invalid _count_mode = %d",_count_mode ); return(0.0);
	}
}

void DevManager::scenario_mode ( void ) {
	int i;
	int success=0;

	int _oldAperture = currentAperture();
	if( !scenarioMutex.locked() ) {
		debug( "try lock scenarioMutex..." );
		scenarioMutex.lock();
		debug( "scenarioMutex locked" );
	}

// Calculation total scenario time:

	double sec = 0;
	for( i=0; i<(int)(currentScenario.size()); i++ ) {
		sec += getAccumTime( currentScenario[i] );
	}

	isScenario = true;
	for( i=0; i<(int)(currentScenario.size()); i++ ) {
		sec -= getAccumTime( currentScenario[i] );
		if( i == 0 ) {
			openScenario( getScenarioName(_currentScenarioNumber),sec );
		} else	setRemainTimeScenario( sec ); // do konza mody
// setMode dobavit siuda vremia do konza tekuchei mody

		debug( "i=%d scenario=%d",i,currentScenario[i] );
		if( _stopScenario ) break;
		if( _pauseScenario ) {
			endCounting();
			debug( "Pause: try lock scenarioMutex..." );
			scenarioMutex.lock();
			debug( "scenarioMutex locked" );
			startCounting();
			_pauseScenario = 0;
		}
		success = 0;
		_count_mode = currentScenario[i];
		_old_count_mode = currentScenario[i-1];
		switch( _count_mode ) {
			case NORMAL_MODE            : success=runNormal(); break;
			case GENERALIZED_MODE       : success=runGeneralized(); break;
			case FOCUS_CONTROL_MODE     : success=runFocusControl(); break;
			case STAR_CENTERING_MODE    : success=runStarCentering(); break;
			case BACKGROUND_MEASUR_MODE : success=runBackgroundMeasurement(); break;
			case EXCHANGE_TEST_MODE		: success=runExchangeTest(); break;
			case DETECTOR_TEST_MODE		: success=runDetectorTest(); break;
			case STATISTIC_TEST_MODE	: success=runStatisticTest(); break;
			case DETECTOR_COUNTING_MODE	: success=runDetectorCountingMeasurement(); break;
			case DETECTOR_STATISTIC_MODE	: success=runDetectorStatisticMeasurement(); break;
			case FLUX_ESTIMATION_MODE	: success=runFluxEstimation(); break;
			default : debug( "Scenario:+++!!!!!!!! Invalid _count_mode = %d",
				currentScenario[i] ); break;
		}
		_old_count_mode = _count_mode;
		if( success != 1 ) break;
	}
	isScenario = false;

	setAperture( _oldAperture,0 );

	terminateCountMode( success );
	closeScenario();
}

void DevManager::initial_tests_mode( void ) {
	int success = 1;
	int success1 = 1;
	int success2 = 1;
	int success3 = 1;

	if( _exchangeTest || _detectorTest || _statisticTest ) {
		if( turnDoor( OFF,0,false ) != 1 ) {
			sendToScreen( notClosed );
			goto ABORT_INIT_TEST;
		}
	}

	if( _stopScenario ) goto ABORT_INIT_TEST;
	if( _exchangeTest ) {
		_count_mode = EXCHANGE_TEST_MODE;
		success1 = runExchangeTest();
	}
	if( _stopScenario ) goto ABORT_INIT_TEST;
	if( _detectorTest ) {
		_count_mode = DETECTOR_TEST_MODE;
		success3 = runDetectorTest();
	}
	if( _stopScenario ) goto ABORT_INIT_TEST;
	if( _statisticTest ) {
		_count_mode = STATISTIC_TEST_MODE;
		success2 = runStatisticTest();
	}

	if( success1 && success2 && success3 ) success=1;
	else success = 0;

ABORT_INIT_TEST:

	turnDoor( _inputDoor,0,false );	// startovoe polozhenie
	setAperture( _initialStop,0 );	// tozhe
	terminateCountMode( success );
}

int DevManager::calcTimeoutInterval( double expos, int number ) {
	int interval = (int) ceil( expos * (double)number
		* TM_PREC + (double)_dataBlockSize * expos );
	if( srv_file ) fprintf( srv_file,"Timeout Interval=%d\n",interval );
	return( interval );
}

/**
	@brief	Convert an argument into its character hexadecimal 
	representation with leading '@'
	@param	addr	Argument (normally, binary file position)
	@return C-string with '@'+hex(addr)
*/
const char * getAddrStr( long addr ) {
	// maximal long number in hex is 7fffffff
	const int maxAddrLength = 8;
	static char strAddr[1+maxAddrLength+1] = "@7fffffff" ;// foo example
	if (addr>=0) {
		sprintf(strAddr,"@%lx",addr);
		return strAddr;
	} else {
		// if addr is negative (invalid) - return empty string
		return "";
	}
}

void DevManager::setStartBaseTime( void ) {
	_startBaseTime = getCurrentUT();
	if( srv_file ) fprintf( srv_file,"StartTime: %s\n",
		_startBaseTime.toString().data() );
}

QTime DevManager::setMidTime( QTime startTime ) {
	QTime stopTime = getCurrentUT();
	int secInterval = startTime.secsTo( stopTime );
	if( secInterval < 0 ) {	// if cherez polnoch'
		secInterval += int(SEC_IN_DAY);
	}
	QTime midTime = startTime.addSecs(
		secInterval / 2 );
	if( srv_file ) fprintf( srv_file,
	"StartAccum: %s StopAccum: %s MidAccum: %s\n",
		startTime.toString().data(),
		stopTime.toString().data(),
		midTime.toString().data() );
	return( midTime );
}

bool DevManager::testUserStop( void ) {
	bool stop;

	userStopMutex.lock();
	stop = isUserStop;	
	userStopMutex.unlock();

	if( stop ) {
		sendToScreen( userStop );
		return true;
	} else return false;
}

int DevManager::runNormal( void ) {
	int jj,iCounter;
	int *p_n_run = &n_run;
	int _abort = 0;
	QTime startAccumTime;
	char * checkfile = "atmos.tmp" ; /* File to put the info on 
		decomposition quality and coefficients, e.g. tmpFile from CFG */

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( normalName,getAccumTime( _count_mode ) );

	disablePooling();

	debug( "_________ NORMAL_MODE:__________" );

	n_run = (int)ceil( _normalAccumTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;

	int expositionNumber;
	int timeoutInterval=4000;

	debug( "old_count_mode=%d", _old_count_mode );

//	sendFluxHead( ( (_old_count_mode != NORMAL_MODE) &&
//			(_old_count_mode != GENERALIZED_MODE) ),
//			1,
//			"CFLUX" );

	sendFluxHead( !( isScenario || (_old_count_mode == _count_mode) ||
			(_old_count_mode == GENERALIZED_MODE) ||
			(_old_count_mode == NORMAL_MODE) ),
			1,
			"CFLUX" );
	debug( "countMode = %d oldMode = %d",
		_count_mode,_old_count_mode );

	fprintf( out_file,"%s %s\n",getStamp("M ",getCurrentUT() ), 
		normalName.data() );

	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_NORM;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_NORM;
	}

	/* Initialize SCIND module: */
	sc::init( expositionNumber, n_run, n_counters );
	sc::writeidx( out_file,sc::HEAD, 
		"#i   UT   ", 
		"Address" );


	/* Update the ATMOS module: */

	atm::update( star.getWFName().data(), checkfile, 
			_zShiftNormal, _zShiftGeneralSecond );
	atm::alloc(n_run) ;
	if( nr::erget() ) {
		str_edit = nr::ermessage().c_str();
		debug( "%s",str_edit.data() );
		sendToScreen( str_edit );
		nr::erreset();
		_abort = 1;
		goto ABORT_NORM;
	}
	/* Update the parameters of observed object: */ 

	star.updateCurrent() ;

	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}
	if( setAperture( _normalApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort = 1;
		goto ABORT_NORM;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	for( jj=0; jj<(*p_n_run); jj++ ) {
		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendToScreen( errCounter );
			_abort = 1;
			goto ABORT_NORM;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();
		long addr = -1;
		cleanData( expositionNumber, &addr );

		sc::compute( expositionNumber, _nCorrected,
			_normalExposition, sc::MODENORM );

		sc::writeidx( out_file,sc::DSI,
			getStamp("i ",_startBaseTime), getAddrStr(addr) );

		for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			fluxToTable( 1,
				iCounter,
				sc::getmean( true,iCounter ) 
				);
		}
		sendTable( 1,1,1,4 );
	/* Compute Cn2-integrals: */
		int meas = sc::getcurmeas() -1 ;
		atm::calcint(
			sc::getidxptr(meas,sc::DSI),	// non-shifted (D)SI
			0 );	// shifted (D)SI - absent 
		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			nr::erreset();
			_abort = 1;
			goto ABORT_NORM;
		}

		setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );

//		if( *p_n_run == 0 ) {
		if( testUserStop() ) {
//			sendToScreen( userStop );
			_abort=1;
			goto ABORT_NORM;
		}
	} // jj-cycle
ABORT_NORM:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {

		QTime midAccumTime = setMidTime( startAccumTime );
		debug( "Normal: write idx" );

		if( accumResume( midAccumTime,false ) != 1 ) {
			_abort = 1;
		} else {
//			sendToScreen( successfully );
		}

		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "!!!%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
			_abort = 1;
		}
	}
	closeMode();
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

bool DevManager::cleanData( int expositionNumber, long *addr = 0 ) {

	// Detect data loss anywhere:
	bool dataLost = false ;
	// Detect data loss in counters individually:
	bool counterDataLost[MAX_N_COUNTERS];
	count_t * countDataPtr[MAX_N_COUNTERS] ;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		counterDataLost[i] = (counters[i]).expandData();
		if (counterDataLost[i]) dataLost = true ;
		countDataPtr[i] = (counters[i]).getData() ;
	}
	if ( addr ) {
		if ( counts_file )
			// cleanData with ("write count!") and count file writing is ON
			*addr = ioc::write(counts_file, countDataPtr,
			(size_t)expositionNumber, MAX_N_COUNTERS) ;
		else
			*addr = -1 ;	// Count-file is not opened: return invalid address	
	}

	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		if( counterDataLost[i] )
			_nCorrected[i] = (counters[i]).repairData();
		else
			_nCorrected[i] = 0;

//		debug( "%s: _nData=%d",(counters[i]).getName(),
//					(counters[i]).getOutNumber() );
		if( srv_file ) {
			fprintf( srv_file,"%s: _nCorrected=%d\n",(counters[i]).getName(),
				_nCorrected[i] );
//			fprintf( srv_file,"%s: _nData=%d\n",(counters[i]).getName(),
//				(counters[i]).getOutNumber() );
		}
	}
	if( srv_file ) fprintf( srv_file,"\n" );

/*	if( counts_file ) {
		int _minNData = (counters[0]).getOutNumber();
		for( int j=0; j<_minNData; j++ ) {
			for( int i=0; i<MAX_N_COUNTERS; i++ ) {
				fprintf( counts_file, "%5d",
					(counters[i]).getData()[j] );
			}
			fprintf( counts_file,"\n" );
		}
	}
*/
	return dataLost;
}

int DevManager::runGeneralized( void ) {
	int jj;
	int *p_n_run = &n_run;
	QTime startAccumTime;
	int timeoutInterval = 4000;

	int _abort = 0;
	int _apert = -1;
	int _blockNumber = -1;
	int expositionNumber;
	double _elapsSec = 0.0;

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( generalName,getAccumTime( _count_mode ) );

	disablePooling();


	debug( "_________ GENERALIZED_MODE:__________" );

	n_run = 1 + 2 * (int)ceil( _generalAccumTime / 
			( _generalBaseTime + _generalSecondBaseTime ) );
	
	n_counters = MAX_N_COUNTERS;
	
	debug( "old_count_mode=%d", _old_count_mode );
	sendFluxHead( !( isScenario || (_old_count_mode == _count_mode) ||
			(_old_count_mode == NORMAL_MODE) ||
			(_old_count_mode == GENERALIZED_MODE) ),
			1,
			"CFLUX" );

	debug( "countMode = %d oldMode = %d",
		_count_mode,_old_count_mode );

//	sendFluxHead( ( (_old_count_mode != NORMAL_MODE) &&
//			(_old_count_mode != GENERALIZED_MODE) ),
//			1,
//			"CFLUX" );
	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ), 
		generalName.data() );
	{	// beginning of block to isolate goto-s
	
	if( setExpos( _generalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_GEN;
	}

	/* Initialize SCIND module: */
	int maxncount = calcExpositionNumber (
		(_generalBlockNumber > _generalSecondBlockNumber) ?
		_generalBlockNumber : _generalSecondBlockNumber );
		
	sc::init( maxncount, n_run, n_counters );
	sc::writeidx( out_file,sc::HEAD,
		"#i   UT   ", "Address" );

	/* Update the ATMOS module: */
	char * checkfile = "atmos.tmp" ; /* see ::runNormal */

	atm::update( star.getWFName().data(), checkfile, 
			_zShiftGeneral, _zShiftGeneralSecond );
	atm::alloc(n_run) ;
	if( nr::erget() ) {
		str_edit = nr::ermessage().c_str();
		debug( "%s",str_edit.data() );
		sendToScreen( str_edit );
		nr::erreset();
		_abort = 1;
		goto ABORT_GEN;
	}
	/* Update the parameters of observed object: */ 

	star.updateCurrent() ;

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	int _regime = 1;
	for( jj=0; jj<(*p_n_run); jj++ ) {

		switch( _regime ) {
			case 1	:	_apert = _generalApert;
						_blockNumber = _generalBlockNumber;
						_elapsSec += _generalBaseTime;
						break;
			case -1	:	_apert = _generalSecondApert;
						_blockNumber = _generalSecondBlockNumber;
						_elapsSec += _generalSecondBaseTime;
						break;
		}
		if( setAperture( _apert,0 ) != 1 ) {
			sendToScreen( errAperture );
			_abort = 1;
			goto ABORT_GEN;
		}
		if( setBlockNumber( _blockNumber,expositionNumber,0 ) != 1 ) {
			sendToScreen( errCounter );
			_abort = 1;
			goto ABORT_GEN;
		}
		timeoutInterval = calcTimeoutInterval( _generalExposition, expositionNumber );
		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendToScreen( errCounter );
			_abort = 1;
			goto ABORT_GEN;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();

		long addr = -1;
		cleanData( expositionNumber, &addr );

		bool isgen = (_regime == -1) ;
		
		sc::compute( expositionNumber, _nCorrected,
			_normalExposition, isgen );
		const char * prefix = isgen ? "j " : "i " ;
		sc::writeidx( out_file,sc::DSI,
			getStamp( prefix, _startBaseTime ), getAddrStr(addr) );

		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			fluxToTable( 1,
				iCounter,
				sc::getmean( true,iCounter )
				);
		}
		sendTable( 1,1,1,4 );
	/* Compute Cn2-integrals: */
		if (isgen) {
			int measnorm = sc::getcurmeas()-2 ;// current SI-measurement num.
			int measgen = sc::getcurmeas()-1 ;// current SI-measurement num.
			atm::calcint(
				sc::getidxptr(measnorm,sc::DSI),	// non-shifted (D)SI
				sc::getidxptr(measgen,sc::DSI) );	// shifted (D)SI
		}
		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			nr::erreset();
			_abort = 1;
			goto ABORT_GEN;
		}

		setRemainTimeMode( _generalAccumTime - _elapsSec );
//		if( *p_n_run == 0 ) {
		if( testUserStop() ) {
//			sendToScreen( userStop );
			_abort=1;
			goto ABORT_GEN;
		}
		_regime *= (-1);
	}	// jj-cycle
	
	}	// end of block to isolate goto-s
ABORT_GEN:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {

		debug( "Generalized: write idx" );

		QTime midAccumTime = setMidTime( startAccumTime );
		if( accumResume( midAccumTime, true ) != 1 ) {
			_abort = 1;
		} else {
//			sendToScreen( successfully );
		}

		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "!!!%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
		}
	}
	closeMode();
	enablePooling();
//	QApplication::beep();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::accumResume( QTime midAccumTime, bool isgen ) {

	int tableRow = 0;
	int success = makeAccumResultScind( midAccumTime, isgen );

	infoExportMutex.lock();
	for( int i=0; i<MAX_ROW; i++ ) {
		table.clearRow( i );
	}

	tableRow = showAccumResultScind( isgen );

	if( !success ) {
		table.isChanged = true;
		table.startRow = 0;
		table.nRows = MAX_ROW;
		table.startCol = 0;
		table.nCols = MAX_COL;
	}

	infoExportMutex.unlock();

	if( !success ) return(0);

	if( makeAccumResultAtmos( midAccumTime,
				 isgen ) == 1 ) {

		infoExportMutex.lock();
		showAccumResultAtmos( tableRow,isgen );
		infoExportMutex.unlock();

	} else success=0;

	infoExportMutex.lock();
	table.isChanged = true;
	table.startRow = 0;
	table.nRows = MAX_ROW;
	table.startCol = 0;
	table.nCols = MAX_COL;
	infoExportMutex.unlock();

	return(success);
}

int DevManager::makeAccumResultScind( QTime midAccumTime, bool isgen ) {
	sc::average( isgen, sc::MAXLAG );
	double *scind = sc::getavgidxptr( sc::DSI,sc::MODENORM );

	sc::writeavgidx( out_file,sc::HEAD,sc::MODENORM,
		"#I   UT   " );
	sc::writeavgidx( out_file,sc::DSI,sc::MODENORM,
		getStamp("I ",midAccumTime) );
	if( isgen ) {
		sc::writeavgidx( out_file,sc::HEAD,sc::MODEGEN,
			"#J   UT   " );
		sc::writeavgidx( out_file,sc::DSI,sc::MODEGEN,
			getStamp("J ",midAccumTime) );
	}
	sc::writeavgflux( out_file,sc::HEAD,sc::MODENORM,
			"#F   UT   " );
	sc::writeavgflux( out_file,sc::AFLUX,sc::MODENORM,
			getStamp("F ",midAccumTime) );
	if (isgen) {
		sc::writeavgflux( out_file,sc::HEAD,sc::MODEGEN,
			"#G   UT   " );
		sc::writeavgflux( out_file,sc::AFLUX,sc::MODEGEN,
			getStamp("G ",midAccumTime) );
	}
	if( (scind[0] <= 0) || (scind[1] <= 0)
		|| (scind[2] <= 0) || (scind[3] <= 0) ) {
		debug( "%s",badIndex.data() );
		sendToScreen( badIndex );
		return(0);
	}
	return(1);
}

int DevManager::makeAccumResultAtmos( QTime midAccumTime, bool isgen ) {
	double *scind = sc::getavgidxptr( sc::DSI,sc::MODENORM );
	double *scindShft = isgen ? sc::getavgidxptr( sc::DSI,sc::MODEGEN ) : 0 ;
/* Average Cn2-integrals and compute atm. parameters */
	const int desichan = 0 ; 	/* DESI - in A-channel */
	double desi = sc::getavgidx(sc::DESI,sc::MODENORM, desichan) ;
	double e2desi = sc::geter2idx(sc::DESI,sc::MODENORM, desichan) ;
	atm::avgint(desi, e2desi, star.getZenitDist()) ;

/* write header and data for integrals: */
	atm::write( out_file, "#A   UT   ",
		atm::ATMPAR, atm::HEAD );
	atm::write( out_file, getStamp( "A ",midAccumTime ),
		atm::ATMPAR );

/* Restore the Cn2-profile */
	double * e2scind = sc::geter2idxptr( sc::DSI,sc::MODENORM ) ;
	double * e2scindShft = isgen ? 
		sc::geter2idxptr( sc::DSI,sc::MODEGEN ) : 0 ;
	const bool IsFixedLayersMethod = true ;
/* we don't need returned Chi-square so - no assignement */
	atm::calcn2( scind, scindShft, e2scind, e2scindShft, 
		IsFixedLayersMethod) ;

/* write the header and data for Cn2 profile: */
	atm::write( out_file, "#T   UT   ",
		atm::CN2PROF, atm::HEAD );
	atm::write( out_file, getStamp( "T ",midAccumTime ),
		atm::CN2PROF );

	if( nr::erget() ) {
		str_edit = nr::ermessage().c_str();
		nr::erreset();
		debug( "%s",str_edit.data() );
		sendToScreen( str_edit );
		if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
		return(0);
	}

	return(1);
}

int DevManager::showAccumResultScind( bool isgen ) {
	debug( "Show Accum Res Scind!!!" );
	int row=0;
	table.setHHead( row,"", nameA, nameB, nameC, nameD );
	row++;
	table.setText( row,0,"CFLUX" ); row++;
	table.setText( row,0,"AFLUX" ); int rowFl = row; row++;
	table.setText( row,0,"err" ); int rowFlErr = row; row++;
	if (isgen) {
		table.setText( row,0,"AFLUXg" ); row++;
		table.setText( row,0,"err" ); row++;
	}
	table.setText( row,0,"SI" ); int rowSI = row; row++;
	table.setText( row,0,"err" ); int rowSIerr = row; row++;
	if (isgen) {
		table.setText( row,0,"SIgen" ); row++;
		table.setText( row,0,"err" ); row++;
	}

	/* we show only one DESI - shifted in generalized and non-shifted
		otherwise: */

	if (isgen) {
		table.setText( row,0,"DESIg" );
	} else {
		table.setText( row,0,"DESI" );
	}
	int rowDESI = row; row++;
	
	table.setText( row,0,"err" ); int rowDESIerr = row; row++;

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		/* Both in Normal and Generalized mode we show "non-shifted"
			fluxes: */
		table.setNum( rowFl,iCounter+1,
			sc::getavgflux( sc::MODENORM,iCounter ) );
		table.setNum( rowFlErr,iCounter+1,
			sqrt( sc::geter2flux( sc::MODENORM,iCounter ) ) );
		table.setNum( rowSI,iCounter+1,
			sc::getavgidx( sc::DSI,sc::MODENORM,iCounter ) );
		table.setNum( rowSIerr,iCounter+1,
			sqrt( sc::geter2idx( sc::DSI,sc::MODENORM,iCounter ) ) );
		if (isgen) {
			table.setNum( rowFl+2,iCounter+1,
				sc::getavgflux( sc::MODEGEN,iCounter ) );
			table.setNum( rowFlErr+2,iCounter+1,
				sqrt( sc::geter2flux( sc::MODEGEN,iCounter ) ) );
			table.setNum( rowSI+2,iCounter+1,
				sc::getavgidx( sc::DSI,sc::MODEGEN,iCounter ) );
			table.setNum( rowSIerr+2,iCounter+1,
				sqrt( sc::geter2idx( sc::DSI,sc::MODEGEN,iCounter ) ) );
			table.setNum( rowDESI,iCounter+1,
				sc::getavgidx( sc::DESI,sc::MODEGEN,iCounter ) );
			table.setNum( rowDESIerr,iCounter+1,
				sqrt( sc::geter2idx( sc::DESI,sc::MODEGEN,iCounter ) ) );
		} else {
			table.setNum( rowDESI,iCounter+1,
				sc::getavgidx( sc::DESI,sc::MODENORM,iCounter ) );
			table.setNum( rowDESIerr,iCounter+1,
				sqrt( sc::geter2idx( sc::DESI,sc::MODENORM,iCounter ) ) );
		}
	}
/* Number of displayed DSI is equal to num.of channels */
	const char* showDSI[MAX_N_COUNTERS] = {"AB","AD","BC","CD"};

	table.setHHead( row,"",showDSI[0],showDSI[1],
			showDSI[2],showDSI[3] ); row++;
	table.setText( row,0,"DSI" ); int rowDSI = row; row++;
	table.setText( row,0,"err" ); int rowDSIerr = row; row++;

	for( int iDSI=0; iDSI<MAX_N_COUNTERS; iDSI++ ) {
		int idx = sc::ind_seqnum(MAX_N_COUNTERS, 
			showDSI[iDSI]);
		table.setNum( rowDSI,iDSI+1,
			sc::getavgidx( sc::DSI,sc::MODENORM,idx  ) );
		table.setNum( rowDSIerr,iDSI+1,
			sqrt( sc::geter2idx( sc::DSI,sc::MODENORM,idx ) ) );
	}
	if (isgen) {
		table.setText( row,0,"DSIg" ); int rowDSIg = row; row++;
		table.setText( row,0,"err" ); int rowDSIgerr = row; row++;

		for( int iDSI=0; iDSI<MAX_N_COUNTERS; iDSI++ ) {
			int idx = sc::ind_seqnum(MAX_N_COUNTERS, 
				showDSI[iDSI]);
			table.setNum( rowDSIg,iDSI+1,
				sc::getavgidx( sc::DSI,sc::MODEGEN,idx  ) );
			table.setNum( rowDSIgerr,iDSI+1,
				sqrt( sc::geter2idx( sc::DSI,sc::MODEGEN,idx ) ) );
		}
	}
	debug( "Show AccumResScind return row=%d",row );
	return( row );
}

void DevManager::showAccumResultAtmos( int row, bool isgen ) {
		
	debug( "Show Accum Res Atmos!!! row=%d", row );

	table.clearRow( row ); row++;
	table.setHHead( row,"","FREE","err",
			"WHOLE","err" ); row++;

	table.clearRow( row );
	table.setText( row,0,"BETA" ); // seeng
	table.setNum( row, 1,
		atm::getval(atm::FSEE) ); // mean free seeing
	table.setNum( row, 2,
		atm::geterr(atm::FSEE) ); // error 
	if (isgen) {
		table.setNum( row, 3,
			atm::getval(atm::WSEE) ); // mean whole-atm. seeing
		table.setNum( row, 4,
			atm::geterr(atm::WSEE) ); // error 
	}
	row++;

	table.setText( row,0,"HEFF" );
	table.setNum( row, 1,
		atm::getval(atm::FHEFF) );// mean free Heff
	table.setNum( row, 2,
		atm::geterr(atm::FHEFF) );// error 
	if (isgen) {
		table.setNum( row, 3,
			atm::getval(atm::HEFF) );// mean whole-atm Heff
		table.setNum( row, 4,
			atm::geterr(atm::HEFF) );// error 
	}
	row++;

	table.clearRow( row ); row++;
	table.setText( row,0,"ISOPL:" ); // isoplan

	table.setNum( row, 1,
		atm::getval(atm::ISP) );// mean isoplan
	table.setNum( row, 2,
		atm::geterr(atm::ISP) );// error 
	row++;

	const double m2Scale = 1e5;
	table.setText( row,0,"M2*1e5:" );
/* Attention! Order of magnitude of M2 is 1e-5! */
	table.setNum( row, 1,
		atm::getval(atm::M2)*m2Scale );// mean second moment of Cn2
	table.setNum( row, 2,
		atm::geterr(atm::M2)*m2Scale );// error 
	row++;

	table.setText( row,0,"TAU:" );
	table.setNum( row, 1,
		atm::getval(atm::TC) );// mean atm. time constant
	table.setNum( row, 2,
		atm::geterr(atm::TC) );// error 
	row++;

	table.clearRow( row ); row++;

	table.setText( row,0,"Z:" );
	table.setText( row+1,0,"Cn2:" );
	double scaleCn2 = 1e-14;
	for (int zidx=0; zidx<atm::getncn2(); zidx++) {
		table.setNum( row, zidx+1,
			atm::getzcn2(zidx) );// layer altitude
		table.setNum( row+1, zidx+1,
			atm::getcn2(zidx)/scaleCn2 );// its Cn2
	}
	row += 2;

	table.setText( row, 0, "Chi2:");
	table.setText( row+1, 0, "Order:");

	table.setNum( row, 1, 
		atm::getcn2(atm::getncn2()) );// Chi2 in last Cn2[]
	table.setNum( row+1, 1, 
		log10(scaleCn2) );// order of Cn2 scale

}

void DevManager::sendFluxHead( bool clear,
			int nHeadRow,
			const char* headRow1,
			const char* headRow2="",
			const char* headRow3="" ) {
	if( clear ) debug( "clear!" );
	else debug( "Not clear!" );

	infoExportMutex.lock();
	table.setHHead( 0,"", nameA, nameB, nameC, nameD );

	table.clearRow( 1 ); table.setText( 1,0,headRow1 );
	if( nHeadRow > 1 ) {
		table.clearRow( 2 );
		table.setText( 2,0,headRow2 );
	}
	if( nHeadRow > 2 ) {
		table.clearRow( 3 );
		table.setText( 3,0,headRow3 );
	}
	if( clear ) table.clear = clear;
	int nCols = 5;
	int nRows = nHeadRow+1;
	if( table.isChanged ) {
// Staroe yeche ne narisovano...
		if( table.nCols < nCols) table.nCols = nCols;
		if( table.nRows < nRows) table.nRows = nRows;
	} else {
		table.nRows = nRows;
		table.nCols = nCols;
	}
	table.isChanged = true;
	table.startRow = 0;
	table.startCol = 0;
	infoExportMutex.unlock();
}

void DevManager::fluxToTable( int nRows, 
				int chan,
				double val1, 
				double val2 = 0,
				double val3 = 0 ) {
	infoExportMutex.lock();

	table.setNum( 1,chan+1, val1 );
	if( nRows > 1 ) table.setNum( 2,chan+1, val2 );
	if( nRows > 2 ) table.setNum( 3,chan+1, val3 );

	infoExportMutex.unlock();
}

void DevManager::sendTable( int startRow, int startCol,
				int nRows, int nCols,
				bool clear=false,bool adjust=false ) {
	infoExportMutex.lock();
	if( table.isChanged ) {
// Staroe yeche ne narisivano...
		if( table.startRow > startRow) table.startRow=startRow;
		if( table.startCol > startCol) table.startCol=startCol;
		if( table.nRows < nRows) table.nRows =
				nRows+abs(startRow-table.startRow);
		if( table.nCols < nCols) table.nCols =
				nCols+abs(startCol-table.startCol);
	} else {
		table.startRow=startRow;
		table.startCol=startCol;
		table.nRows = nRows;
		table.nCols = nCols;
	}
	table.isChanged = true;
	if( clear ) table.clear = true;
	if( adjust ) table.adjust = true;
	infoExportMutex.unlock();
}

void DevManager::waitData( void ) {
//	debug( "mode: dataMutex.tryLock..." );
	if( !dataMutex.locked() ) dataMutex.lock(); // if byl unlock po timeout-u
	dataMutex.lock();
//	debug( "mode: dataMutex.locked" );
}

int DevManager::catchAppendix( void ) {
	int *p_timeout_flag = &count_timeout_flag;
	int *p_n_read_data = &n_read_data;
	int ret = 0;

	if( !(*p_timeout_flag) ) {
		int block_appendix = 0;
		while( 1 ) {
			jkwait( DEVICE_WAIT_INTERVAL );
			if( *p_n_read_data == block_appendix ) break;
			if( srv_file ) fprintf( srv_file," appendix! %d != %d\n",*p_n_read_data,block_appendix );
			block_appendix = *p_n_read_data;
			ret = 1;
		}
	}
	return( ret );
}

void DevManager::outCountResult( void ) {
	int *p_timeout_flag = &count_timeout_flag;
	int *p_n_read_data = &n_read_data;
	int n_block;

	if( srv_file ) fprintf( srv_file,"EndTime: %s\n",
		getCurrentUT().toString().data() );
	if( !(*p_timeout_flag) ) n_block = n_counters*block_number;
	else n_block = *p_n_read_data;
	if( srv_file ) fprintf( srv_file,"tm_out=%d n_bl=%d\n",
			*p_timeout_flag,n_block );
//	debug( "tm_out=%d n_bl=%d",
//			*p_timeout_flag,n_block );
}

int DevManager::runFluxEstimation( void ) {
	int iCounter;
	QTime startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;
//	double sig_avg[MAX_N_COUNTERS]={0,0,0,0};	// storage for updated flux errors

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( estimName,getAccumTime( _count_mode ) );
	disablePooling();

	debug( "_________ FLUX_ESTIMATION_MODE:__________" );

	n_run = (int)ceil( _estimationAccumTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;
	int expositionNumber;
	int timeoutInterval=4000;

	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_ESTIM;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_ESTIM;
	}

	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ),
		estimName.data() );

	sc::init( expositionNumber, n_run, n_counters );
	writeavgflux( out_file,sc::HEAD, false,
			"#F   UT   " );

	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}
	
	sendFluxHead( ((_old_count_mode != _count_mode) && (!isScenario)),
			2,
			"CFLUX",
			"err" );

	for( int jj=0; jj<(*p_n_run); jj++ ) {

		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendToScreen( errCounter );
			_abort = 1;
			goto ABORT_ESTIM;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();
		cleanData( expositionNumber );

		sc::statflux( expositionNumber, _nCorrected,
			 _normalExposition, jj+1, 0 );
		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
			_abort = 1;
			goto ABORT_ESTIM;
		}
		for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			sqrt( sc::geter2flux(0,iCounter) );
			fluxToTable( 2,
				iCounter,
				sc::getavgflux(0,iCounter),
				sqrt( sc::geter2flux(0,iCounter) ) );
		}
		sendTable( 1,1,2,4);

		setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );
		if( testUserStop() ) {
			_abort=1;
			goto ABORT_ESTIM;
		}
	}	// jj-cycle
ABORT_ESTIM:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		QTime midAccumTime = setMidTime( startAccumTime );
		writeavgflux( out_file,sc::AFLUX,false,
			getStamp( "F ", midAccumTime ) );
		sendToScreen( successfully );
	}

	closeMode();
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorStatisticMeasurement( void ) {
	int iCounter,iLight;
	QTime startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;
	double nonPois[MAX_N_COUNTERS];

	debug( "_________ DETECTOR_STATISTIC_MODE:__________" );

	disablePooling();

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	double elapsTime = 0;
	openMode( detStatMeasName, modeTime );

	n_run = (int)ceil( _detectorStatisticTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;
	int _oldDoor = motor.isDoorOpen();
	int timeoutInterval=4000;

	infoExportMutex.lock();
	table.setHHead( 0,"Light","Param",
				nameA, nameB, nameC, nameD );
	infoExportMutex.unlock();

	sendTable( 0,0,1,6,true );

	fprintf( out_file, "%s %s\n",getStamp( "M ",getCurrentUT() ),
		detStatMeasName.data() );

	if( setLight( _detectorStatisticLight[0],0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( setLightOn( 0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	if( turnDoor( OFF,0,false ) != 1 ) {
		sendToScreen( notClosed );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	if( setAperture( _statisticApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort=1;
		goto ABORT_DET_STAT;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_STAT;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	sc::init( expositionNumber, n_run, n_counters );
	writeavgflux( out_file,sc::HEAD, false, "#L   UT    Light", true );

	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter], 0, -1, 
			(counters[iCounter]).getData() );
	}

	for( iLight=0; iLight<_nDetectorStatisticLight; iLight++ ) {
		for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendToScreen( errCounter );
				_abort = 1;
				goto ABORT_DET_STAT;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();
			cleanData( expositionNumber );

			sc::statflux( expositionNumber, _nCorrected,
				 _normalExposition, jj+1, true, nonPois );

			if( nr::erget() ) {
				str_edit = nr::ermessage().c_str();
				nr::erreset();
				debug( "%s",str_edit.data() );
				sendToScreen( str_edit );
				if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
				_abort = 1;
				goto ABORT_DET_STAT;
			}

			setRemainTimeMode( modeTime - elapsTime );
			if( testUserStop() ) {
				_abort=1;
				goto ABORT_DET_STAT;
			}
		} // jj-cycle
		QTime midAccumTime = setMidTime( startAccumTime );

		infoExportMutex.lock();
		table.setNum( iLight*4+1,0,
			_detectorStatisticLight[iLight] );
		table.clearRow( iLight*4+2 );
		table.clearRow( iLight*4+3 );
		table.setText( iLight*4+1,1, "FLUX" );
		table.setText( iLight*4+2,1, "err" );
		table.setText( iLight*4+3,1, "P" );
		table.clearRow( iLight*4+4 );
		infoExportMutex.unlock();

		str_add.sprintf( "%5.2f", _detectorStatisticLight[iLight] );
		writeavgflux( out_file,sc::AFLUX,false,
			getStamp( "L ", midAccumTime, str_add.data() ),
			true );

		for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {

			infoExportMutex.lock();
			table.setNum( iLight*4+1,iCounter+2,
				sc::getavgflux(0,iCounter) );
			table.setNum( iLight*4+2,iCounter+2,
				sqrt( sc::geter2flux(0,iCounter) ) );
			table.setNum( iLight*4+3,iCounter+2,
					nonPois[iCounter] );
			infoExportMutex.unlock();

		}
		sendTable( iLight*4+1,0,4,6 );
		if( (iLight+1) < _nDetectorStatisticLight ) {
			debug( "_SET LIGHT %5.2f", _detectorStatisticLight[iLight+1] );
			if( setLight( _detectorStatisticLight[iLight+1],0 ) != 1 ) {
				sendToScreen( errLight );
				_abort = 1;
				goto ABORT_DET_STAT;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	}  // iLight
ABORT_DET_STAT:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		sendToScreen( successfully );
	}
	setLightOff(0);
	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorCountingMeasurement( void ) {
	int *p_n_run = &n_run;
	QTime startAccumTime;
	QTime midAccumTime;
	int _abort = 0;
	int iLight,iVolt,iLevel,iCounter;

	double **nonPois = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	double **avg = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	double **sig = nr::dmatrix(0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );

	QArray<double> _oldLevels(MAX_N_COUNTERS);
	QArray<double> _levels(MAX_N_COUNTERS);

	if( !isScenario ) sendToScreen( emptyLine );
	disablePooling();

	debug( "_________ DETECTOR_COUNTING_MODE:__________" );

	n_run = (int)ceil( _detectorCountingTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;

	int timeoutInterval = 4000;
	int _oldDoor = motor.isDoorOpen();
	int _oldVoltage = getVoltage();
	getDiscriminations( _oldLevels );

	infoExportMutex.lock();
	table.setHHead( 0,"U","Level","Param",
			nameA, nameB, nameC, nameD );
	table.clearRow( 1 );
	infoExportMutex.unlock();

	sendTable( 0,0,1,7,true );

	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ),
		detCountMeasName.data() );

	double modeTime = getAccumTime( _count_mode );
	double elapsTime = 0;
	openMode( detCountMeasName,modeTime );
	if( setLight( _detectorCountingLight[0],0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( setLightOn( 0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	if( turnDoor( OFF,0,false ) != 1 ) {
		sendToScreen( notClosed );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	if( setAperture( _countApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort=1;
		goto ABORT_DET_COUNT;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_COUNT;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );

	sc::init( expositionNumber, n_run, n_counters );
	writeavgflux( out_file,sc::HEAD,false,"#U   UT    Light  U  Lev  ",
		true );

	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter], 0, -1, 
			(counters[iCounter]).getData() );
	}

	for( iLight=0; iLight<_nDetectorCountingLight; iLight++ ) {
		for( iVolt=0; iVolt<_nDetectorCountingVoltage; iVolt++ ) {

			debug( "____ SET VOLTAGE %d", _detectorCountingVoltage[iVolt] );
			if( setVoltage( _detectorCountingVoltage[iVolt],0 ) != 1 ) {
				sendToScreen( errVoltage );
				_abort = 1;
				goto ABORT_DET_COUNT;
			}
			elapsTime += HV_WAIT_INTERVAL/S2MS;
			for( int jj=0; jj<*p_n_run; jj++ ) {
				for( iLevel=0; iLevel<_nDetectorCountingDiscrimination; iLevel++ ) {
					for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
						_levels[iCounter] =  _detectorCountingDiscrimination[iLevel];
					} 
					if( setDiscriminations( _levels,0 ) != 1 ) {
						sendToScreen( errLevel );
						_abort = 1;
						goto ABORT_DET_COUNT;
					}

					startDevTimer( timeoutInterval );
					if( !runAllCounters() ) {
						sendToScreen( errCounter );
						_abort = 1;
						goto ABORT_DET_COUNT;
					}
					if( jj == 0 ) startAccumTime = _startBaseTime;

					waitData();

					stopDevTimer();
					elapsTime += _normalBaseTime;
					outCountResult();
					catchAppendix();
					cleanData( expositionNumber );

					sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
						jj+1, false, nonPois[iLevel], avg[iLevel], sig[iLevel] );

					if( nr::erget() ) {
						str_edit = nr::ermessage().c_str();
						nr::erreset();
						sendToScreen( str_edit );
						if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
						_abort = 1;
						goto ABORT_DET_COUNT;
					}
					if( testUserStop() ) {
						debug( "Users Stop!!!" );
						_abort=1;
						goto ABORT_DET_COUNT;
					}
	// if end of jj-cycle - write to file
					if( jj == (*p_n_run-1) ) {
						if( iLevel == 0 ) {
							midAccumTime = setMidTime( startAccumTime );
						}
						str_add.sprintf( "%5.2f%5d%5.2f",
							_detectorCountingLight[iLight],
							_detectorCountingVoltage[iVolt],
							_detectorCountingDiscrimination[iLevel] );
//						writeavgflux( out_file,sc::AFLUX,false,
//						getStamp( "U ", midAccumTime, str_add.data() ),
//								true );
					}
				} // iLevel
				setRemainTimeMode( modeTime - elapsTime );
			} // jj-cycle

			infoExportMutex.lock();
			table.setText( 1,0,"Light:" );
			table.setNum( 1,1, _detectorCountingLight[iLight] );
			infoExportMutex.unlock();
//			fprintf( out_file, "%5.2f",
//				_detectorCountingLight[iLight] );
			for( iLevel=0; iLevel<_nDetectorCountingDiscrimination; iLevel++ ) {

				infoExportMutex.lock();
				table.clearRow( 2+iLevel*3 );
				table.clearRow( 2+iLevel*3+1 );
				table.clearRow( 2+iLevel*3+2 );
				table.setNum( 2+iLevel*3,0,
					_detectorCountingVoltage[iVolt] );
				table.setNum( 2+iLevel*3,1,
				_detectorCountingDiscrimination[iLevel] );
				table.setText( 2+iLevel*3,2,"FLUX" );
				table.setText( 2+iLevel*3+1,2,"err" );
				table.setText( 2+iLevel*3+2,2,"P" );
				infoExportMutex.unlock();
				fprintf( out_file,"%s%5.2f%5d%5.2f",
					getStamp( "U ",getCurrentUT() ),
					_detectorCountingLight[iLight],
					_detectorCountingVoltage[iVolt],
					_detectorCountingDiscrimination[iLevel]
					);

				for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
					double sig_avg = sqrt( sig[iLevel][iCounter] );

					infoExportMutex.lock();
					table.setNum( 2+iLevel*3,iCounter+3,avg[iLevel][iCounter] );
					table.setNum( 2+iLevel*3+1,iCounter+3,sig_avg );
					table.setNum( 2+iLevel*3+2,iCounter+3,nonPois[iLevel][iCounter] );
					infoExportMutex.unlock();
					fprintf( out_file,"%10.3f %6.3f %6.3f",
						avg[iLevel][iCounter],
						sig_avg,
						nonPois[iLevel][iCounter]
						);

				}
				fprintf( out_file, "\n" );
			}

			sendTable( 1,0,
				1+3*_nDetectorCountingDiscrimination,7 );
		}  // iVolt
		if( (iLight+1) < _nDetectorCountingLight ) {
			debug( "_SET LIGHT %5.2f", _detectorCountingLight[iLight+1] );
			if( setLight( _detectorCountingLight[iLight+1],0 ) != 1 ) {
				sendToScreen( errLight );
				_abort = 1;
				goto ABORT_DET_COUNT;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	}  // iLight
ABORT_DET_COUNT:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	} else {
		sendToScreen( successfully );
	}
	debug( "ABORT_DET_COUNT:" );
	setVoltage( _oldVoltage,0 );
	setDiscriminations( _oldLevels,0 );
	turnDoor( _oldDoor,0,false );
	setLightOff( 0 );
	closeMode();
	enablePooling();

	nr::free_dmatrix( avg, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	nr::free_dmatrix( sig, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );
	nr::free_dmatrix( nonPois, 0, _nDetectorCountingDiscrimination-1, 
					0, MAX_N_COUNTERS-1 );

	if( _abort ) return(0);
	else return(1);
}

int DevManager::runExchangeTest( void ) {
	int *p_n_run = &n_run;
	QTime startAccumTime;
	int timeout_error;
	double percentage = 0.0;
	int _abort = 0;
	int iCounter = 0;

	disablePooling();

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( commonTestName,getAccumTime( _count_mode ) );

	debug( "_________ EXCHANGE_TEST_MODE:__________" );

	int _oldDoor = motor.isDoorOpen();
	
	n_run = (int)ceil( _exchangeTestTime / _normalBaseTime );

	int n_run0 = *p_n_run;
	int timeoutInterval = 4000;

//  --------------- Test Counters A,B,C,D -------------------

	int N_RUN_1 = 1;
	int jjReal = 0;

	if( turnDoor( OFF,0,false ) != 1 ) {
		sendToScreen( notClosed );
		_abort = 1;
		goto ABORT_TEST_ALL;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_TEST_ALL;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_TEST_ALL;
	}

	n_counters = 1;
	if( !resetTestCounter() ) {
		sendToScreen( errCounter );
		_abort=1;
		goto ABORT_IND_TEST;
	}

	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ),
		commonTestName.data() );

	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		timeout_error = 0;
		resetTestData();
		for( int jj=0; jj<N_RUN_1; jj++ ) {

			startDevTimer( timeoutInterval );
			if( !runTestOneCounter() ) {
				sendToScreen( errCounter );
				_abort=1;
				break;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();

			searchTestErrorsOneCounter();

			jjReal++;
		}
		if( srv_file ) fprintf( srv_file," n_run=%d tm_out_err=%d\n",
			jjReal,timeout_error );
ABORT_IND_TEST:
		if( jjReal < N_RUN_1 ) {
			if( testCounter ) {
				sendToScreen( abortProcess );
				if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
				_abort=1;
				fprintf( out_file,"%s %s\n",
					getStamp( "> ",getCurrentUT() ),
					abortProcess.data() );
			}
			percentage += 1.0; 
		} else {
			percentage += resumeTestOneCounter() / (double)N_RUN_1;
		}

		if( !switchTestCounter() ) {
			sendToScreen( errCounter );
			_abort=1;
			goto TEST_ALL;
		}

	}
	if( percentage == 0 ) {
		str_edit = indTestName;
		str_edit.append( successfully );
	} else {
		percentage = 100.0 * percentage / (double)MAX_N_COUNTERS;
		str_edit.sprintf( "%s Fault = %5.2f per cent",
				indTestName.data(), percentage );
	}
	sendToScreen( str_edit );
	fprintf( out_file,"%s %s\n",
		getStamp("> ",getCurrentUT() ),
		str_edit.data() );
// ---------------- Test All Counters ----------------

TEST_ALL: 
	n_counters = MAX_N_COUNTERS;
	resetTestData();
	if( !setInduceRegime(0) ) {
		sendToScreen( errCounter );
		_abort=1;
		goto ABORT_TEST_ALL;
	}
	timeout_error = 0;

	for( int jj=0; jj<*p_n_run; jj++ ) {
		startDevTimer( timeoutInterval );
		if( !runTestAllCounters() ) {
			sendToScreen( errCounter );
			_abort=1;
			break;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		waitData();

		stopDevTimer();

		outCountResult();
		catchAppendix();

		searchTestErrorsAllCounters();

		setRemainTimeMode( ((*p_n_run) - jj)/_normalBaseTime );
		if( testUserStop() ) {
			_abort=1;
			goto ABORT_TEST_ALL;
		}
	}
	if( srv_file ) fprintf( srv_file,"tm_out_err=%d\n",
		timeout_error );
ABORT_TEST_ALL:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
		fprintf( out_file,"%s %s\n",
			getStamp("> ",getCurrentUT() ),
			abortProcess.data() );
		_abort = 1;
	} else {
		percentage = resumeTestAllCounters() / (double)n_run0;
		if( percentage == 0 ) {
			str_edit = commonTestName;
			str_edit.append( successfully );
		} else {
			percentage = 100.0 * percentage;
			str_edit.sprintf( "%s Fault = %5.2f per cent",
				commonTestName.data(), percentage );
		}
		fprintf( out_file,"%s %s\n",
			getStamp("> ",getCurrentUT() ),
			str_edit.data() );
		sendToScreen( str_edit );
	}

	closeMode();
	turnDoor( _oldDoor,0,false );
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runDetectorTest( void ) {
	int *p_n_run = &n_run;
	QTime startAccumTime;
	int _abort = 0;
	int success = 1;
	int timeoutInterval = 4000;
	double nonPois[MAX_N_COUNTERS];

	if( !isScenario ) sendToScreen( emptyLine );
	openMode( detTestName, getAccumTime( _count_mode ) );

	disablePooling();

	debug( "_________ DETECTOR_TEST_MODE:__________" );

	n_run = (int)ceil( _detectorTestTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;

	int _oldDoor = motor.isDoorOpen();

	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ),
		detTestName.data() );
	if( setLight( _detectorTestLight,0,false ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	if( setLightOn( 0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	if( turnDoor( OFF,0,false ) != 1 ) {
		sendToScreen( notClosed );
		_abort = 1;
		goto ABORT_DET_TEST;
	}
	if( setAperture( _testApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort=1;
		goto ABORT_DET_TEST;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_TEST;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_DET_TEST;
	}

	sc::init( expositionNumber, n_run, n_counters );
	writeavgflux( out_file,sc::HEAD, false,"#L   UT    Light", true );
	
	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	sendFluxHead( _old_count_mode != _count_mode,
			3,
			"CFLUX",
			"err",
			"P" );

	for( int jj=0; jj<*p_n_run; jj++ ) {
		startDevTimer( timeoutInterval );
		if( !runAllCounters() ) {
			sendToScreen( errCounter );
			_abort = 1;
			goto ABORT_DET_TEST;
		}
		if( jj == 0 ) startAccumTime = _startBaseTime;

		waitData();

		stopDevTimer();
		outCountResult();
		catchAppendix();
		cleanData( expositionNumber );

		sc::statflux( expositionNumber, _nCorrected,
				 _normalExposition, jj+1, false, nonPois );
		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
			_abort = 1;
			goto ABORT_DET_TEST;
		}
		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			fluxToTable( 3,
				iCounter,
				sc::getavgflux(0,iCounter),
				sqrt( sc::geter2flux(0,iCounter) ),
				nonPois[iCounter]
				);
		}
		sendTable( 1,1,3,4 );

		setRemainTimeMode( ((*p_n_run) - jj) /_normalBaseTime );
		if( testUserStop() ) {
			_abort=1;
			goto ABORT_DET_TEST;
		}
	} // jj-cycle
ABORT_DET_TEST:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s %s\n",detTestName.data(),abortProcess.data() );
		fprintf( out_file,"%s %s\n",
			getStamp( "> ",getCurrentUT() ),
			abortProcess.data() );
		success = 0;
	}
	else {
		QTime midAccumTime = setMidTime( startAccumTime );
		str_add.sprintf( "%5.2f",_detectorTestLight );
		writeavgflux( out_file,sc::AFLUX,false,
			getStamp( "L ", midAccumTime,str_add.data() ),
			true );

		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			if( fabs(sc::getavgflux(0,iCounter)-_testCounts[iCounter])
					/_testCounts[iCounter] > _fluxTolerance ) {
				str_edit = fluxError;
				success = 0;
			} else if( fabs(nonPois[iCounter]-_nonPoisson[iCounter])
					> _nonPoisTolerance ) {
				str_edit = nonPoisError;
				success = 0;
			}
		}
		if( success == 0 ) {
			fprintf( out_file, "%s %s\n",
				getStamp( "> ",getCurrentUT() ),
				str_edit.data() );
			sendToScreen( str_edit );
		} else {
			fprintf( out_file, "%s %s\n",
				getStamp( "> ",getCurrentUT() ),
				successfully.data() );
			sendToScreen( successfully );
		}
	}
	setLightOff(0);
	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runStatisticTest( void ) {
	int iLight=0;
	QTime startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;

	if( !isScenario ) sendToScreen( emptyLine );
	disablePooling();

	debug( "_________ STATISTIC_TEST_MODE:__________" );

	n_run = (int)ceil( _normalAccumTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;

	int timeoutInterval = 4000;

	double elapsTime = 0;
	double modeTime = getAccumTime( _count_mode );
	openMode( statTestName, modeTime );

	int _oldDoor = motor.isDoorOpen();

	debug( "old_mode = %d _count_mode = %d",_old_count_mode,_count_mode );

	sendFluxHead( _old_count_mode != _count_mode,
			3,
			"CFLUX",
			"SI",
			"ESI" );

	fprintf( out_file,"%s %s\n",getStamp( "M ",getCurrentUT() ),
		statTestName.data() );

	{ // "try"
	if( setLight( _statisticTestLight[0],0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
	if( setLightOn( 0 ) != 1 ) {
		sendToScreen( errLight );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	if( turnDoor( OFF,0, false ) != 1 ) {
		sendToScreen( notClosed );
		_abort = 1;
		goto ABORT_ST_TEST;
	}
	if( setAperture( _testApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort=1;
		goto ABORT_ST_TEST;
	}
	int expositionNumber;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_ST_TEST;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_ST_TEST;
	}

	sc::init( expositionNumber, n_run * 2, n_counters );
	sc::writeidx( out_file,sc::HEAD,
			"#i   UT   " );

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, 0,
		_nonLinearity[iCounter],
		_nonPoisson[iCounter],
		(counters[iCounter]).getData() );
	}

	for( iLight=0; iLight<_nStatisticTestLight; iLight++ ) {
		double ampl = _testVAmpl * 
				_statisticTestLight[iLight];
		debug( "______AMPL = %.2f",ampl );
		if( (_statisticTestLight[iLight] + ampl) > MAX_LIGHT_VALUE )
			ampl = MAX_LIGHT_VALUE - _statisticTestLight[iLight];
		if( (_statisticTestLight[iLight] - ampl) < MIN_LIGHT_VALUE )
			ampl = _statisticTestLight[iLight] - MIN_LIGHT_VALUE;
		debug( "___AMPL=%.2f",ampl );
		fprintf( out_file,"%s VAmpl=%.2f\n",
			getStamp( "> ",getCurrentUT() ),
				ampl );

		if( light.setVAmpl( ampl,0 ) != 1 ) {
			sendToScreen( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}
		if( light.setVaryOn( 0 ) != 1 ) {
			sendToScreen( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}

		int nSkipped = 0 ;
		for( int jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendToScreen( errCounter );
				_abort = 1;
				goto ABORT_ST_TEST;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();
			bool isCorrected = cleanData( expositionNumber );
			if( isCorrected ) {
				if( srv_file ) fprintf( srv_file,"Skip baseTime \n" );
				nSkipped ++ ;
				continue;
			}

			sc::compute( expositionNumber, _nCorrected,
				_normalExposition, sc::MODENORM );
			long addr = -1;	// so far we do not write counts in stattest mode
			sc::writeidx( out_file,sc::DSI,
				getStamp("i ",_startBaseTime), getAddrStr(addr) );

			sc::stattest(expositionNumber, _normalExposition) ;
			sc::writeidx( out_file,sc::DSI,
				getStamp("e ",_startBaseTime), getAddrStr(addr) );

			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fluxToTable( 3,
					iCounter,
					sc::getmean( 0,iCounter ), //  mean count per ms CFLUX
					sc::getidx( sc::getcurmeas()-2,sc::DSI,iCounter ), // SI computed index 
					sc::getidx( sc::getcurmeas()-1,sc::DSI,iCounter ) // ESI expected index
					);
			}
			if( nr::erget() ) {
				str_edit = nr::ermessage().c_str();
				nr::erreset();
				debug( "%s",str_edit.data() );
				sendToScreen( str_edit );
				if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
				_abort = 1;
				goto ABORT_ST_TEST;
			}
			sendTable( 1,1,3,4 );

			setRemainTimeMode( modeTime - elapsTime );
			if( testUserStop() ) {
				_abort=1;
				goto ABORT_ST_TEST;
			}
		} // jj-cycle
		QTime midAccumTime = setMidTime( startAccumTime );
		sc::average( true, sc::MAXLAG );
		sc::writeavgidx( out_file,sc::HEAD,false,
			"#I   UT   " );
		sc::writeavgidx( out_file,sc::DSI,false,getStamp( "I ",midAccumTime ) );
		sc::writeavgidx( out_file,sc::DSI,true ,getStamp( "E ",midAccumTime ) );
		sc::writeavgflux( out_file,sc::HEAD,sc::MODENORM,
			"#F   UT   " );

		sc::writeavgflux( out_file,sc::AFLUX,sc::MODENORM,
			getStamp("F ",midAccumTime) );

		sc::again();

		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
			_abort = 1;
			goto ABORT_ST_TEST;
		}

		if( light.setVaryOff(0) != 1 ) {
			sendToScreen( errLight );
			_abort=1;
			goto ABORT_ST_TEST;
		}
		if( (iLight+1) < _nStatisticTestLight ) {
			if( setLight( _statisticTestLight[iLight+1],0 ) != 1 ) {
				sendToScreen( errLight );
				_abort = 1;
				break;
			}
			elapsTime += LIGHT_WAIT_INTERVAL/S2MS;
		}
	} // iLight
	} // "catch"
ABORT_ST_TEST:
	if( _abort ) {
		fprintf( out_file,"%s %s\n",
			getStamp( "> ",getCurrentUT() ),
			abortProcess.data() );
	} else {
		fprintf( out_file,"%s %s\n",
			getStamp( "> ",getCurrentUT() ),
			successfully.data() );
		sendToScreen( successfully );
	}
	light.setVaryOff(0);
	setLightOff(0);
	turnDoor( _oldDoor,0,false );
	closeMode();
	enablePooling();
	if( _abort ) return(0);
	else return(1);
}

int DevManager::runBackgroundCorrection( void ) {
	QTime startAccumTime;
	int jj;
	int k;
	int iCounter;
	int nShift = 0;
	int seqNum = 0;
	const char* shift = "";
	const int nPoint = 3 ;
	double avg[nPoint][MAX_N_COUNTERS];
	double sig[nPoint][MAX_N_COUNTERS];

	int *p_n_run = &n_run;
	int _abort = 0;
	int success;

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	openMode( backCorrName, modeTime );
	double elapsTime = 0;

	disablePooling();

	debug( "_________ BACKGROUND CORRECTION MODE:__________" );

	n_run = (int)ceil( _backgroundTime / _normalBaseTime );
	n_counters = MAX_N_COUNTERS;

	int expositionNumber;
	int timeoutInterval = 4000;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_CORR;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_CORR;
	}

	sc::init( expositionNumber, n_run, n_counters );
	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",getCurrentUT()),
		backCorrName.data() );

	sendFluxHead( _old_count_mode != _count_mode,
			2,
			"CFLUX",
			"err" );

	for( k=0; k<4; k++ ) {	// Aperture shift-cycle (center-left-right)
		switch( k ) {
			case 0: 
			case 2:
				if( setAperture( _backgroundApert,0 ) != 1 ) {
						sendToScreen( errAperture );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 0;
				shift = "Center";
				break;	// center
			case 1: if( shiftAperture( _backgroundApShift,0 ) != 1 ) {
						sendToScreen( errShiftApert );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 1;
				shift = "Left";
				break;	// left
			case 3: if( shiftAperture( -2 * _backgroundApShift,0 ) != 1 ) {
						sendToScreen( errShiftApert );
						_abort = 1;
						goto ABORT_CORR;
				}
				nShift = 2;
				shift = "Right";
				break;	// right
		}

		if( !isScenario ) {
			str_edit = shift;
			sendToScreen( str_edit );
		}

		for( jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendToScreen( errCounter );
				_abort = 1;
				goto ABORT_CORR;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;
			outCountResult();
			catchAppendix();
			cleanData( expositionNumber );

			switch( k ) {
				case 0: seqNum = jj+1;
					break;		// center
				case 1: seqNum = jj+1;
					break;		// left
				case 2: seqNum = (*p_n_run) + jj;
					break;		// center
				case 3: seqNum = jj+1;
					break;		// right
			}

			sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
				seqNum, false, 0, avg[nShift], sig[nShift] );

			if( nr::erget() ) {
				str_edit = nr::ermessage().c_str();
				nr::erreset();
				debug( "%s",str_edit.data() );
				sendToScreen( str_edit );
				if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
				_abort = 1;
				goto ABORT_CORR;
			}
			for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fluxToTable( 2,
					iCounter,
					avg[nShift][iCounter],
					sqrt( sig[nShift][iCounter] )
					);
			}
			sendTable( 1,1,2,4 );

			setRemainTimeMode( modeTime - elapsTime );
			if( testUserStop() ) {
				_abort=1;
				goto ABORT_CORR;
			}
		}	// jj-cycle
	}	// k-cycle
ABORT_CORR:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	}
	else {
		sendToScreen( successfully );
		if( coef ) {
			for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fprintf( out_file,"%s Channel %d:",
					getStamp( "> ",getCurrentUT() ),
					iCounter );
				for( nShift=1; nShift<3; nShift++ ) {
					if( avg[nShift][iCounter] != 0 ) {
						(*coef)[iCounter][nShift-1] =
						avg[0][iCounter]/avg[nShift][iCounter];
					} else (*coef)[iCounter][nShift-1] = 0.0;
					fprintf( out_file,"%10.3f", (*coef)[iCounter][nShift-1] );
				}
				(*coef)[iCounter][2] = ( (*coef)[iCounter][0] +
					(*coef)[iCounter][1] ) / 2.0;
				fprintf( out_file,"%10.3f\n", (*coef)[iCounter][2] );
			}
		}
		stateExportMutex.lock();
		StateExport::isCorrectionReady = true;
		stateExportMutex.unlock();

	}
	if( _abort ) success=0;
	else success=1;

	closeMode();
	enablePooling();

	coef = NULL;
	return( success );
}

int DevManager::runBackgroundMeasurement( void ) {
	int jj,iCounter;
	QTime startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;
	int success;

	const char* shift = "";
	const int nPoint = 2;
	double avg[nPoint][MAX_N_COUNTERS];
	double sig[nPoint][MAX_N_COUNTERS];

	if( !isScenario ) sendToScreen( emptyLine );
	double modeTime = getAccumTime( _count_mode );
	openMode( backMeasName, modeTime );
	double elapsTime = 0;

	disablePooling();

	debug( "_________ BACKGROUND MEASUREMENT MODE:__________" );

	n_run = (int)ceil( _backgroundTime / (2.0*_normalBaseTime) );
	n_counters = MAX_N_COUNTERS;

	int expositionNumber;
	int timeoutInterval;
	if( setBlockNumber( _normalBlockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_BACKG;
	}
	timeoutInterval = calcTimeoutInterval( _normalExposition, expositionNumber );
	if( setExpos( _normalExposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_BACKG;
	}
	if( setAperture( _backgroundApert,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort = 1;
		goto ABORT_BACKG;
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",getCurrentUT() ), backMeasName.data() );

	sc::init( expositionNumber, n_run, n_counters );
	for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		sc::chan_init( iCounter, _backgrounds[iCounter],
			_nonLinearity[iCounter],
			_nonPoisson[iCounter],
			(counters[iCounter]).getData() );
	}

	sendFluxHead( ((_old_count_mode != _count_mode) && (!isScenario)),
			2,
			"CFLUX",
			"err" );

	for( int k=0; k<nPoint; k++ ) {	// Aperture shift-cycle (left-right)
		switch( k ) {
			case 0: if( shiftAperture( _backgroundApShift,0 ) != 1 ) {
						sendToScreen( errShiftApert );
						_abort = 1;
						goto ABORT_BACKG;
				}
				shift = "Left";
				break;	// left
			case 1: if( shiftAperture( -2 * _backgroundApShift,0 ) != 1 ) {
						sendToScreen( errShiftApert );
						_abort = 1;
						goto ABORT_BACKG;
				}
				shift = "Right";
				break;	// right
		}

		if( !isScenario ) {
			str_edit = shift;
			sendToScreen( str_edit );
		}

		for( jj=0; jj<*p_n_run; jj++ ) {
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendToScreen( errCounter );
				_abort = 1;
				goto ABORT_BACKG;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			elapsTime += _normalBaseTime;

			outCountResult();
			catchAppendix();
			cleanData( expositionNumber );

			sc::statflux( expositionNumber, _nCorrected, _normalExposition, 
				jj+1, false, 0, avg[k], sig[k] );

			if( nr::erget() ) {
				str_edit = nr::ermessage().c_str();
				nr::erreset();
				debug( "%s",str_edit.data() );
				sendToScreen( str_edit );
				if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
				_abort = 1;
				goto ABORT_BACKG;
			}

			for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fluxToTable( 2,
					iCounter,
					avg[k][iCounter],
					sqrt( sig[k][iCounter] )
					);
			}
			sendTable( 1,1,2,4 );
			setRemainTimeMode( modeTime - elapsTime );
			if( testUserStop() ) {
				_abort=1;
				goto ABORT_BACKG;
			}
		}	// jj-cycle
	}	// k-cycle
ABORT_BACKG:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	}
	else {
		infoExportMutex.lock();
		table.clearRow( 2 );
		table.setText( 1, 0, "AFLUX" );
		infoExportMutex.unlock();
		
		str_edit = "";
		str_edit1 = "Background: ";
		for( iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			_backgrounds[iCounter] = (avg[0][iCounter] +
			avg[1][iCounter]) / 2 *
			_backgroundCorrectionCoefficients[iCounter];
			fluxToTable( 1, iCounter,
				_backgrounds[iCounter] );
			str_edit1 += str_add
				.setNum(_backgrounds[iCounter],'f',3 );
			str_edit += str_add.setNum(_backgrounds[iCounter],'f',3 );
			if( iCounter != MAX_N_COUNTERS-1 ) {
				str_edit1 += "  ";
				str_edit += ", ";
			}
		}

		sendTable( 1,0,2,5 );
		sendToScreen( str_edit1 );
		tree->findBranch("Operations")
			->findBranch("Background measurement")
			->findLeaf("Background")
			->replaceData( str_edit );
		tree->setChanged();
	}
	if( _abort ) success=0;
	else success=1;
	closeMode();
	enablePooling();
	return( success );
}

int DevManager::scanAperture( double accumTime, double baseTime,
			int blockNumber, double exposition, int aperture,
			int scanStart, const QString& modeName,
			bool searchCorrection, bool isFocus ) {
	QTime startAccumTime;
	int *p_n_run = &n_run;
	int _abort = 0;

	double modeTime = getAccumTime( _count_mode );
	openMode( modeName, modeTime );
	double elapsTime = 0;

	disablePooling();

	debug( "_________ SCAN APERTURE:__________" );

	n_run = (int)ceil( accumTime / (2.0*baseTime) );
	n_counters = MAX_N_COUNTERS;
	
	int _nSteps = (blockNumber * _dataBlockSize / BYTE_PER_COUNT) - 1;
	
	debug( "nSteps = %d",_nSteps );

	if( setAperture( aperture,0 ) != 1 ) {
		sendToScreen( errAperture );
		_abort=1;
		goto ABORT_SCAN;
	}
	if( shiftAperture( scanStart,0 ) != 1 ) {
		sendToScreen( errShiftApert );
		_abort = 1;
		goto ABORT_SCAN;
	}
	int expositionNumber;
	int timeoutInterval;
	if( setBlockNumber( blockNumber,expositionNumber,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_SCAN;
	}
	timeoutInterval = calcTimeoutInterval( exposition, expositionNumber );
	if( setExpos( exposition,0 ) != 1 ) {
		sendToScreen( errCounter );
		_abort = 1;
		goto ABORT_SCAN;
	}

	for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
		(scan[iCounter]).set( expositionNumber,
			_nonLinearity[iCounter]/exposition );
	}

	fprintf( out_file,"%s %s\n",getStamp("M ",getCurrentUT() ), 
		modeName.data() );
	int iDirect;
	int numbError;
	for( int jj=0; jj<*p_n_run; jj++ ) {
		iDirect = 1;
		numbError = 0;
		for( int i=0; i<2; i++ ) {
			debug( "iDirect=%d",iDirect );
				int position;
				motor.getPosition( position,0 );
				if( srv_file ) fprintf( srv_file,"\njj=%d _nSteps=%d START: position=%d\n",jj,iDirect*_nSteps,position );
			if( synchAt( iDirect*_nSteps,0 ) != 1 ) {
				sendToScreen( errShiftApert );
				_abort = 1;
				goto ABORT_SCAN;
			}
			startDevTimer( timeoutInterval );
			if( !runAllCounters() ) {
				sendToScreen( errCounter );
				_abort = 1;
				goto ABORT_SCAN;
			}
			if( jj == 0 ) startAccumTime = _startBaseTime;

			waitData();

			stopDevTimer();
			outCountResult();
			catchAppendix();

			if( testUserStop() ) {
				_abort=1;
				goto ABORT_SCAN;
			}

			bool isCorrected = cleanData( expositionNumber );
			if( isCorrected ) {
				numbError++;
				debug( "Retry scan!!! numbError=%d",numbError );
				if( numbError >= N_REPEAT_COMMAND ) {
					sendToScreen( errScan );
					_abort = 1;
					goto ABORT_SCAN;
				}
// Vozvrachaemsia dlia povtora scana:
				if( shiftAperture( -1*iDirect*_nSteps*SYNCH_STEP,0 ) != 1 ) {
					sendToScreen( errShiftApert );
					_abort = 1;
					goto ABORT_SCAN;
				}
				i--;
				continue;
			}
			elapsTime += baseTime;
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				scan[iCounter].accum( (counters[iCounter]).getData(),
				scan_t::direct( iDirect ) );
			}
			if( nr::erget() ) {
				str_edit = nr::ermessage().c_str();
				nr::erreset();
				debug( "%s",str_edit.data() );
				sendToScreen( str_edit );
				if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
				_abort = 1;
				goto ABORT_SCAN;
			}

			iDirect *= -1;  
			jkwait( DEVICE_WAIT_INTERVAL );
		}
		setRemainTimeMode( modeTime - elapsTime );
	}	// jj-cycle
ABORT_SCAN:
	if( _abort ) {
		if( srv_file ) fprintf( srv_file,"%s\n",abortProcess.data() );
	}
	else {
		QTime midAccumTime = setMidTime( startAccumTime );

		if( !isScenario ) {
			infoExportMutex.lock();
			if( !isScenario ) table.clear = true;
			table.setHHead( 0,"", nameA, nameB,
					nameC, nameD );
			if( isFocus ) {
				table.setText( 1,0, "sharp" );
				table.clearRow( 2 );
			} else {
				table.setText( 1,0, "dx" );
				table.setText( 2,0, "dy" );
			}
			infoExportMutex.unlock();
		}

		const int DEFRANGE = 10;
		for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
			scan[iCounter].merge();
			double cx1,cx2;
			scan[iCounter].center( &cx1,&cx2,scan_t::MERG );
			if( isFocus ) {
				double sharp = scan[iCounter].sharpness( DEFRANGE );
				debug( "cx1 = %.2f cx2 = %.2f", cx1,cx2 );
				debug( "sharp = %f",sharp );
				if( !isScenario )
					fluxToTable( 1,iCounter, sharp );
				fprintf( out_file,"%s sharp = %.2f\n", 
					getStamp("Y ",midAccumTime), sharp );
			} else {
				double dx = scan[iCounter].xshift();
				double dy = scan[iCounter].yshift();
				if( !isScenario )
					fluxToTable( 2,iCounter,dx,dy );
				debug( "dx:%f dy:%f",dx,dy );
				fprintf( out_file,"%s dx = %f dy = %f\n", 
					getStamp("X ",midAccumTime), dx,dy );
			}
		}
		if( isFocus ) {
			str_edit.sprintf( "Chan D sharp=%.2f",
				scan[3].sharpness( DEFRANGE ) );
		} else {
			str_edit.sprintf( "Chan D dx=%.2f dy=%.2f",
				scan[3].xshift(), scan[3].yshift() );
		}
		sendToScreen( str_edit );

		if( !isScenario )
			sendTable( 0,0,3,5 );

		fprintf( out_file,"#     FORW(A)   BACK(A)   MERG(A)"
				"     FORW(B)   BACK(B)   MERG(B)"
				"     FORW(C)   BACK(C)   MERG(C)"
				"     FORW(D)   BACK(D)   MERG(D)\n" );
		for( int i=0; i<expositionNumber; i++ ) {
			fprintf( out_file,"# " );
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				fprintf( out_file,"%12.3f%10.3f%10.3f",
					scan[iCounter].get( scan_t::FORW )[i],
					scan[iCounter].get( scan_t::BACK )[i],
					scan[iCounter].get( scan_t::MERG )[i] );
			}
			fprintf( out_file,"\n" );
		}
		if( nr::erget() ) {
			str_edit = nr::ermessage().c_str();
			nr::erreset();
			debug( "%s",str_edit.data() );
			sendToScreen( str_edit );
			if( srv_file ) fprintf( srv_file,"%s\n",str_edit.data() );
			_abort = 1;
		} else 	if( searchCorrection && delta ) {
			for( int iCounter=0; iCounter<MAX_N_COUNTERS; iCounter++ ) {
				delta->setX( scan[iCounter].xshift() );
				delta->setY( scan[iCounter].yshift() );
			}
		}
	}
	closeMode();
	enablePooling();

	stateExportMutex.lock();
	StateExport::isCorrectionReady = true;
	stateExportMutex.unlock();

	if( searchCorrection ) delta = NULL;
	if( _abort ) return(0);
	else return(1);;
}

int DevManager::runCenterCorrection( void ) {
	if( !isScenario ) sendToScreen( emptyLine );
	return( scanAperture( _centerAccumTime, _centerBaseTime,
		_centerBlockNumber, _centerExposition, _centerApert,
		_centerScanStart, centerCorName, true, false ) );
}

int DevManager::runStarCentering( void ) {
	if( !isScenario ) sendToScreen( emptyLine );
	return( scanAperture( _centerAccumTime, _centerBaseTime,
		_centerBlockNumber, _centerExposition, _centerApert,
		_centerScanStart, starCenterName, false, false ) );
}

int DevManager::runFocusControl( void ) {
	if( !isScenario ) sendToScreen( emptyLine );
	return( scanAperture( _focusAccumTime, _focusBaseTime,
		_focusBlockNumber, _focusExposition, _focusApert,
		_focusScanStart, focusContrName, false, true ) );
}

int DevManager::setVoltage( int _hvLev, int _dialog, bool _wait=true ) {
	if( !isPowerExist() ) return(0);
	return( power.setVoltage( _hvLev,_dialog,_wait ) );
}

int DevManager::setLight( double _light_, int _dialog, bool wait=true ) {
	if( !isLightExist() ) return(0);
	int ret = light.setLight( _light_,_dialog );
	if( wait ) jkwait( LIGHT_WAIT_INTERVAL );
	return( ret );
}

int DevManager::setLightOn( int _dialog ) {
	if( !isLightExist() ) return(0);
	return( light.setLightOn( _dialog ) );
}

int DevManager::setLightOff( int _dialog ) {
	if( !isLightExist() ) return(0);
	return( light.setLightOff( _dialog ) );
}

int DevManager::setDiscriminations( QArray<double>& _levels,
						int _dialog ) {
	int i;
	int ret = 1;
	if( !isCounterExist() ) return(0);
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		ret = ret * (counters[i]).setLevel( _levels[i],_dialog );
		debug( "_______ SET DISCRIM %f", _levels[i] );
	}
	return( ret );
}

void DevManager::getDiscriminations( QArray<double>& _levels ) {
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		_levels[i] = (counters[i]).getLevel();
	}
}

void DevManager::sendToScreen( const QString& text ) {
	infoExportMutex.lock();
	ExportText::isChanged = true;
	ExportText::text = text;
	infoExportMutex.unlock();
}

void DevManager::openProgressor( const QString& _text ) {

	infoExportMutex.lock();
	InfoExport::progress.message = _text;
	InfoExport::progress.progress = -1;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeProgressor( void ) {

	infoExportMutex.lock();
	InfoExport::progress.progress = -2;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();
}

void DevManager::setProgress( double percentage ) {

	infoExportMutex.lock();
	InfoExport::progress.progress = percentage;
	InfoExport::isProgressChanged = true;
	infoExportMutex.unlock();

}

void DevManager::openMode( const QString& _text, double _time ) {

	debug( "Open Mode: %s",_text.data() );
	infoExportMutex.lock();
	InfoExport::mode.message = _text;
	InfoExport::mode.progress = _time;
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeMode( void ) {

	infoExportMutex.lock();
	InfoExport::mode.message = "";
	InfoExport::mode.progress = 0;
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();

}

void DevManager::setRemainTimeMode( double _time_ ) {

	infoExportMutex.lock();
	InfoExport::mode.progress = _time_;
	InfoExport::isModeChanged = true;
	infoExportMutex.unlock();
	
}

void DevManager::openScenario( const QString& _text, double _time ) {

	infoExportMutex.lock();
	InfoExport::scenario.message = _text;
	InfoExport::scenario.progress = _time;
	InfoExport::isScenario = true;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();

}

void DevManager::closeScenario( void ) {

	infoExportMutex.lock();
	InfoExport::scenario.message = "";
	InfoExport::scenario.progress = 0;
	InfoExport::isScenario = false;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();

}

void DevManager::setRemainTimeScenario( double _time_ ) {

	infoExportMutex.lock();
	InfoExport::scenario.progress = _time_;
	InfoExport::isScenarioChanged = true;
	infoExportMutex.unlock();
	
}

double DevManager::resumeTestAllCounters( void ) {
	double percentage = 0;

	if( masterCounter ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		masterCounter->getName(),_total_test_err1 );
	}
	if( slaveCounter1 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter1->getName(),_total_test_err2 );
	}
	if( slaveCounter2 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter2->getName(),_total_test_err3 );
	}
	if( slaveCounter3 ) {
		if( srv_file ) fprintf( srv_file,"%s: tst_err=%d\n",
		slaveCounter3->getName(),_total_test_err4 );
	}

	percentage = (double)(_total_test_err1 + _total_test_err2
		+ _total_test_err3 + _total_test_err4) 
		* BYTE_PER_COUNT / _dataBlockSize;

	if( n_counters*block_number != 0 )
		percentage = percentage / (double)(n_counters*block_number);
	else {
		debug( "Error! n_counters*block_number = 0!" );
		myQAppExit(1);
	}

	return( percentage );
}

double DevManager::resumeTestOneCounter( void ) {
	double percentage = 0;

	if( srv_file ) fprintf( srv_file,"tst_err=%d\n",
		_total_test_err1 );
//	if( srv_file ) fprintf( srv_file,"bl_numb_err=%d fifo_err=%d size_err=%d\n",
//		_total_number_err,_total_fifo_err,_total_size_err );

	percentage += (double)_total_test_err1 * 
		BYTE_PER_COUNT / _dataBlockSize;
//		+ (double)(_total_fifo_err + _total_size_err);

	if( n_counters*block_number != 0 )
		percentage = percentage / (double)(n_counters*block_number);
	else {
		debug( "Error! n_counters*block_number = 0!" );
		myQAppExit(1);
	}
	return( percentage );
}

void terminate_program( void ) {

	debug( "Terminate programm" );

	if( out_file ) {
		fclose( out_file );
		out_file = NULL;
	}
	if( srv_file ) {
		fclose( srv_file );
		srv_file = NULL;
	}
	if( counts_file ) {
		fclose( counts_file );
		counts_file = NULL;
	}

	close( id_out );
	close( id_data );
	close( id_stat );
	close( id_proc );
//	dataMutex.unlock();
//	countMutex.unlock();

	free( msgdata );
}

void initThreads() {

	debug( "start of initThreads:" );

	msgdata = (unsigned char *) malloc(SIZE_STREAM_FIFO*sizeof(uchar));

	id_proc = open( "/proc/moxa/mcua0/device", O_RDWR ); // nomera blockov
	id_data = open( "/dev/rtf12", O_RDONLY | O_NONBLOCK ); 	// 
	id_stat = open( "/dev/rtf10", O_RDONLY ); 	// 

	ioctl( id_data, IO_FIFO_FLUSH,0 );


	if( (id_out = open( "/dev/mcua0",O_RDWR )) < 0 ) { // 
		perror( "?Open mcua" );
		exit(3);
	}
	
	atexit( terminate_program );

}

int sendCommand( const char* command, bool save=true ) {
	char buf[30];
	int ibuf[4];
	int retval;
//	QString str;
	int i;
	int yy;

	int key = sscanf( command,"%X %X %X %X",&ibuf[0],
                                        &ibuf[1],&ibuf[2],&ibuf[3] );
	if( key > 0 ) {
		for( i=0; i<key; i++ ) buf[i] = ibuf[i]; // Yavno privodim tipy
		if( save && srv_file ) {
			fprintf( srv_file,"-> %X",(uchar)buf[0] );
			for( i=1; i<key; i++ ) fprintf( srv_file," %X",(uchar)buf[i] );
		}
		retval = write( id_out, buf, key ); // 
		if( retval < 0 ) { 
			perror("txd: ");
			switch( errno ) {
//				case -87	: Protocol...
				case ETIMEDOUT	: retval = DSC; break;
				case EBUSY	: retval = BUS; break;
				case ENOSPC	: str_message = "No space left on device mcua0"; 
					showFatalError( str_message ); break;
				case ECONNREFUSED : retval = COL; break;
				default	: debug( "!!! unknown errno=%d",errno ); 
							if( srv_file ) fprintf( srv_file, "Unknown errno=%d\n",errno );
							myQAppExit(1);
			}
		}
		if( retval > 0x80 ) {	// Signal
			respbuf[0] = (uchar)retval;
			for( i=0; i<N_DEV_SIGNAL; i++ ) {
				if( respbuf[0] == DEV_SIGNAL_CODE[i] ) {
					if( save && srv_file ) fprintf( srv_file,"   <- %s\n",DEV_SIGNAL_ABBR[i] );
					if( respbuf[0] == DSC ) return(-2);
					return(0);
				}
			}
			if( srv_file ) fprintf( srv_file,"   <- Unknown signal: %X\n",respbuf[0] );
			return(0);
		} else {
			if( retval >= MSG_SIZE ) retval = MSG_SIZE - 1;
			respbuf[0] = retval+1;
			yy = read( id_stat, respbuf+1, retval );
			if( yy <= 0 ) { 
				perror("read id_stat: ");
			}
			if( save && srv_file ) {
				fprintf( srv_file,"   <-... " );
				for( i=1; i<respbuf[0]; i++ ) {
					fprintf( srv_file,"%X ",respbuf[i] );
				}
				fprintf( srv_file,"\n" );
			}
			return(1);
		}
	}
	return(-1);
}

int sendCommand( char address, char command, unsigned char** r, bool save=true  ) {
	char	buf[30];
	int retval;
//	QString str;
	int i;
	int yy;

	*r = respbuf;

	buf[0] = address;
	buf[1] = command;

	if( save && srv_file ) fprintf( srv_file,"-> %X %X     ",(uchar)address,(uchar)command );
	retval = write( id_out, buf, 2); // 
	if( retval < 0 ) { 
		perror("txd: ");
		switch( errno ) {
			case ETIMEDOUT	: retval = DSC; break;
			case EBUSY	: retval = BUS; break;
			case ENOSPC	: str_message = "No space left on device mcua0"; 
					showFatalError( str_message ); break;
			case ECONNREFUSED : retval = COL; break;
			default	: debug( "!!! unknown errno=%d", errno );
						if( srv_file ) fprintf( srv_file, "Unknown errno=%d\n",errno );
						myQAppExit(1);
		}
	}
	if( retval > 0x80 ) {	// Signal
		respbuf[0] = (uchar)retval;
		for( i=0; i<N_DEV_SIGNAL; i++ ) {
			if( respbuf[0] == DEV_SIGNAL_CODE[i] ) {
				if( save && srv_file ) fprintf( srv_file,"   <- %s\n",DEV_SIGNAL_ABBR[i] );
				if( respbuf[0] == DSC ) return(-2);
				return(0);
			}
		}
		if( srv_file ) fprintf( srv_file,"   <- Unknown signal: %X\n",respbuf[0] );
		return(0);
	} else {
		if( retval >= MSG_SIZE ) retval = MSG_SIZE - 1;
		respbuf[0] = retval+1;
		yy = read( id_stat, respbuf+1, retval );
		if( yy <= 0 ) { 
			perror("read id_stat: ");
		}
		if( save && srv_file ) {
			fprintf( srv_file,"   <-..." );
			for( i=1; i<respbuf[0]; i++ ) {
				fprintf( srv_file,"%X ",respbuf[i] );
			}
			fprintf( srv_file,"\n" );
		}
		return(1);
	}
}

int sendCommand( char address, char command, char _data,
			unsigned char** r, bool save=true ) {
	char	buf[30];
	int retval;
//	QString str;
	int i;
	int yy;

	*r = respbuf;

	buf[0] = address;
	buf[1] = command;
	buf[2] = _data;
	buf[3] = 0;
		
	if( save && srv_file ) fprintf( srv_file,"-> %X %X %X  ",(uchar)address,(uchar)command,(uchar)_data );
	retval = write( id_out, buf, 3 ); // 
	if( retval < 0 ) { 
		perror("txd: ");
		switch( errno ) {
			case ETIMEDOUT	: retval = DSC; break;
			case EBUSY	: retval = BUS; break;
			case ENOSPC	: str_message = "No space left on device mcua0"; 
					showFatalError( str_message ); break;
			case ECONNREFUSED : retval = COL; break;
			default	: debug( "!!! unknown errno=%d", errno );
						if( srv_file ) fprintf( srv_file, "Unknown errno=%d\n",errno );
						myQAppExit(1);
		}
	} 
	if( retval > 0x80 ) {	// Signal
		respbuf[0] = (uchar)retval;
		for( i=0; i<N_DEV_SIGNAL; i++ ) {
			if( respbuf[0] == DEV_SIGNAL_CODE[i] ) {
				if( save && srv_file ) fprintf( srv_file,"   <- %s\n",DEV_SIGNAL_ABBR[i] );
				if( respbuf[0] == DSC ) return(-2);
				return(0);
			}
		}
		if( srv_file ) fprintf( srv_file,"   <- Unknown signal: %X\n",respbuf[0] );
		return(0);
	} else {
		if( retval >= MSG_SIZE ) retval = MSG_SIZE - 1;
		respbuf[0] = retval+1;
		yy = read( id_stat, respbuf+1, retval );
		if( yy <= 0 ) { 
			perror("read id_stat: ");
		}
		if( save && srv_file ) {
			fprintf( srv_file,"   <-... " );
			for( i=1; i<respbuf[0]; i++ ) {
				fprintf( srv_file,"%X ",respbuf[i] );
			}
			fprintf( srv_file,"\n" );
		}
		return(1);
	}
}

int sendCommand( char address, char command, char low, char high,
			unsigned char** r, bool save=true ) {
	char	buf[30];
	int retval;
//	QString str;
	int i;
	int yy;

	*r = respbuf;

	buf[0] = address;
	buf[1] = command;
	buf[2] = low;
	buf[3] = high;
		
	if( save && srv_file ) fprintf( srv_file,"-> %X %X %X %X",(uchar)address,(uchar)command,
				(uchar)low,(uchar)high );
	retval = write( id_out, buf, 4 ); // 
	if( retval < 0 ) { 
		switch( errno ) {
			case ETIMEDOUT	: retval = DSC; break;
			case EBUSY	: retval = BUS; break;
			case ENOSPC	: str_message = "No space left on device mcua0"; 
					showFatalError( str_message ); break;
			case ECONNREFUSED : retval = COL; break;
			default	: debug( "!!! unknown errno=%d", errno );
					if( srv_file ) fprintf( srv_file, "Unknown errno=%d\n",errno );
					myQAppExit(1);
		}
	}
	if( retval > 0x80 ) {	// Signal
		respbuf[0] = (uchar)retval;
		for( i=0; i<N_DEV_SIGNAL; i++ ) {
			if( respbuf[0] == DEV_SIGNAL_CODE[i] ) {
				if( save && srv_file ) fprintf( srv_file,"   <- %s\n",DEV_SIGNAL_ABBR[i] );
				if( respbuf[0] == DSC ) return(-2);
				return(0);
			}
		}
		if( srv_file ) fprintf( srv_file,"   <- Unknown signal: %X\n",respbuf[0] );
		return(0);
	} else {
		if( retval >= MSG_SIZE ) retval = MSG_SIZE - 1;
		respbuf[0] = retval+1;
		yy = read( id_stat, respbuf+1, retval );
		if( yy <= 0 ) { 
			perror("read id_stat: ");
		}
		if( save && srv_file ) {
			fprintf( srv_file,"   <-... " );
			for( i=1; i<respbuf[0]; i++ ) {
				fprintf( srv_file,"%X ",respbuf[i] );
			}
			fprintf( srv_file,"\n" );
		}
		return(1);
	}
}

const char* talk( const QString& text ) {
//	QString str;
	int i;
	int resp;

	response.truncate(0);
	resp = sendCommand( text.data() );
	if(  resp == 1 ) {
		for( i=1; i<respbuf[0]; i++ ) {
			str_message.sprintf( "%X ",respbuf[i] );
			response.append( str_message );
		}
		return( response.data() );
	} else if( (resp == 0) || (resp == -2) ) {
		for( i=0; i<N_DEV_SIGNAL; i++ ) {
			if( respbuf[0] == DEV_SIGNAL_CODE[i] ) {
				response.sprintf( "%s",DEV_SIGNAL_ABBR[i] );
				return( response.data() );
			}
		}
		response.sprintf( "Unknown signal: %X", respbuf[0] );
		return( response.data() );
	} else return( "Invalid command!" );
}

void DevManager::startDevTimer( int _interval ) {
	count_timeout_flag = 0;
	_baseTimeTimerPooling = 1;
	QString startTime = getCurrentUT().toString();
	if( srv_file ) fprintf( srv_file,"StartDevTimer: %s\n",
		startTime.data() );
	_baseTimeTimer = startTimer( _interval );
}

void DevManager::stopDevTimer( void ) {
	QString stopTime = getCurrentUT().toString();
	if( srv_file ) fprintf( srv_file,"StopDevTimer: %s\n",QTime::currentTime().toString().data() );
	_baseTimeTimerPooling = 0;
	killTimer( _baseTimeTimer );
	if( srv_file ) fprintf( srv_file,"baseTimer killed... timerId=%d\n",_baseTimeTimer );
}

DevManager::DevManager( void ) {
	debug( "DevManager:  Constructor!" );

	isScenario = false;
	isUserStop = false;
	_discrLevels.resize( MAX_N_COUNTERS );
	_testCounts.resize( MAX_N_COUNTERS );
	_backgroundCorrectionCoefficients.resize( MAX_N_COUNTERS );
	_backgrounds.resize( MAX_N_COUNTERS );
	_nonLinearity.resize( MAX_N_COUNTERS );
	_nonPoisson.resize( MAX_N_COUNTERS );
	_nScenario = 2;
	scenarioName.resize( _nScenario );
	scenarioScript.resize( _nScenario );

	_counterExist = 0;
	_lightExist = 0;
	_motorExist = 0;
	_powerExist = 0;

	_pooling = 1;
	_testOverLight_flag = 0;
	_processButtons_flag = 0;
	_getTemper_flag = 0;
	_temperature = -100.0;
	_temperPooling = 0;
	_overLightPooling = 0;
	_buttonsPooling = 0;
	_baseTimeTimerPooling = 0;

	coef = NULL;
	delta = NULL;

	_stopScenario = 0;
	_pauseScenario = 0;
	_currentScenarioNumber = 0;
}

DevManager::~DevManager() {
	debug( "Destructor of devmanager..." );
	for( int i=0; i<_nScenario; i++ ) {
		debug( "delete scenarioName=%s",
				scenarioName[i]->data() );
		delete scenarioName[i];
	}
	sc::done() ;
	atm::done() ;
}

void DevManager::create( void ) {
	_programStartTime.setHMS( 0,0,0 );
	masterCounter = NULL;
	slaveCounter1 = NULL;
	slaveCounter2 = NULL;
	slaveCounter3 = NULL;
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		(counters[i]).create( 1000 );
	}
}

int DevManager::runTestAllCounters( void ) {
	n_read_data = 0;

	resetDriverBlockCounter();

	if( slaveCounter3 ) {
		slaveCounter3->blockCounter.reset();
		slaveCounter3->resetData();
		slaveCounter3->resetTest();
		if( !slaveCounter3->runTest() ) {
			return(0);
		}
	}
	if( slaveCounter2 ) {
		slaveCounter2->blockCounter.reset();
		slaveCounter2->resetData();
		slaveCounter2->resetTest();
		if( !slaveCounter2->runTest() ) {
			resetCounters(0);
			return(0);
		}
	}
	if( slaveCounter1 ) {
		slaveCounter1->blockCounter.reset();
		slaveCounter1->resetData();
		slaveCounter1->resetTest();
		if( !slaveCounter1->runTest() ) {
			resetCounters(0);
			return(0);
		}
	}
	if( masterCounter ) {
		masterCounter->blockCounter.reset();
		masterCounter->resetData();
		masterCounter->resetTest();
		setStartBaseTime();
		if( !masterCounter->runTest() ) {
			resetCounters(0);
			return(0);
		}
	}
	return(1);
}

int DevManager::runAllCounters( void ) {
	n_read_data = 0;

	resetDriverBlockCounter();

	if( slaveCounter3 ) {
		slaveCounter3->blockCounter.reset();
		slaveCounter3->resetData();
		if( !slaveCounter3->runMst() ) {
			return(0);
		}
	}
	if( slaveCounter2 ) {
		slaveCounter2->blockCounter.reset();
		slaveCounter2->resetData();
		if( !slaveCounter2->runMst() ) {
//  Peresmotret' reset, m.b. svoya expose!!!
// A master->slave oni pomniat???
			resetCounters(0);
			return(0);
		}
	}
	if( slaveCounter1 ) {
		slaveCounter1->blockCounter.reset();
		slaveCounter1->resetData();
		if( !slaveCounter1->runMst() ) {
			resetCounters(0);
			return(0);
		}
	}
	if( masterCounter ) {
		masterCounter->blockCounter.reset();
		masterCounter->resetData();
		setStartBaseTime();
		if( !masterCounter->runMst() ) {
			resetCounters(0);
			return(0);
		}
	}
	return(1);
}

int DevManager::resetTestCounter( void ) {
	int i;
	testCounter = &(counters[0]);
	for( i=1; i<MAX_N_COUNTERS; i++ ) {
		if( !((counters[i]).setMasterOff(0)) ) return(0);
		if( !((counters[i]).setActiveOff(0)) ) return(0);
	}
	if( !((counters[0]).setMasterOn(0)) ) return(0);
	if( !((counters[0]).setActiveOn(0)) ) return(0);
	return(1);
}

int DevManager::switchTestCounter( void ) {

	if( testCounter == &(counters[0]) ) {
		if( !((counters[0]).setMasterOff(0)) ) return(0);
		if( !((counters[0]).setActiveOff(0)) ) return(0);
		if( !((counters[1]).setMasterOn(0)) ) return(0);
		if( !((counters[1]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[1]);
		return(1);
	}
	if( testCounter == &(counters[1]) ) {
		if( !((counters[1]).setMasterOff(0)) ) return(0);
		if( !((counters[1]).setActiveOff(0)) ) return(0);
		if( !((counters[2]).setMasterOn(0)) ) return(0);
		if( !((counters[2]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[2]);
		return(1);
	}
	if( testCounter == &(counters[2]) ) {
		if( !((counters[2]).setMasterOff(0)) ) return(0);
		if( !((counters[2]).setActiveOff(0)) ) return(0);
		if( !((counters[3]).setMasterOn(0)) ) return(0);
		if( !((counters[3]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[3]);
		return(1);
	}
	if( testCounter == &(counters[3]) ) {
		if( !((counters[3]).setMasterOff(0)) ) return(0);
		if( !((counters[3]).setActiveOff(0)) ) return(0);
		if( !((counters[0]).setMasterOn(0)) ) return(0);
		if( !((counters[0]).setActiveOn(0)) ) return(0);
		testCounter = &(counters[0]);
		return(1);
	}
	testCounter = NULL;
	return(0);
}

int DevManager::runTestOneCounter( void ) {
	n_read_data = 0;

	resetDriverBlockCounter();

	if( testCounter ) {
		if( srv_file ) fprintf( srv_file,"Test %s: \n",testCounter->getName() );
		testCounter->blockCounter.reset();
		testCounter->resetData();
		testCounter->resetTest();
		if( !testCounter->runTest() ) {
//			str_message.sprintf( "Counter: %s: Running aborted",
//				testCounter->getName() );
//			debug( "%s",str_message.data() );
			return(0);
		}
	}
	return(1);
}

void DevManager::searchTestErrorsAllCounters( void ) {
	int err;
	count_t test_number = (count_t)(block_number * _dataBlockSize
			/ BYTE_PER_COUNT );

	if( masterCounter ) {
		err = masterCounter->searchTestErrors( test_number );
		_total_test_err1 += err;
		if( srv_file ) fprintf( srv_file," master: tst_err=%d", err );
	}
	if( slaveCounter1 ) {
		err = slaveCounter1->searchTestErrors( test_number );
		_total_test_err2 += err;
		if( srv_file ) fprintf( srv_file," slave1: tst_err=%d\n",err );
	}
	if( slaveCounter2 ) {
		err = slaveCounter2->searchTestErrors( test_number );
		_total_test_err3 += err;
		if( srv_file ) fprintf( srv_file,"slave2: tst_err=%d", err );
	}
	if( slaveCounter3 ) {
		err = slaveCounter3->searchTestErrors( test_number );
		_total_test_err4 += err;
		if( srv_file ) fprintf( srv_file," slave3: tst_err=%d\n", err );
	}
	if( srv_file ) fprintf( srv_file," start: %s",
		_startBaseTime.toString().data() );
//	if( srv_file ) fprintf( srv_file," start: %s bl_numb_err=%d fifo_err=%d size_err=%d\n",
//		_startBaseTime.toString().data(),_number_errors,_fifo_errors,_size_errors );
//	_total_fifo_err += _fifo_errors;
//	_total_size_err += _size_errors;
//	_total_number_err += _number_errors;
}

void DevManager::searchTestErrorsOneCounter( void ) {
	count_t test_number = (count_t)(block_number * _dataBlockSize
			/ BYTE_PER_COUNT );
	if( testCounter ) {
		_total_test_err1 += testCounter->searchTestErrors( test_number );
		if( srv_file ) fprintf( srv_file," %s: tst_err=%d",
			testCounter->getName(), _total_test_err1 );
	}
//	_total_fifo_err += _fifo_errors;
//	_total_size_err += _size_errors;
//	_total_number_err += _number_errors;
}

void DevManager::test( void ) {
	int i;
	for( i=0; i<500; i++ ) {
		(counters[0]).setInduceOn(0);
//		usleep( 5000L );
		(counters[0]).setInduceOff(0);
//		usleep( 5000L );
	}
}

int DevManager::calcExpositionNumber( int _blockNumber ) {
	return _blockNumber * ( _dataBlockSize / BYTE_PER_COUNT );
}

int DevManager::setBlockNumber( int _blockNumber,
				int& expositionNumber, int _dialog ) {
	expositionNumber = calcExpositionNumber( _blockNumber ) ;
	for( int i=0; i<MAX_N_COUNTERS; i++ ) {
		if( (counters[i]).setNumber( expositionNumber, _dialog ) != 1 ) return( 0 );
	}
	block_number = _blockNumber;
//	debug( "block_number = %d",block_number );
	return(1);
}

int DevManager::setExpos( double _expos,int _dialog ) {
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		if( (counters[i]).setExpos( _expos, _dialog ) != 1 ) return( 0 );
	}
	return(1);
}

int DevManager::setBlSize( int _size,int _dialog ) {
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		if( (counters[i]).setBlSize( _size, _dialog ) != 1 ) return( 0 );
	}
	return(1);
}

void DevManager::resetDriverBlockCounter( void ) {
	write( id_proc,to_proc_file,11 ); // obnuleniye nomerov po addr counter-ov
	if( srv_file ) fprintf( srv_file,"-> id_proc=%d  : %s\n",id_proc,to_proc_file );
}

int DevManager::resetCounters( int dialog ) {
	debug( "____________ Reset Counters ____________" );
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		if( (counters[i]).reset( dialog ) != 1 ) return( 0 );
		(counters[i]).blockCounter.reset();
	}
	setBlSize( _dataBlockSize,dialog );
	int expositionNumber;
	setBlockNumber( _normalBlockNumber,expositionNumber,0 );
	setExpos( _normalExposition,dialog );

	
	resetDriverBlockCounter();
/*	
	close( id_out );
	ioctl( id_data, IO_FIFO_FLUSH,0 );  // sbrasyvaem draivernyi 
					// schiotchik paketov

	if( (id_out = open( "/dev/mcua0",O_RDWR )) < 0 ) { // 
		perror( "Open mcua" );
		myQAppExit(3);
	}
*/
	return(1);	
}

int DevManager::stopCounters( int _dialog ) {
	debug( "____________ Stop Counters ____________" );
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		if( (counters[i]).stop( _dialog ) != 1 ) return(0);
	}
	return(1);	
}

void DevManager::stopCounting( void ) {
	debug( "slotStopCountong!!!" );

	userStopMutex.lock();
	isUserStop = true;
	userStopMutex.unlock();

//	n_run = 0;
}

void DevManager::stopScenario( void ) {
	_stopScenario = 1;
}

void DevManager::playScenario( void ) {

	debug( "current scenarioScript: %s",scenarioScript[0]->data() );

	_old_count_mode = _count_mode;
	_count_mode = SCENARIO_MODE;

	_stopScenario = 0;

	startCounting();

	debug( "startScenario: try unlock lockcount..." );
	countMutex.unlock();
	debug( "startScenario: unlock lockcount" );
}

void DevManager::setPause( void ) {
	_pauseScenario = 1;
	debug( "&&&&&&&&&&&&&&&&&& pauseScenario = 1" );
}

Counter* DevManager::searchSlave( int& a, int& b, int& c, int& d,
			Counter* master, int _sync_stat ) {
	Counter* slave = NULL;
	int s;
//	QString str;

	if( !master ) return NULL;

	if( a && !strcasecmp( (counters[0]).getInductor(),
				master->getName()) ) {
		slave = &(counters[0]);
		a = 0; 
	}
	if( b && !strcasecmp( (counters[1]).getInductor(),
				master->getName()) ) {
		slave = &(counters[1]);
		b = 0; 
	}
	if( c && !strcasecmp( (counters[2]).getInductor(),
				master->getName()) ) {
		slave = &(counters[2]);
		c = 0; 
	}
	if( d && !strcasecmp( (counters[3]).getInductor(),
				master->getName()) ) {
		slave = &(counters[3]);
		d = 0; 
	}
	s = a + b + c + d;
	if( s > (3-_sync_stat) ) {
		str_message.sprintf( "Invalid sync. status:\nNo Slave %d counter!",
			_sync_stat );
		showFatalError( str_message );
	} else if( s < (3-_sync_stat) ) {
		str_message.sprintf( "Invalid sync. status:\nToo many Slave %d - counters!",
			_sync_stat );
		showFatalError( str_message );
	} 
	return( slave );
}

int DevManager::calcBlockNumber(double& baseTime, double exposition)
// baseTime: [sec]
// exposition: [ms]
// aim: compute number of blocks which contain all required counts and
// ask the device to produce more counts if needed: last block may thus
// contain a few "excessive" counts
{
	// Estimate fractional "exposition number" :
	double expositionNumber = S2MS * baseTime / exposition;
	// number of blocks should be integer and contain ALL counts:
	int blockNumber = (int)ceil(
			(expositionNumber / _dataBlockSize ) * BYTE_PER_COUNT);
	expositionNumber = blockNumber * 
		_dataBlockSize / BYTE_PER_COUNT;
	baseTime = expositionNumber * exposition / S2MS;
	return blockNumber;
}

void DevManager::prepareFiles( void ) {
	debug( "DevManager: prepareFiles" );

// FILES & DIR-s:

	_logDirName = *leafDataToStr( cfgFile,"General",
				"Outputs","LogDir" );
	if( _logDirName.find( "/",_logDirName.length()-1 ) == -1 ) {
		_logDirName.append( "/" );
	}
	_logDirName.prepend( dataPath );
	testDirName( _logDirName,1 );

	_dataDirName = *leafDataToStr( cfgFile,"General",
				"Outputs","DataDir" );
	if( _dataDirName.find( "/",_dataDirName.length()-1 ) == -1 ) {
		_dataDirName.append( "/" );
	}
	_dataDirName.prepend( dataPath );
	testDirName( _dataDirName,1 );

	QString _dateName;
	QDate _date = QDate::currentDate();
	if( QTime::currentTime().hour() < HOUR_DATE_CHANG ) {
		_date = _date.addDays( -1 );	// Vecher byl vchera :-)
	}
	_dateName.sprintf( "%02d%02d%02d",
			_date.year()-2000,_date.month(),_date.day() );
	QString _fileName;
	if( !out_file ) {
		_fileName = _dateName;
		_fileName.prepend( _dataDirName );
		_fileName.append( "." );
		_fileName.append( indexFileExt );
		out_file = fopen( _fileName.data(),"a");
//		chown( _fileName.data(), (uid_t) -1, (gid_t) 52 );
		chmod( _fileName.data(),S_IRUSR | S_IWUSR |
			S_IRGRP | S_IWGRP | S_IROTH );
	}
	_countsSave = leafDataOnOffToInt( cfgFile,"Operations","Common",
				"CountsSave" );
	if( _countsSave && !counts_file ) {
		_fileName = _dateName;
		_fileName.prepend( _dataDirName );
		_fileName.append( "." );
		_fileName.append( countsFileExt );
		counts_file = fopen( _fileName.data(),"a+b" );
	}
	_logSave = leafDataOnOffToInt( cfgFile,"Operations","Common",
				"LogSave" );
	if( _logSave && !srv_file ) {
		_fileName = _dateName;
		_fileName.prepend( _logDirName );
		_fileName.append( "." );
		_fileName.append( logFileExt );
		srv_file = fopen( _fileName.data(),"a" );
	}
}

void DevManager::loadConst( void ) {
	int i;
	QString *p_str;
	debug( "DevManager: LoadConst" );

// Scenario:

	scenarioScript[0] = leafDataToStr( cfgFile,"Operations",
				"Scenario","Scenario_1" );
	unroll( scenarioScript[0]->data(),scenarioDict );
	if( nr::erget() ) {
		str_message = nr::ermessage().c_str();
		nr::erreset();
		showFatalError( str_message );
	}
	scenarioName[0] = new QString( "Scenario 1" );

	scenarioScript[1] = leafDataToStr( cfgFile,"Operations",
				"Scenario","Scenario_2" );
	unroll( scenarioScript[1]->data(),scenarioDict );
	if( nr::erget() ) {
		str_message = nr::ermessage().c_str();
		nr::erreset();
		showFatalError( str_message );
	}
	scenarioName[1] = new QString( "Scenario 2" );

	_dataBlockSize = readIntConst( "Counters","Common",
			"DataBlockSize", out_file );
	if( _dataBlockSize <= 0 ) {debug("Invalid dataBlockSize <= 0!!!");myQAppExit(1);}
	_dataBlockSize0 = _dataBlockSize+HEAD_SIZE;
	max_block_size = _dataBlockSize0 + HEAD_SIZE; // dlia udlinennykh paketov
	if( max_block_size >= SIZE_STREAM_FIFO ) {
		max_block_size = SIZE_STREAM_FIFO - 1;
		_dataBlockSize0 = max_block_size - HEAD_SIZE;
		_dataBlockSize = _dataBlockSize0-HEAD_SIZE;
	}

	_normalExposition = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_generalExposition = leafDataToDouble( cfgFile,"Operations","Generalized",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_focusExposition = leafDataToDouble( cfgFile,"Operations","Focus control",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_centerExposition = leafDataToDouble( cfgFile,"Operations","Star centering",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE, out_file ); // ???
	_focusScanStart = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanStart",-64,0, out_file );
	_focusScanFinish = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanFinish",0,64, out_file );
	_centerScanStart = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanStart",-64,0, out_file );
	_centerScanFinish = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanFinish",0,64, out_file );

	_normalBaseTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"BaseTime",0.0,10.0, out_file );
	_generalBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"BaseTime",0.0,10.0, out_file );
	_generalSecondBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"SecondBaseTime",0.0,10.0, out_file );

// NORMAL:
	_normalBlockNumber = calcBlockNumber(_normalBaseTime,_normalExposition);

// GENERALIZED 1:
	_generalBlockNumber = calcBlockNumber(_generalBaseTime, _generalExposition);

// GENERALIZED 2:
	_generalSecondBlockNumber = calcBlockNumber(_generalSecondBaseTime, 
		_generalExposition);

// FOCUS:

	int expositionNumber = _focusScanFinish - _focusScanStart + 1;
    int _maxRealShift = (int)( floor(
		((double)(MAX_SYNCH_SHIFT+1)*BYTE_PER_COUNT)/_dataBlockSize )
			* _dataBlockSize/BYTE_PER_COUNT );
	_focusBlockNumber = (int)ceil( BYTE_PER_COUNT *
			(double)expositionNumber / _dataBlockSize );
	expositionNumber = _focusBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_focusBlockNumber = expositionNumber*BYTE_PER_COUNT
				 / _dataBlockSize ;
	}
	_focusBaseTime = (double)expositionNumber * 
						(double)_focusExposition / S2MS ;

// CENTERING:

	expositionNumber = _centerScanFinish - _centerScanStart + 1;
	_centerBlockNumber = (int)ceil( BYTE_PER_COUNT *
			expositionNumber / (double)_dataBlockSize );
	expositionNumber = _centerBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_centerBlockNumber = (int)( (double)(expositionNumber*BYTE_PER_COUNT)
				 / (double)_dataBlockSize );
	}
	_centerBaseTime = (double)expositionNumber * 
						(double)_centerExposition / S2MS;

	_normalAccumTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"AccumTime",0.0,10000000.0, out_file );
	_generalAccumTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"AccumTime",0.0,10000000.0, out_file );
	_estimationAccumTime = leafDataToDouble( cfgFile,"Operations","Common",
		"FluxEstimationTime",0.0,10000000.0, out_file );
	_focusAccumTime = leafDataToDouble( cfgFile,"Operations","Focus control",
		"AccumTime",0.0,10000000.0, out_file );
	_centerAccumTime = leafDataToDouble( cfgFile,"Operations","Star centering",
		"AccumTime",0.0,10000000.0, out_file );
	_exchangeTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"ExchangeTestTime",0.0,10000000.0, out_file );
	_detectorTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"DetectorTestTime",0.0,10000000.0, out_file );
	_detectorCountingTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors counting measurement", "AccumTime",
				0.0,10000000.0, out_file );
	_detectorStatisticTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors statistics measurement", "AccumTime",
				0.0,10000000.0, out_file );
	_backgroundTime = leafDataToDouble( cfgFile,"Operations",
		"Background measurement", "AccumTime",
				0.0,10000000.0, out_file );

	_centerDeltaX0 = leafDataToInt( cfgFile,"Operations",
		"Star centering","DeltaX0", out_file );
	_centerDeltaY0 = leafDataToInt( cfgFile,"Operations",
		"Star centering","DeltaY0", out_file );

	_backgroundApShift = leafDataToInt( cfgFile,"Operations",
		"Background measurement", "ApertureShift",0,127, out_file );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Correction", out_file );
	int _n_ = stringToDoubleArray( p_str,
		_backgroundCorrectionCoefficients,
		"turbina.cfg\\Operations\\Background measurement\\Correction\n",
		0.0,10.0 );
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Background measurement\\Correction:\n Numb. of coeff != MAX_N_COUNTERS!";
		showFatalError( str_message );
	}

	p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Background", out_file );
	_n_ = stringToDoubleArray( p_str, _backgrounds,
		"turbina.cfg\\Operations\\Background measurement\\Background\n",
		0.0,1000000.0 );
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Background measurement\\Background:\n Numb. of backgrouns != MAX_N_COUNTERS!";
		showFatalError( str_message );
	}

	_name_A = leafDataToStr( cfgFile,"Devices","Channel A",
		"Detector", out_file )->data();
	(counters[0]).setChannel( "A" );
	(counters[0]).setName( _name_A );

	_name_B = leafDataToStr( cfgFile,"Devices","Channel B",
		"Detector", out_file )->data();
	(counters[1]).setChannel( "B" );
	(counters[1]).setName( _name_B );

	_name_C = leafDataToStr( cfgFile,"Devices","Channel C",
		"Detector", out_file )->data();
	(counters[2]).setChannel( "C" );
	(counters[2]).setName( _name_C );

	_name_D = leafDataToStr( cfgFile,"Devices","Channel D",
		"Detector", out_file )->data();
	(counters[3]).setChannel( "D" );
	(counters[3]).setName( _name_D );

	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		(counters[i]).loadConst();
	}
	sprintf( to_proc_file,"%02X %02X %02X %02X",
		(counters[0]).getModule(),(counters[1]).getModule(),
		(counters[2]).getModule(),(counters[3]).getModule() );
	debug( "%s",to_proc_file );
	_counterExist = 1;

	_moduleA = (counters[0]).getModule();
	_moduleB = (counters[1]).getModule();
	_moduleC = (counters[2]).getModule();
	_moduleD = (counters[3]).getModule();

	int a=1,b=1,c=1,d=1;
	int s;

	if( (counters[0]).master() ) { masterCounter = &(counters[0]); a = 0; }
	if( (counters[1]).master() ) { masterCounter = &(counters[1]); b = 0; }
	if( (counters[2]).master() ) { masterCounter = &(counters[2]); c = 0; }
	if( (counters[3]).master() ) { masterCounter = &(counters[3]); d = 0; }
	s = a + b + c + d;
	if( s < 3 ) { str_message = "Invalid sync. status:\nToo many masters!"; showFatalError( str_message ); }
	if( s > 3 ) { str_message = "Invalid sync. status:\nNo Master !!!"; showFatalError( str_message ); }

	slaveCounter1 = searchSlave( a,b,c,d,masterCounter,1 );
	slaveCounter2 = searchSlave( a,b,c,d,slaveCounter1,2 );
	slaveCounter3 = searchSlave( a,b,c,d,slaveCounter2,3 );

	if( masterCounter && slaveCounter1 && slaveCounter2 && slaveCounter3 )
		if( srv_file ) fprintf( srv_file,"%s -> %s -> %s -> %s\n",masterCounter->getName(),
		slaveCounter1->getName(),slaveCounter2->getName(),slaveCounter3->getName() );

	if( _discrLevels.size() != MAX_N_COUNTERS ) _discrLevels.resize( MAX_N_COUNTERS );
	if( _nonLinearity.size() != MAX_N_COUNTERS ) _nonLinearity.resize( MAX_N_COUNTERS );
	if( _nonPoisson.size() != MAX_N_COUNTERS ) _nonPoisson.resize( MAX_N_COUNTERS );
	_discrLevels[0] = leafDataToDouble( cfgFile,"Devices",
			"Channel A","Discrimination", out_file );
	_nonLinearity[0] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel A","NonLinearity", out_file );
	_nonPoisson[0] = leafDataToDouble( cfgFile,"Devices",
			"Channel A","NonPoisson", out_file );
	_discrLevels[1] = leafDataToDouble( cfgFile,"Devices",
			"Channel B","Discrimination", out_file );
	_nonLinearity[1] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel B","NonLinearity", out_file );
	_nonPoisson[1] = leafDataToDouble( cfgFile,"Devices",
			"Channel B","NonPoisson", out_file );
	_discrLevels[2] = leafDataToDouble( cfgFile,"Devices",
			"Channel C","Discrimination", out_file );
	_nonLinearity[2] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel C","NonLinearity", out_file );
	_nonPoisson[2] = leafDataToDouble( cfgFile,"Devices",
			"Channel C","NonPoisson", out_file );
	_discrLevels[3] = leafDataToDouble( cfgFile,"Devices",
			"Channel D","Discrimination", out_file );
	_nonLinearity[3] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel D","NonLinearity", out_file );
	_nonPoisson[3] = leafDataToDouble( cfgFile,"Devices",
			"Channel D","NonPoisson", out_file );

	light.loadConst();
	_lightExist = 1;

	_testVAmpl = leafDataToDouble( cfgFile,"Operations",
		"Tests","Modulation", out_file );
	_hvLevel = leafDataToInt( cfgFile,
			"Operations","Common","HighVoltage", out_file );
	_highPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","HighPoolingTime" );
	_buttonPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","ButtonPoolingTime" );
	_temperPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","TemperPoolingTime" );

	p_str = leafDataToStr( cfgFile,"Operations","Tests",
				"StatisticLight", out_file );

	_nStatisticTestLight = stringToDoubleArray( p_str,
		_statisticTestLight,"turbina.cfg\\Operations\\Tests\\Light",
			0,1 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Light" );

	_nDetectorCountingLight = stringToDoubleArray( p_str,
		_detectorCountingLight,"turbina.cfg\\Operations\\Detectors counting measurement\\Light",
			0,1 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Voltage" );

	_nDetectorCountingVoltage = stringToIntArray( p_str,
		_detectorCountingVoltage,"turbina.cfg\\Operations\\Detectors counting measurement\\Voltage" );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Discrimination" );

	_nDetectorCountingDiscrimination = stringToDoubleArray( p_str,
		_detectorCountingDiscrimination,
		"turbina.cfg\\Operations\\Detectors counting measurement\\Discrimination\n",
		0.0,10.0 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement","Light" );

	_nDetectorStatisticLight = stringToDoubleArray( p_str,
		_detectorStatisticLight,"turbina.cfg\\Operations\\Detectors statistic measurement\\Light",
			0,1 );

	_detectorTestLight = leafDataToDouble( cfgFile,
		"Operations","Tests","DetectorLight",0.0,1.0 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Tests","TestCounts" );
	_n_ = stringToDoubleArray( p_str,
		_testCounts,
		"turbina.cfg\\Operations\\Tests\\TestCounts\n",
		0.0,1000000000.0);
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Tests\\TestCounts:\n Numb. of testCounts != MAX_N_COUNTERS!";
		showFatalError( str_message );
	}

	_fluxTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","FluxTolerance" ); 
	_nonPoisTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","NonPoisTolerance" ); 

	motor.loadConst();
	_motorExist = 1;

	_exchangeTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"ExchangeTest" );
	_statisticTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"StatisticTest" );
	_detectorTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"DetectorTest" );
	_inputDoor = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"InputDoor" );
	_inputDoorPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"InputDoor" );
	QString _apName = *leafDataToStr( cfgFile,"Operations",
			"Initialization", "InitialStop" );
	_initialStop = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Normal mode","WithAperture", out_file );
	_normalApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithAperture", out_file );
	_generalApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithSecond", out_file );
	_generalSecondApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Focus control","WithAperture", out_file );
	_focusApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Star centering","WithAperture", out_file );
	_centerApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Background measurement","WithAperture", out_file );
	_backgroundApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement",
					"WithAperture", out_file );
	_countApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement",
					"WithAperture", out_file );
	_statisticApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Tests","WithAperture", out_file );
	_testApert = motor.getApertureNumber( _apName,1 );


	_illumValue = (int)rint( 15.0 * leafDataToDouble( cfgFile,"Operations",
			"Initialization","IlluminationLevel",0.0,1.0 ) );
	_illumination = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Illumination" );
	_illumPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Illumination" );
	_light = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Light" );
	_lightPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Light" );
	_buttons = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Buttons" );
	_buttonsPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Buttons" );

	_aperturesMode = readIntConst( "Lighters & Buttons",
			"ButtonModes","Apertures" );
	_illumMode = readIntConst( "Lighters & Buttons",
			"ButtonModes","Illumination" );
	_shiftMode = readIntConst( "Lighters & Buttons",
			"ButtonModes","FineShift" );
	_doorMode = readIntConst( "Lighters & Buttons",
			"ButtonModes","InputDoor" );
	QString _bMode_str = *leafDataToStr( cfgFile,"Operations","Initialization",
				"ButtonMode" );
	_buttonMode = -1;
	if( operator==( _bMode_str.upper(),"APERTURES" ) ) _buttonMode = _aperturesMode;
	if( operator==( _bMode_str.upper(),"ILLUMINATION" ) ) _buttonMode = _illumMode;
	if( operator==( _bMode_str.upper(),"FINESHIFT" ) ) _buttonMode = _shiftMode;
	if( operator==( _bMode_str.upper(),"INPUTDOOR" ) ) _buttonMode = _doorMode;
	if( _buttonMode == -1 ) {
		str_edit = "Undefined initial Buttons Mode!";
		switch( showStrongWarning( str_edit ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
		}
	}

	_highVoltage = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"HighVoltage" );
	_hvPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"HighVoltage" );
	_safety = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Safety" );

	power.loadConst();
	_powerExist = 1;


// Optics & WF

	double _foclenFeed = readDoubleConst( "Optics","Feeding optics",
					"FocalLength", out_file );
	double _foclen = motor.getFoclen( _normalApert );
	if( _foclen != 0 ) {
		_zShiftNormal = wf::getzshift( _foclenFeed, _foclen );
	} else _zShiftNormal = 0.0;

	_foclen = motor.getFoclen( _generalApert );
	if( _foclen != 0 ) {
		_zShiftGeneral = wf::getzshift( _foclenFeed, _foclen );
	} else _zShiftGeneral = 0.0;
	_foclen = motor.getFoclen( _generalSecondApert );
	if( _foclen != 0 ) {
		_zShiftGeneralSecond = wf::getzshift( _foclenFeed, _foclen );
	} else _zShiftGeneralSecond = 0.0;

	connect( tree, SIGNAL(changed()), this, SLOT(change()) );

}

void DevManager::change( void ) {
	QString *p_str;
	debug( "DevManager: change: ReLoadConst" );

	QString* scScrp;
	scScrp = leafDataToStr( cfgFile,"Operations",
				"Scenario","Scenario_1" );
	unroll( scScrp->data(),scenarioDict );
	if( nr::erget() ) {
		str_message = nr::ermessage().c_str();
		nr::erreset();
		showStrongMessage( str_message,"Ignore" );
	} else scenarioScript[0] = scScrp;
	if( _currentScenarioNumber == 0 ) {
		setCurrentScenario( 0 );
	}
	scScrp = leafDataToStr( cfgFile,"Operations",
			"Scenario","Scenario_2" );
	unroll( scScrp->data(),scenarioDict );
	if( nr::erget() ) {
		str_message = nr::ermessage().c_str();
		nr::erreset();
		showStrongMessage( str_message,"Ignore" );
	} else scenarioScript[1] = scScrp;
	if( _currentScenarioNumber == 1 ) {
		setCurrentScenario( 1 );
	}

	_normalExposition = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE ); // ???
	_generalExposition = leafDataToDouble( cfgFile,"Operations","Generalized",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE ); // ???
	_focusExposition = leafDataToDouble( cfgFile,"Operations","Focus control",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE ); // ???
	_centerExposition = leafDataToDouble( cfgFile,"Operations","Star centering",
		"Exposition",MIN_EXPOSE,MAX_EXPOSE ); // ???
	_focusScanStart = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanStart",-64,0 );
	_focusScanFinish = leafDataToInt( cfgFile,"Operations","Focus control",
		"ScanFinish",0,64 );
	_centerScanStart = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanStart",-64,0 );
	_centerScanFinish = leafDataToInt( cfgFile,"Operations","Star centering",
		"ScanFinish",0,64 );

	_normalBaseTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"BaseTime",0.0,10.0 );
	_generalBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"BaseTime",0.0,10.0 );
	_generalSecondBaseTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"SecondBaseTime",0.0,10.0 );

// NORMAL:
	_normalBlockNumber = calcBlockNumber(_normalBaseTime, _normalExposition);

// GENERALIZED 1:
	_generalBlockNumber = calcBlockNumber(_generalBaseTime, _generalExposition);

// GENERALIZED 2:
	_generalSecondBlockNumber = calcBlockNumber(_generalSecondBaseTime, 
		_generalExposition);

// FOCUS:

	int expositionNumber = _focusScanFinish - _focusScanStart + 1;
    int _maxRealShift = (int)( floor(
		((double)(MAX_SYNCH_SHIFT+1)*BYTE_PER_COUNT)/_dataBlockSize )
			* _dataBlockSize/BYTE_PER_COUNT );
	_focusBlockNumber = (int)ceil( BYTE_PER_COUNT *
			(double)expositionNumber / _dataBlockSize );
	expositionNumber = _focusBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_focusBlockNumber = expositionNumber*BYTE_PER_COUNT
				 / _dataBlockSize ;
	}
	_focusBaseTime = (double)expositionNumber * 
						(double)_focusExposition / S2MS ;

// CENTERING:

	expositionNumber = _centerScanFinish - _centerScanStart + 1;
	_centerBlockNumber = (int)ceil( BYTE_PER_COUNT *
			expositionNumber / (double)_dataBlockSize );
	expositionNumber = _centerBlockNumber * 
				_dataBlockSize / BYTE_PER_COUNT;
	if( expositionNumber > (_maxRealShift) ) { 
		expositionNumber = _maxRealShift;
		_centerBlockNumber = (int)( (double)(expositionNumber*BYTE_PER_COUNT)
				 / (double)_dataBlockSize );
	}
	_centerBaseTime = (double)expositionNumber * 
						(double)_centerExposition / S2MS;

	_normalAccumTime = leafDataToDouble( cfgFile,"Operations","Normal mode",
		"AccumTime",0.0,10000000.0 );
	_generalAccumTime = leafDataToDouble( cfgFile,"Operations","Generalized",
		"AccumTime",0.0,10000000.0 );
	_estimationAccumTime = leafDataToDouble( cfgFile,"Operations","Common",
		"FluxEstimationTime",0.0,10000000.0 );
	_focusAccumTime = leafDataToDouble( cfgFile,"Operations","Focus control",
		"AccumTime",0.0,10000000.0 );
	_centerAccumTime = leafDataToDouble( cfgFile,"Operations","Star centering",
		"AccumTime",0.0,10000000.0 );
	_exchangeTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"ExchangeTestTime",0.0,10000000.0 );
	_detectorTestTime = leafDataToDouble( cfgFile,"Operations","Tests",
		"DetectorTestTime",0.0,10000000.0 );
	_detectorCountingTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors counting measurement", "AccumTime",0.0,10000000.0 );
	_detectorStatisticTime = leafDataToDouble( cfgFile,"Operations",
		"Detectors statistics measurement", "AccumTime",0.0,10000000.0 );
	_backgroundTime = leafDataToDouble( cfgFile,"Operations",
		"Background measurement", "AccumTime",0.0,10000000.0 );

	_centerDeltaX0 = leafDataToInt( cfgFile,"Operations",
		"Star centering","DeltaX0" );
	_centerDeltaY0 = leafDataToInt( cfgFile,"Operations",
		"Star centering","DeltaY0" );

	_backgroundApShift = leafDataToInt( cfgFile,"Operations",
		"Background measurement", "ApertureShift",0,127 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Correction" );
	int _n_ = calcNumberDoubleInString( p_str,
		"turbina.cfg\\Operations\\Background measurement\\Correction\n" );
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Background measurement\\Correction:\n Numb. of coeff != MAX_N_COUNTERS!";
		showStrongMessage( str_message,"Ignore" );
	} else {
		_n_ = stringToDoubleArray( p_str,
			_backgroundCorrectionCoefficients,
			"turbina.cfg\\Operations\\Background measurement\\Correction\n",
			0.0,10.0 );
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Operations\\Background measurement\\Correction:\n Numb. of coeff != MAX_N_COUNTERS!";
			showFatalError( str_message );
		}
	}

	p_str = leafDataToStr( cfgFile,"Operations",
			"Background measurement","Background" );
	_n_ = calcNumberDoubleInString( p_str,
		"turbina.cfg\\Operations\\Background measurement\\Background\n" );
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Background measurement\\Background:\n Numb. of backgrounds != MAX_N_COUNTERS!";
		showStrongMessage( str_message,"Ignore" );
	} else {
		_n_ = stringToDoubleArray( p_str, _backgrounds,
			"turbina.cfg\\Operations\\Background measurement\\Background\n",
			0.0,1000000.0);
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Operations\\Background measurement\\Background:\n Numb. of backgrouns != MAX_N_COUNTERS!";
			showFatalError( str_message );
		}
	}

	_discrLevels[0] = leafDataToDouble( cfgFile,"Devices",
			"Channel A","Discrimination" );
	_nonLinearity[0] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel A","NonLinearity" );
	_nonPoisson[0] = leafDataToDouble( cfgFile,"Devices",
			"Channel A","NonPoisson" );
	_discrLevels[1] = leafDataToDouble( cfgFile,"Devices",
			"Channel B","Discrimination" );
	_nonLinearity[1] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel B","NonLinearity" );
	_nonPoisson[1] = leafDataToDouble( cfgFile,"Devices",
			"Channel B","NonPoisson" );
	_discrLevels[2] = leafDataToDouble( cfgFile,"Devices",
			"Channel C","Discrimination" );
	_nonLinearity[2] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel C","NonLinearity" );
	_nonPoisson[2] = leafDataToDouble( cfgFile,"Devices",
			"Channel C","NonPoisson" );
	_discrLevels[3] = leafDataToDouble( cfgFile,"Devices",
			"Channel D","Discrimination" );
	_nonLinearity[3] = NS2MS * leafDataToDouble( cfgFile,"Devices",
			"Channel D","NonLinearity" );
	_nonPoisson[3] = leafDataToDouble( cfgFile,"Devices",
			"Channel D","NonPoisson" );

	_testVAmpl = leafDataToDouble( cfgFile,"Operations",
		"Tests","Modulation" );
	int int_val = leafDataToInt( cfgFile,
			"Operations","Common","HighVoltage" );
	if( int_val != _hvLevel ){ 
		_hvLevel = int_val;
		setVoltage( _hvLevel,1,false );
	}

	_highPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","HighPoolingTime" );
	_buttonPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","ButtonPoolingTime" );
	_temperPoolingTime = leafDataToDouble( cfgFile,
			"Operations","Common","TemperPoolingTime" );

	p_str = leafDataToStr( cfgFile,"Operations","Tests",
				"StatisticLight" );

	_nStatisticTestLight = stringToDoubleArray( p_str,
		_statisticTestLight,"turbina.cfg\\Operations\\Tests\\Light",
			0,1 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Light" );

	_nDetectorCountingLight = stringToDoubleArray( p_str,
		_detectorCountingLight,"turbina.cfg\\Operations\\Detectors counting measurement\\Light",
			0,1 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Voltage" );

	_nDetectorCountingVoltage = stringToIntArray( p_str,
		_detectorCountingVoltage,"turbina.cfg\\Operations\\Detectors counting measurement\\Voltage" );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","Discrimination" );

	_nDetectorCountingDiscrimination = stringToDoubleArray( p_str,
		_detectorCountingDiscrimination,
		"turbina.cfg\\Operations\\Detectors counting measurement\\Discrimination\n",
		0.0,10.0 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement","Light" );

	_nDetectorStatisticLight = stringToDoubleArray( p_str,
		_detectorStatisticLight,"turbina.cfg\\Operations\\Detectors statistic measurement\\Light",
			0,1 );

	_detectorTestLight = leafDataToDouble( cfgFile,
		"Operations","Tests","DetectorLight",0.0,1.0 );

	p_str = leafDataToStr( cfgFile,"Operations",
			"Tests","TestCounts" );
	_n_ = calcNumberDoubleInString( p_str,
		"turbina.cfg\\Operations\\Tests\\TestCounts\n" );
	if( _n_ != MAX_N_COUNTERS ) {
		str_message = "turbina.cfg\\Operations\\Tests\\TestCounts:\n Numb. of testCounts != MAX_N_COUNTERS!";
		showStrongMessage( str_message,"Ignore" );
	} else {
		_n_ = stringToDoubleArray( p_str,
			_testCounts,
			"turbina.cfg\\Operations\\Tests\\TestCounts\n",
			0.0,1000000000.0);
		if( _n_ != MAX_N_COUNTERS ) {
			str_message = "turbina.cfg\\Operations\\Tests\\TestCounts:\n Numb. of testCounts != MAX_N_COUNTERS!";
			showFatalError( str_message );
		}
	}
	_fluxTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","FluxTolerance" ); 
	_nonPoisTolerance = leafDataToDouble( cfgFile,"Operations",
			"Tests","NonPoisTolerance" ); 

	_exchangeTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"ExchangeTest" );
	_statisticTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"StatisticTest" );
	_detectorTest = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"DetectorTest" );
	_inputDoor = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"InputDoor" );
	_inputDoorPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"InputDoor" );
	QString _apName = *leafDataToStr( cfgFile,"Operations",
			"Initialization", "InitialStop" );
	_initialStop = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Normal mode","WithAperture" );
	_normalApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithAperture" );
	_generalApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Generalized","WithSecond" );
	_generalSecondApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Focus control","WithAperture" );
	_focusApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Star centering","WithAperture" );
	_centerApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Background measurement","WithAperture" );
	_backgroundApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors counting measurement","WithAperture" );
	_countApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Detectors statistics measurement","WithAperture" );
	_statisticApert = motor.getApertureNumber( _apName,1 );
	_apName = *leafDataToStr( cfgFile,"Operations",
			"Tests","WithAperture" );
	_testApert = motor.getApertureNumber( _apName,1 );


	_illumValue = (int)rint( 15.0 * leafDataToDouble( cfgFile,"Operations",
			"Initialization","IlluminationLevel",0.0,1.0 ) );
	_illumination = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Illumination" );
	_illumPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Illumination" );
	_light = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Light" );
	_lightPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Light" );
	_buttons = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Buttons" );
	_buttonsPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"Buttons" );

	_highVoltage = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"HighVoltage" );
	_hvPark = leafDataOnOffToInt( cfgFile,"Operations","Parking",
				"HighVoltage" );
	_safety = leafDataOnOffToInt( cfgFile,"Operations","Initialization",
				"Safety" );

}

int DevManager::testShutter( Counter* counter ) {
//	QString str;
	uchar status;

	if( !counter ) return(0);
	while( 1 ) {
		counter->getStatus( status );
		if( !(status & SHUTTER) ) {
			str_message.sprintf( "Channel %s: Shutter closed!",counter->getChannel() );
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: return(0);
				case  QMessageBox::Retry : break;
			}
		} else return(1);
	}

}

void DevManager::init( void ) {
	int i;
	int ret0;
	int ret1;
	int ret5,ret6;

	debug( "++++++ DevManager.init:" );

	light.reset();
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		(counters[i]).reset(1);
	}
//	motor.reset();

	int cycle = 1;
	while( cycle ) {
		ret0 = light.getIdent();
		ret1 = 1;
		for( i=0; i<MAX_N_COUNTERS; i++ ) {
			if( (counters[i]).getIdent() == -1 ) ret1 = -1;
		}
		ret5 = motor.getIdent();
		ret6 = power.getIdent();
		if( (ret0 == -1) && (ret1 == -1)
			&& (ret5 == -1) && (ret6 == -1) ) {
			str_message = "Device disconnected. Connect it!";
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: cycle=0; break;
				case  QMessageBox::Retry : break;
			}
		} else cycle=0;
	}

	cycle = 1;
	while( cycle ) {
		ret0 = light.getConst();
		ret1 = 1;
		for( i=0; i<MAX_N_COUNTERS; i++ ) {
			if( (counters[i]).getConst() == -1 ) ret1 = -1;
		}
		ret5 = motor.getConst();
		ret6 = power.getConst();
		if( (ret0 == -1) && (ret1 == -1)
			&& (ret5 == -1) && (ret6 == -1) ) {
			str_message = "Device disconnected. Connect it!";
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: cycle=0; break;
				case  QMessageBox::Retry : break;
			}
		} else cycle=0;
	}

	cycle = 1;
	while( cycle ) {
		ret0 = light.testCrc();
		ret1 = 1;
		for( i=0; i<MAX_N_COUNTERS; i++ ) {
			if( (counters[i]).testCrc() == -1 ) ret1 = -1;
		}
		ret5 = motor.testCrc();
		ret6 = power.testCrc();
		if( (ret0 == -1) && (ret1 == -1)
			&& (ret5 == -1) && (ret6 == -1) ) {
			str_message = "Device disconnected. Connect it!";
			switch ( showWarning( str_message ) ) {
				case  QMessageBox::Abort : myQAppExit(1); 
				case  QMessageBox::Ignore: cycle=0; break;
				case  QMessageBox::Retry : break;
			}
		} else cycle=0;
	}
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		testShutter( &(counters[i]) );
	}
	setDiscriminations( _discrLevels,1 );

	startTemperPooling();
	setVoltage( _hvLevel,1,false );
//	testPowerStatus(1);
	turnSafety( OFF,1 );
	turnSafety( _safety,1 );
	turnHV( _highVoltage,1 );
//	QTime t;
//	t.start();

	testMotorStatus(1);
	testButtons();
	testIllum();

	motor.loadStops(1);
	motor.setStep( SYNCH_STEP,1 );

	setBlSize( _dataBlockSize,1 );

//	turnDoor( _inputDoor,1 );	// posle init testov
//	setAperture( _initialStop,1 );
	setBrightness( _illumValue,1 );
	turnIllum( _illumination,1 );
	light.turnLight( _light,1 );
	turnButtons( _buttons,1 );
	light.setBMode( _buttonMode,1 );
	setCurrentScenario( _currentScenarioNumber );

	setInduceRegime(1);

	initTests();
}

void DevManager::startTemperPooling( void ) {
	getTemper(0);
	_getTemper_flag = 0;
	if( !_temperPooling ) {
		_temperTimer = startTimer( (int)(_temperPoolingTime*1000.0) );
		_temperPooling = 1;
	}
}

void DevManager::stopTemperPooling( void ) {
	if( _temperPooling ) {
		killTimer( _temperTimer );
		_temperPooling = 0;
	}
}

void DevManager::startOverLightPooling( void ) {
	_testOverLight_flag = 0;
	if( !_overLightPooling ) {
		_hvTimer = startTimer( (int)(_highPoolingTime*1000.0) ); 
		_overLightPooling = 1;
	}
}

void DevManager::stopOverLightPooling( void ) {
	if( _overLightPooling ) {
		killTimer( _hvTimer );
		_overLightPooling = 0;
	}
}

void DevManager::startButtonsPooling( void ) {
	_processButtons_flag = 0;
	if( !_buttonsPooling ) {
		_buttonsTimer = startTimer( (int)(_buttonPoolingTime*1000.0) );
		_buttonsPooling = 1;
	}
}

void DevManager::stopButtonsPooling( void ) {
	if( _buttonsPooling ) {
		killTimer( _buttonsTimer );
		_buttonsPooling = 0;
	}
}

void DevManager::timerEvent( QTimerEvent *e ) {
	int _timer = e->timerId();
	if( _baseTimeTimerPooling && (_timer == _baseTimeTimer) ) {
		if( srv_file ) fprintf( srv_file,"_timer=%d,_baseTimeTimer=%d\n",
						_timer,_baseTimeTimer );
		timeout_action();
		return;
	}
	if( _overLightPooling && (_timer == _hvTimer) ) {
		testOverLight(1);
		return;
	}
	if( _buttonsPooling && (_timer == _buttonsTimer) ) {
		processButtons(1);
		return;
	}
	if( _temperPooling && (_timer == _temperTimer) ) {
		getTemper(1);
		return;
	}
}

int DevManager::testOverLight( int _dialog ) {
//	debug( "testOverLight" );
	int ret = 0;
	int _oldPooling;
	if( !isPowerExist() ) return(0);
	if( !_pooling ) _testOverLight_flag = 1;
	else {
		if( srv_file ) fprintf( srv_file,"TestOverLight! _pooling=%d",_pooling );
		_oldPooling = _overLightPooling;
		_overLightPooling = 0;
		if( power.testOverLight( _dialog ) == 0 ) {

			stateExportMutex.lock();
			StateExport::state.hv = OFF;
			StateExport::isChanged = true;
			stateExportMutex.unlock();

			debug( "Overlight!!! signalHV(OFF) emitted!!!" );
			ret = 1;
		}
		_overLightPooling = _oldPooling;
	}
	return( ret );
}

void DevManager::processButtons( int _dialog ) {
	int _bParam;
	int _bMode;
	int _oldPooling;
	if( !isLightExist() ) return;
	if( !_pooling ) _processButtons_flag = 1;
	else {
		_oldPooling = _buttonsPooling;
		_buttonsPooling = 0;
		light.getParam( _bParam,_dialog );
		if( _bParam ) {
			light.getBMode( _bMode,_dialog );
//			debug( "---- Mode=%d Param=%d",
//				_bMode,_bParam );
			if( _bMode == _aperturesMode ) { changeAperture( _bParam,_dialog ); }
			if( _bMode == _illumMode ) { changeBrightness( _bParam ); }
			if( _bMode == _shiftMode ) { fineShift( _bParam ); }
			if( _bMode == _doorMode ) { 
				if( _bParam == 0xFF ) _bParam = 0;
				turnDoor( _bParam,_dialog );
			}
		}
		_buttonsPooling = _oldPooling;
	}
	return;
}

void DevManager::getTemper( int _dialog ) {
	double _temper;
	int _oldPooling;

	if( !isPowerExist() ) return;
	if( !_pooling ) _getTemper_flag = 1;
	else {
		_oldPooling = _temperPooling;
		_temperPooling = 0;
		power.getTemper( _temper,_dialog );
//		debug( "Temperature = %.1f",_temper );
		if( _temper != _temperature ) {
			_temperature = _temper;
			fprintf( out_file, "%s Temperature: %.1f\n",
				getStamp( "# ",getCurrentUT() ),
				_temperature );

			stateExportMutex.lock();
			StateExport::state.temperature = _temperature;
			StateExport::isChanged = true;
			stateExportMutex.unlock();

		}
		_temperPooling = _oldPooling;
	}
}

int DevManager::setInduceRegime( int dialog ) {
	if( !slaveCounter3 || !slaveCounter2
		|| !slaveCounter1 || !masterCounter  ) return(0);

	if( !slaveCounter3->setMasterOff( dialog ) && !dialog ) return(0);
	if( !slaveCounter3->setActiveOff( dialog ) && !dialog ) return(0);
	if( (slaveCounter3->setInduc( slaveCounter2->getModule(),
		dialog ) != 1) && !dialog ) return(0);
	if( !slaveCounter3->setInduceOn( dialog ) && !dialog ) return(0);

	if( !slaveCounter2->setMasterOff( dialog ) && !dialog ) return(0);
	if( !slaveCounter2->setActiveOff( dialog ) && !dialog ) return(0);
	if( (slaveCounter2->setInduc( slaveCounter1->getModule(),
		dialog) != 1) && !dialog ) return(0);
	if( !slaveCounter2->setInduceOn( dialog ) && !dialog ) return(0);

	if( !slaveCounter1->setMasterOff( dialog ) && !dialog ) return(0);
	if( !slaveCounter1->setActiveOff( dialog ) && !dialog ) return(0);
	if( (slaveCounter1->setInduc( masterCounter->getModule(),
		dialog) != 1) && !dialog ) return(0);
	if( !slaveCounter1->setInduceOn( dialog ) && !dialog ) return(0);

	if( !masterCounter->setInduceOff( dialog ) && !dialog ) return(0);
	if( !masterCounter->setMasterOn( dialog ) && !dialog ) return(0);
	if( !masterCounter->setActiveOn( dialog ) && !dialog ) return(0);

	return(1);
}


void DevManager::resetTestData( void ) {
	_total_test_err1 = 0;
	_total_test_err2 = 0;
	_total_test_err3 = 0;
	_total_test_err4 = 0;
//	_total_fifo_err = 0;
//	_total_size_err = 0;
//	_total_number_err = 0;
}

void DevManager::exchangeTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = EXCHANGE_TEST_MODE;

	startCounting();

	debug( "startExchangeTest: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "startExchangeTest: unlock lockcount" );
}

void DevManager::detectorTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_TEST_MODE;

	startCounting();

	debug( "startDetectorTest: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "startDetectorTest: unlock lockcount" );
}

void DevManager::statisticTest( void ) {
	_old_count_mode = _count_mode;
	_count_mode = STATISTIC_TEST_MODE;

	startCounting();

	debug( "startStatisticTest: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "startStatisticTest: unlock lockcount" );
}

void DevManager::initialTests( void ) {
	_old_count_mode = _count_mode;
	_count_mode = INITIAL_TESTS_MODE;

	_stopScenario = 0;

	startCounting();

	debug( "startInitialTests: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "startInitialTests: unlock lockcount" );
}

char DevManager::decodeMode( char code ) {
	char mode = -1;
	switch( code ) {
		case 'N'	: mode = NORMAL_MODE;  break;
		case 'G'	: mode = GENERALIZED_MODE;  break;
		case 'F'	: mode = FOCUS_CONTROL_MODE;  break;
		case 'C'	: mode = STAR_CENTERING_MODE;  break;
		case 'B'	: mode = BACKGROUND_MEASUR_MODE;  break;
		case 'D'	: mode = DETECTOR_TEST_MODE;  break;
		case 'S'	: mode = STATISTIC_TEST_MODE;  break;
		case 'X'	: mode = EXCHANGE_TEST_MODE;  break;

		default		: mode = -1; break;
	}
	if( mode == -1 ) {
		str_message = "Invalid mode " + mode; 
		showFatalError( str_message );
	}
	return( mode );
}

QString& DevManager::getScenarioName( int number ) {
	if( number >= _nScenario ) {
		debug( "Invalid scenario number: %d",number );
		return( invalid );
	}
	return( (*scenarioName[number]) ); 

}

void DevManager::setCurrentScenario( int number ) {
	if( number >= _nScenario ) {
		debug( "Invalid scenario number: %d",number );
		return;
	}
	string scen_str = unroll( scenarioScript[number]->data(),
					scenarioDict );
	const char* scen = scen_str.c_str();
	debug( "Set scenario: %s: %s",
		(scenarioScript[number])->data(),scen );

	_currentScenarioNumber = number;
	int size = strlen( scen );
	currentScenario.resize( size );
	for( int i=0; i<size; i++ ) {
		char mode = decodeMode( scen[i] ); 
		if( mode < 0 ) return;
		else currentScenario[i] = mode;
	}
}

void DevManager::detectorStatisticMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_STATISTIC_MODE;

	startCounting();

	debug( "start DetectorStatisticMeasurement: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "start DetectorStatisticMeasurement: unlock lockcount" );
}

void DevManager::detectorCountingMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = DETECTOR_COUNTING_MODE;

	startCounting();

	debug( "start DetectorCountingMeasurement: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "start DetectorCountingMeasurement: unlock lockcount" );
}

void DevManager::focusControl( void ) {
	_old_count_mode = _count_mode;
	_count_mode = FOCUS_CONTROL_MODE;

	startCounting();

	debug( "start Focus Control: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "start Focus Control: unlock lockcount" );
}

void DevManager::starCentering( void ) {
	_old_count_mode = _count_mode;
	_count_mode = STAR_CENTERING_MODE;

	startCounting();

	debug( "start Star Centering: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "start Start Centering: unlock lockcount" );
}

void DevManager::normal( void ) {
	_old_count_mode = _count_mode;
	_count_mode = NORMAL_MODE;

	startCounting();

	debug( "start Normal Mode: try unlock lockcount..." );
	countMutex.unlock();
	debug( "start Normal Mode: unlocked lockcount" );
}

void DevManager::generalized( void ) {
	_old_count_mode = _count_mode;
	_count_mode = GENERALIZED_MODE;

	startCounting();

	debug( "start Generalized Mode: try unlock lockcount..." );
	countMutex.unlock();
//	pthread_mutex_unlock( &lockcount );
	debug( "start Generalized Mode: unlock lockcount" );
}

void DevManager::acceptBackgroundCorrection( matrix43 *_p_matrix ) {
	coef = _p_matrix;
	if( !coef ) return;
	int i;
	QString _str,_str_num;
//	debug( "DevManager: Accepted!" );
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		_backgroundCorrectionCoefficients[i] = (*coef)[i][2];
//		debug( "Average: %f",_backgroundCorrectionCoefficients[i] );
		_str_num.setNum(_backgroundCorrectionCoefficients[i],'f',2 );
		_str.append( _str_num );
		if( i != MAX_N_COUNTERS-1 ) _str.append( ", " ); 
	}
	tree->findBranch("Operations")
			->findBranch("Background measurement")
			->findLeaf("Correction")->replaceData( _str );
	tree->setChanged();
}

void DevManager::acceptCenterCorrection( QPoint* _p_point ) {
	delta = _p_point;
	if( !delta ) return;
	QString _str;
//	debug( "DevManager: Accepted!" );
	_centerDeltaX0 = delta->x();
	_centerDeltaY0 = delta->y();
	_str.setNum( _centerDeltaX0 );
	tree->findBranch("Operations")
			->findBranch("Star centering")
			->findLeaf("DeltaX0")->replaceData( _str );
	_str.setNum( _centerDeltaY0 );
	tree->findBranch("Operations")
			->findBranch("Star centering")
			->findLeaf("DeltaY0")->replaceData( _str );
	tree->setChanged();
}

void DevManager::getBackgroundCorrectionCoefficients( matrix43* _p_matrix )
{
	if( !_p_matrix ) return;
	int i;
	for( i=0; i<MAX_N_COUNTERS; i++ ) {
		(*_p_matrix)[i][2] = _backgroundCorrectionCoefficients[i];
	}
}

void DevManager::getCenterCorrectionCoefficients( QPoint* _p_point )
{
	if( !_p_point ) return;
	_p_point->setX( _centerDeltaX0 );
	_p_point->setY( _centerDeltaY0 );
}

void DevManager::backgroundCorrection( matrix43* _p_matrix ) {
	coef = _p_matrix;
	_old_count_mode = _count_mode;
	_count_mode = BACKGROUND_CORR_MODE;
	startCounting();
	debug( "startBackgroundCorrection: try unlock lockcount..." );
	countMutex.unlock();
//    pthread_mutex_unlock( &lockcount );
    debug( "startBackgroundCorrection: unlock lockcount" );
}

void DevManager::backgroundMeasurement( void ) {
	_old_count_mode = _count_mode;
	_count_mode = BACKGROUND_MEASUR_MODE;
	startCounting();
	debug( "startBackgroundMeasurement: try unlock lockcount..." );
	countMutex.unlock();
//    pthread_mutex_unlock( &lockcount );
    debug( "startBackgroundMeasurement: unlock lockcount" );
}

void DevManager::fluxEstimation( void ) {
	_old_count_mode = _count_mode;
	_count_mode = FLUX_ESTIMATION_MODE;
	startCounting();
	debug( "startFluxEstimation: try unlock lockcount..." );
	countMutex.unlock();
//    pthread_mutex_unlock( &lockcount );
	debug( "startFluxEstimation: unlock lockcount" );
}

void DevManager::centerCorrection( QPoint* _p_point ) {
	delta = _p_point;
	_old_count_mode = _count_mode;
	_count_mode = CENTERING_CORR_MODE;
	startCounting();
	debug( "startCenterCorrection: try unlock lockcount..." );
	countMutex.unlock();
//    pthread_mutex_unlock( &lockcount );
    debug( "startCenterCorrection: unlock lockcount" );
}

void DevManager::initTests( void ) {
//	int i;
	int changed = 0;
//	int ampl;
//	QString str;

	for( int i=0; i<_nStatisticTestLight; i++ ) {
		double ampl = _testVAmpl * 
				_statisticTestLight[i];
		if( (_statisticTestLight[i] + ampl) > MAX_LIGHT_VALUE ) {
			changed = 1;
			break;
		}
		if( (_statisticTestLight[i] - ampl) < MIN_LIGHT_VALUE ) {
			changed = 1;
			break;
		}
	}
	if( changed ) {
		str_message = "Statistic test:\nturbina.cfg/Operations/Tests/Amplitude\n"
			"is too high and will be modified";
		switch( showStrongWarning( str_message ) ) {
			case  QMessageBox::Abort : myQAppExit(1);
			case  QMessageBox::Ignore: break;
			default : break;
		}
	}
}

void DevManager::startCounting( void ) {
	debug( "------------------ Start Counting -----------" );
	userStopMutex.lock();
	isUserStop = false;
	userStopMutex.unlock();
	
	stateExportMutex.lock();
	StateExport::isEndCounting = false;
	StateExport::isStartCounting = true;
	stateExportMutex.unlock();
}

void DevManager::disablePooling( void ) {
	_pooling = 0;	// disable
}

void DevManager::enablePooling( void ) {
	_pooling = 1;	// enable

	star.updateCurrent();
	if( _testOverLight_flag ) {
		_testOverLight_flag = 0;
		if( testOverLight(0) == 1 ) {
			sendToScreen( overLightText );
		}
	}
	if( _processButtons_flag ) {
		_processButtons_flag = 0;
		processButtons(0);
	}
	if( _getTemper_flag ) {
		_getTemper_flag = 0;
		getTemper(0);
	}
}

void DevManager::endCounting( void ) {
	debug( "------------------- End Counting -------------------" );
//	emit signalEndCounting();
	stateExportMutex.lock();
	StateExport::isStartCounting = false;
	StateExport::isEndCounting = true;
	stateExportMutex.unlock();
	debug( "signal End Counting emitted!" );
}

int DevManager::testMotorStatus( int _dialog ) {
	int ret;
	if( !isMotorExist() ) return(0);
	ret = motor.testInitStatus( _dialog );
	stateExportMutex.lock();
	StateExport::state.door = motor.isDoorOpen();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
//	emit signalDoor( motor.isDoorOpen() );
//	debug( "QQQQQQQQQQQQQQQQQQQQQQQQQQQQ door: %d", motor.isDoorOpen() );
	return( ret );
}

int DevManager::testPowerStatus( int _dialog ) {
	int ret;
	if( !isPowerExist() ) return(0);
	ret = power.testOverLight( _dialog );
	stateExportMutex.lock();
	StateExport::state.hv = power.isHVOn();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
//	emit signalHV( power.isHVOn() );
	return( ret );
}

int DevManager::testButtons( void ) {
	if( !isLightExist() ) return(0);
	int ret = light.testButtons();
	stateExportMutex.lock();
	StateExport::state.buttons = isButtonsOn();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
//	emit signalButtons( isButtonsOn() );
	return( ret );
}

int DevManager::testIllum( void ) {
	if( !isLightExist() ) return(0);
	int ret = light.testIllum();
	stateExportMutex.lock();
	StateExport::state.illum = isIllumOn();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
//	emit signalIllumination( light.isIllumOn() );
	return (ret );
}

int DevManager::turnDoor( const QString& _open, int _dialog,
					bool _backApert = true ) {
	int _open_int;
	int ret = 0;
	if( !isMotorExist() ) return(0);
	int _oldAperture = currentAperture();
	ret = motor.turnDoor( _open,_dialog );
	if( ret == 1 ) {
		_open_int = motor.isDoorOpen();
		stateExportMutex.lock();
		StateExport::state.door = _open_int;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalDoor( _open_int );
	}
	if( _backApert ) setAperture( _oldAperture,_dialog ); // T.k. shtorka dvigaet apertureu
	return( ret );
}

int DevManager::turnDoor( int _open, int _dialog,
							bool _backApert = true ) {
	int ret=0;
	if( !isMotorExist() ) return(0);
	int _oldAperture = currentAperture();
	ret = motor.turnDoor( _open,_dialog );
	if( ret == 1 ) {
		stateExportMutex.lock();
		StateExport::state.door = _open;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalDoor( _open );
	}
	if( _backApert ) setAperture( _oldAperture,_dialog ); // T.k. shtorka dvigaet apertureu
	return( ret );
}

void DevManager::turnHV( int _on, int _dialog ) {
	if( !isPowerExist() ) return;
	debug( "DevManager: turnHV %d", _on );
	if( power.turnHV( _on, _dialog ) ) {
		stateExportMutex.lock();
		StateExport::state.hv = _on;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalHV( _on );
	}
}

int DevManager::synchAt( int _shift, int _dialog ) {
	if( !isMotorExist() ) return(0);
	if( _shift > MAX_SYNCH_SHIFT ) return(0);
	if( _shift < (-1)*MAX_SYNCH_SHIFT ) return(0);
	return( motor.synchAt( (char)_shift,_dialog ) );
}

int DevManager::shiftAperture( int _shift, int _dialog ) {
	if( !isMotorExist() ) return(0);
	return( motor.shiftAperture( _shift,_dialog ) );
}

int DevManager::setAperture( const QString& _apName, int _dialog,
			bool _hard=false ) {
	int _stop_number;
	int ret;
	if( !isMotorExist() ) return(0);
	if( (ret=motor.setAperture( _apName, _dialog, _hard )) == 1 ) {
		_stop_number = currentAperture();
		stateExportMutex.lock();
		StateExport::state.aperture = _stop_number;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalAperture( _stop_number );
	}
	return( ret );
}

int DevManager::setAperture( int _stop_number, int _dialog,
			bool _hard=false ) {
	int ret;
	if( !isMotorExist() ) return(0);
	if( (ret=motor.setAperture( _stop_number,_dialog,_hard )) == 1 ) {
		stateExportMutex.lock();
		StateExport::state.aperture = _stop_number;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalAperture( _stop_number );
	}
	return( ret );
}

int DevManager::changeAperture( int _shift, int _dialog ) {
	int ret;
//	debug( "change Aperture shift=%d",_shift );
	if( !isMotorExist() ) return(0);
	int _aperture_ = currentAperture();
	if( _shift == 1 ) {
		_aperture_++;
		if( _aperture_ >= motor.getNStops() ) _aperture_= 0;
	} else if( _shift == 0xFF ) {
		_aperture_--;
		if( _aperture_ < 0 ) _aperture_= motor.getNStops()-1;
	} else return(0);
//	debug( "change Aperture: new aperture = %d", _aperture_ );
	if( (ret=motor.setAperture( _aperture_,_dialog )) == 1 ) {
		stateExportMutex.lock();
		StateExport::state.aperture = currentAperture();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalAperture( currentAperture() );
//		debug( "set aperture ret=%d",ret );
	}
	return( ret );
}

void DevManager::turnIllum( const QString& _on, int _dialog ) {
	int _on_int;
	if( !isLightExist() ) return;
	if( light.turnIllum( _on,_dialog ) ) {
		_on_int = light.isIllumOn();
		stateExportMutex.lock();
		StateExport::state.illum = _on_int;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalIllumination( _on_int );
	}
}

void DevManager::turnIllum( int _on, int _dialog ) {
	if( !isLightExist() ) return;
	if( light.turnIllum( _on, _dialog ) ) {
		stateExportMutex.lock();
		StateExport::state.illum = _on;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalIllumination( _on );
	}
}

void DevManager::turnIllumOn( void ) {
	turnIllum( 1,1 );
}

void DevManager::turnIllumOff( void ) {
	turnIllum( 0,1 );
}

void DevManager::turnButtons( const QString& _on, int _dialog ) {
	int _on_int;
	if( !isLightExist() ) 	return;
	if( light.turnButtons( _on,_dialog ) ) {
		_on_int = isButtonsOn();
		if( _on_int == 1 ) startButtonsPooling();
		else stopButtonsPooling();
		stateExportMutex.lock();
		StateExport::state.buttons = _on_int;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalButtons( _on_int );
	}
}

void DevManager::turnButtons( int _on, int _dialog ) {
	if( !isLightExist() ) return;
	if( light.turnButtons( _on,_dialog ) ) {
		_on = isButtonsOn();
		if( _on == 1 ) startButtonsPooling();
		else stopButtonsPooling();
		stateExportMutex.lock();
		StateExport::state.buttons = _on;
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalButtons( _on );
	}
}

void DevManager::turnSafety( const QString& _on, int _dialog ) {
	int _on_int;
	if( !isPowerExist() ) 	return;
	if( power.turnSafety( _on,_dialog ) ) {
		_on_int = isSafetyOn();
		if( _on_int == ON ) startOverLightPooling();
		else stopOverLightPooling();
	}
}

void DevManager::turnSafety( int _on, int _dialog ) {
	if( !isPowerExist() ) return;
	if( power.turnSafety( _on,_dialog ) ) {
		_on = isSafetyOn();
		if( _on == ON ) startOverLightPooling();
		else stopOverLightPooling();
	}
}

void DevManager::nullSet( void ) {
	if( !isMotorExist() ) return;
	int _oldAperture = currentAperture();
	int _oldDoor = motor.isDoorOpen();

	motor.nullSet(1);

	turnDoor( _oldDoor,1 );
	setAperture( _oldAperture,1 ); // T.k. shtorka dvigaet apertureu
	stateExportMutex.lock();
	StateExport::state.door = motor.isDoorOpen();
	StateExport::isChanged = true;
	stateExportMutex.unlock();
//	emit signalDoor( motor.isDoorOpen() );
}

void DevManager::setBrightness( int _brightness,int _dialog ) {
	if( isLightExist() ) {
		light.setBrightness( _brightness,_dialog );
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalBrightness( getCurrentBrightness() );
	}
}

void DevManager::changeBrightness( int _up ) {
//	debug( "changeBrightness _up=%d", _up );
	if( _up == 1 ) { setBrightnessUp(); return; }
	if( _up == 0xFF ) { setBrightnessDown(); return; }
}

void DevManager::setBrightnessUp( void ) {
//	debug( "brightness up" );
	if( isLightExist() ) {
		light.setBrightnessUp( 1 );
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalBrightness( getCurrentBrightness() );
	}
}

void DevManager::setBrightnessDown( void ) {
//	debug( "brightness down" );
	if( isLightExist() ) {
		light.setBrightnessDown( 1 );
		stateExportMutex.lock();
		StateExport::state.brightness = getCurrentBrightness();
		StateExport::isChanged = true;
		stateExportMutex.unlock();
//		emit signalBrightness( getCurrentBrightness() );
	}
}

int DevManager::fineShift( int _direct ) {
//	debug( "DevManager::fineShift %d",_direct );
	if( _direct == 1 ) return( movePlus() );
	if( _direct == 0xFF ) return( moveMinus() );
	str_message = "Invalid shift direction!";
	switch( showStrongWarning( str_message ) ) {
		case  QMessageBox::Abort : myQAppExit(1);
		case  QMessageBox::Ignore: return(0);
		default : return(0);
	}
}

int DevManager::movePlus( void ) {
	if( isMotorExist() ) return( motor.movePlus( 1 ) );
	return(0);
}

int DevManager::moveMinus( void ) {
	if( isMotorExist() ) return( motor.moveMinus( 1 ) );
	return(0);
}

QString DevManager::getStopName( int _stopNumber ) {
	return( motor.getStopName(_stopNumber) );
}

int DevManager::isHVOn( void ) {
//	debug( "DevManager: isPowerExist: %d",isPowerExist() );
	if( !isPowerExist() ) return(-1);
	return( power.isHVOn() );
}

void DevManager::startAdjustAperture( void ) {
	motor.startCorrection();
}

void DevManager::acceptAdjustAperture( int _dialog ) {
	motor.acceptCorrection( _dialog );
}

void DevManager::park( void ) {
	debug( "Parking..." );

//	emit signalShowMessage( "Parking...",-1 );
	if( _inputDoorPark == OFF ) {
		if( motor.isReady() ) {
			if( motor.isDoorOpen() ) turnDoor( OFF,0,false );
			else motor.moveTo( POSITION_CLOSE,0 );
		}
	}
// lll
//	turnDoor( _inputDoorPark,0,false );
	turnHV( _hvPark,0 );
	if( isLightExist() ) light.reset(); // Perevodim buttons v avtonom.rezhim
	turnIllum( _illumPark,0 );
	if( isLightExist() ) light.turnLight( _lightPark,0 );
	killTimers();
	data_cycle_condition = 0;
	dataMutex.unlock();

	_count_mode = END_OF_PROGRAM;
	count_cycle_condition = 0;
	countMutex.unlock();

	scenarioMutex.unlock();
}
